Package: BRCore
Title: A Unified Framework for Identification and Ecological
        Interpretation of Microbial Data from Bioenergy Research
        Centers
Version: 2.0.7
Authors@R: c(
    person("Bolívar", "Aponte Rolón",
           email = "bolaponte@pm.me", 
           role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-2544-4551")),
    person("Gian Maria Niccolò", "Benucci",
           email = "benucci@msu.edu",
           role = "aut", 
           comment = c(ORCID = "0000-0003-1589-947X", "Co-maintainer")),
    person("Brandon", "Kristy", 
           role = "aut", 
           comment = c(ORCID = "0000-0003-2421-7143")),
    person("Ashley", "Shade", 
           role = "aut", 
           comment = c(ORCID = "0000-0002-7189-3067")),
    person("Nejc", "Stopnisek", 
           role = "aut", 
           comment = c(ORCID = "0000-0002-1532-6826")),
    person("Adina", "Howe", 
           role = "aut", 
           comment = c(ORCID = "0000-0002-7705-343X")),
    person("Center for Advanced Bioenergy and Bioproducts Innovation", 
           role = c("cph", "fnd")),
    person("Great Lakes Bioenergy Research Center", 
           role = c("cph", "fnd"))
  )
Description: A unified framework for identification and ecological interpretation of core microbiomes across time and space, enhancing robustness and reproducibility in microbiome data analysis. 'BRCore' implements the workflow proposed by Shade and Stopnisek (2019) and incorporates additional rarefaction steps. The proposed workflow aims to identify persistent microbiomes using abundance-occupancy distributions and neutral community model fitting. For more details on abundance-occupancy distributions see Shade A, Stopnisek N (2019) <doi:10.1016/j.mib.2019.09.008>, for neutral models, see Sloan et al. (2006) <doi:10.1111/j.1462-2920.2005.00956.x> and Burns et al. (2015) <doi:10.1038/ismej.2015.142>.
Depends: R (>= 4.4)
biocViews:
Imports: cli (>= 3.6.5), dplyr (>= 1.1.4), ggplot2 (>= 4.0.1), ggpubr
        (>= 0.6.2), ggrepel (>= 0.9.6), Hmisc (>= 5.2.4), magrittr (>=
        2.0.4), minpack.lm (>= 1.2.4), phyloseq, rlang (>= 1.1.6),
        scales (>= 1.4.0), stats4, tibble (>= 3.3.0), tidyr (>= 1.3.1),
        vegan (>= 2.7.2)
Suggests: knitr (>= 1.50), rmarkdown (>= 2.30), testthat (>= 3.2.3),
        tidyverse (>= 2.0.0), viridis (>= 0.6.5)
License: MIT + file LICENSE
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
Config/testthat/edition: 3
LazyData: true
VignetteBuilder: knitr
URL: https://github.com/germs-lab/BRCore,
        https://www.germslab.org/BRCore/
BugReports: https://github.com/germs-lab/BRCore/issues
NeedsCompilation: no
Packaged: 2026-05-19 23:26:14 UTC; baponte
Author: Bolívar Aponte Rolón [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-2544-4551>),
  Gian Maria Niccolò Benucci [aut] (ORCID:
    <https://orcid.org/0000-0003-1589-947X>, Co-maintainer),
  Brandon Kristy [aut] (ORCID: <https://orcid.org/0000-0003-2421-7143>),
  Ashley Shade [aut] (ORCID: <https://orcid.org/0000-0002-7189-3067>),
  Nejc Stopnisek [aut] (ORCID: <https://orcid.org/0000-0002-1532-6826>),
  Adina Howe [aut] (ORCID: <https://orcid.org/0000-0002-7705-343X>),
  Center for Advanced Bioenergy and Bioproducts Innovation [cph, fnd],
  Great Lakes Bioenergy Research Center [cph, fnd]
Maintainer: Bolívar Aponte Rolón <bolaponte@pm.me>
Repository: CRAN
Date/Publication: 2026-05-20 06:50:08 UTC
Built: R 4.5.2; ; 2026-05-21 23:39:49 UTC; unix

Package: BRCore
Title: A Unified Framework for Identification and Ecological
        Interpretation of Microbial Data from Bioenergy Research
        Centers
Version: 2.0.7
Authors@R: c(
    person("Bolívar", "Aponte Rolón",
           email = "bolaponte@pm.me", 
           role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-2544-4551")),
    person("Gian Maria Niccolò", "Benucci",
           email = "benucci@msu.edu",
           role = "aut", 
           comment = c(ORCID = "0000-0003-1589-947X", "Co-maintainer")),
    person("Brandon", "Kristy", 
           role = "aut", 
           comment = c(ORCID = "0000-0003-2421-7143")),
    person("Ashley", "Shade", 
           role = "aut", 
           comment = c(ORCID = "0000-0002-7189-3067")),
    person("Nejc", "Stopnisek", 
           role = "aut", 
           comment = c(ORCID = "0000-0002-1532-6826")),
    person("Adina", "Howe", 
           role = "aut", 
           comment = c(ORCID = "0000-0002-7705-343X")),
    person("Center for Advanced Bioenergy and Bioproducts Innovation", 
           role = c("cph", "fnd")),
    person("Great Lakes Bioenergy Research Center", 
           role = c("cph", "fnd"))
  )
Description: A unified framework for identification and ecological interpretation of core microbiomes across time and space, enhancing robustness and reproducibility in microbiome data analysis. 'BRCore' implements the workflow proposed by Shade and Stopnisek (2019) and incorporates additional rarefaction steps. The proposed workflow aims to identify persistent microbiomes using abundance-occupancy distributions and neutral community model fitting. For more details on abundance-occupancy distributions see Shade A, Stopnisek N (2019) <doi:10.1016/j.mib.2019.09.008>, for neutral models, see Sloan et al. (2006) <doi:10.1111/j.1462-2920.2005.00956.x> and Burns et al. (2015) <doi:10.1038/ismej.2015.142>.
Depends: R (>= 4.4)
biocViews:
Imports: cli (>= 3.6.5), dplyr (>= 1.1.4), ggplot2 (>= 4.0.1), ggpubr
        (>= 0.6.2), ggrepel (>= 0.9.6), Hmisc (>= 5.2.4), magrittr (>=
        2.0.4), minpack.lm (>= 1.2.4), phyloseq, rlang (>= 1.1.6),
        scales (>= 1.4.0), stats4, tibble (>= 3.3.0), tidyr (>= 1.3.1),
        vegan (>= 2.7.2)
Suggests: knitr (>= 1.50), rmarkdown (>= 2.30), testthat (>= 3.2.3),
        tidyverse (>= 2.0.0), viridis (>= 0.6.5)
License: MIT + file LICENSE
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
Config/testthat/edition: 3
LazyData: true
VignetteBuilder: knitr
URL: https://github.com/germs-lab/BRCore,
        https://www.germslab.org/BRCore/
BugReports: https://github.com/germs-lab/BRCore/issues
NeedsCompilation: no
Packaged: 2026-05-19 23:26:14 UTC; baponte
Author: Bolívar Aponte Rolón [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-2544-4551>),
  Gian Maria Niccolò Benucci [aut] (ORCID:
    <https://orcid.org/0000-0003-1589-947X>, Co-maintainer),
  Brandon Kristy [aut] (ORCID: <https://orcid.org/0000-0003-2421-7143>),
  Ashley Shade [aut] (ORCID: <https://orcid.org/0000-0002-7189-3067>),
  Nejc Stopnisek [aut] (ORCID: <https://orcid.org/0000-0002-1532-6826>),
  Adina Howe [aut] (ORCID: <https://orcid.org/0000-0002-7705-343X>),
  Center for Advanced Bioenergy and Bioproducts Innovation [cph, fnd],
  Great Lakes Bioenergy Research Center [cph, fnd]
Maintainer: Bolívar Aponte Rolón <bolaponte@pm.me>
Repository: CRAN
Date/Publication: 2026-05-20 06:50:08 UTC
Built: R 4.6.0; ; 2026-05-20 08:34:03 UTC; unix

# Generated by roxygen2: do not edit by hand

export("%>%")
export(add_rarefaction_metrics)
export(fit_neutral_model)
export(identify_core)
export(multi_rarefy)
export(plot_abundance_occupancy)
export(plot_core_distribution)
export(plot_identified_core)
export(plot_neutral_model)
export(plot_rarefaction_metrics)
export(plot_variance_propagation)
export(sncm.fit)
export(update_otu_table)
importFrom(Hmisc,binconf)
importFrom(cli,cli_abort)
importFrom(cli,cli_alert_danger)
importFrom(cli,cli_alert_info)
importFrom(cli,cli_alert_success)
importFrom(cli,cli_alert_warning)
importFrom(cli,cli_h1)
importFrom(cli,cli_h2)
importFrom(cli,cli_inform)
importFrom(cli,cli_progress_step)
importFrom(cli,cli_text)
importFrom(cli,cli_warn)
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_rows)
importFrom(dplyr,desc)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,last)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,near)
importFrom(dplyr,select)
importFrom(dplyr,slice_head)
importFrom(dplyr,summarise)
importFrom(dplyr,summarize)
importFrom(dplyr,transmute)
importFrom(dplyr,ungroup)
importFrom(dplyr,where)
importFrom(ggplot2,aes)
importFrom(ggplot2,after_scale)
importFrom(ggplot2,alpha)
importFrom(ggplot2,annotate)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,expansion)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_boxplot)
importFrom(ggplot2,geom_histogram)
importFrom(ggplot2,geom_jitter)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_x_discrete)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,scale_y_log10)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_classic)
importFrom(ggpubr,ggarrange)
importFrom(ggrepel,geom_text_repel)
importFrom(grid,unit)
importFrom(magrittr,"%>%")
importFrom(minpack.lm,nlsLM)
importFrom(phyloseq,"sample_data<-")
importFrom(phyloseq,otu_table)
importFrom(phyloseq,phy_tree)
importFrom(phyloseq,phyloseq)
importFrom(phyloseq,prune_samples)
importFrom(phyloseq,prune_taxa)
importFrom(phyloseq,refseq)
importFrom(phyloseq,sample_data)
importFrom(phyloseq,sample_names)
importFrom(phyloseq,sample_sums)
importFrom(phyloseq,tax_table)
importFrom(phyloseq,taxa_are_rows)
importFrom(phyloseq,taxa_sums)
importFrom(rlang,.data)
importFrom(rlang,as_name)
importFrom(rlang,ensym)
importFrom(scales,comma)
importFrom(stats,AIC)
importFrom(stats,BIC)
importFrom(stats,IQR)
importFrom(stats,coef)
importFrom(stats,confint)
importFrom(stats,dnorm)
importFrom(stats,pbeta)
importFrom(stats,pbinom)
importFrom(stats,ppois)
importFrom(stats,quantile)
importFrom(stats4,mle)
importFrom(tibble,as_tibble)
importFrom(tibble,column_to_rownames)
importFrom(tibble,rownames_to_column)
importFrom(tidyr,pivot_longer)
importFrom(utils,combn)
importFrom(utils,head)
importFrom(utils,tail)
importFrom(vegan,decostand)
importFrom(vegan,rrarefy)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# BRCore

<!-- badges: start -->

[![Lifecycle:
experimental](https://img.shields.io/badge/lifecycle-experimental-orange.svg)](https://lifecycle.r-lib.org/articles/stages.html#experimental)
[![](https://www.r-pkg.org/badges/version-last-release/BRCore)](https://cran.r-project.org/package=BRCore)
[![](https://cranlogs.r-pkg.org/badges/grand-total/BRCore)](https://www.r-pkg.org/pkg/BRCore)
[![Codecov test
coverage](https://codecov.io/gh/germs-lab/BRCore/graph/badge.svg)](https://app.codecov.io/gh/germs-lab/BRCore)
[![R-CMD-check](https://github.com/germs-lab/BRCore/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/germs-lab/BRCore/actions/workflows/R-CMD-check.yaml)

<!-- badges: end -->

## Overview

BRCore provides a unified framework for identification and ecological
interpretation of core microbiomes across time and space, enhancing
robustness and reproducibility in microbiome data analysis.

BRCore provides tools for:

**Rarefaction analysis**: Calculate pre-rarefaction metrics and perform
rarefactions

- `add_rarefaction_metrics()`
- `multi_rarefy()`
- `plot_variance_propagation()`

**Core microbiome identification**: Identify core microbial taxa using
abundance-occupancy distributions

- `identify_core()`

**Neutral model fitting**: Fit and visualize neutral community models

- `fit_neutral_model()`
- `plot_neutral_model()`
- `sncm.fit()`

**Visualization**: Plot rarefaction diagnostics, abundance-occupancy
curves, and core distributions

- `plot_rarefaction_metrics()`
- `plot_abundance_occupancy()`
- `plot_core_distribution()`
- `plot_identified_core()`

## Installation

### *Stable* version from CRAN with:

``` r
install.packages("BRCore")

# or if using with `renv`:
renv::init(bioconductor = TRUE)
renv::install("BRCore")
```

***NOTE:*** *If using `install.packages()` in an `renv` project, `renv`
will intercept the call. Run `renv::init(bioconductor = TRUE)` first to
ensure Bioconductor dependencies can be resolved and installed.*

### *Development* version from GitHub with:

``` r
# install.packages("pak")
pak::pak("germs-lab/BRCore")
```

***Note:*** *`pak` handles dependencies automatically.*

## Quick Start

``` r
library(BRCore)
library(phyloseq)

# Add rarefaction metrics
bcse_metrics <- add_rarefaction_metrics(data = bcse)

# Perform rarefaction
bcse_rarefied_list <- multi_rarefy(
  physeq_obj = bcse,
  depth_level = 1000,
  num_iter = 3,
  set_seed = 7642
)

# Update phyloseq object with rarefied data
bcse_rare_single <- update_otu_table(
  physeq_obj = bcse,
  rarefied_otus = bcse_rarefied_list,
  iteration = 2
) # Your preffered iteration can be used here

# Identify core microbiome

# With a single iteration of rarefaction
bcse_core <- identify_core(
  physeq_obj = bcse_rare_single,
  priority_var = "Crop",
  increase_value = 0.02,
  seed = 2134
)


# With multiple iterations of rarefaction
bcse_core_multi <- identify_core(
  physeq_obj = bcse,
  rarefied_list = bcse_rarefied_list,
  priority_var = "Crop",
  increase_value = 0.02,
  depth_level = 1000,
  seed = 2134
)

# Visualize abundance-occupancy distribution
plot_abundance_occupancy(core_result = bcse_core, core_set = "increase")

# Fit and plot neutral model
bcse_neutral <- fit_neutral_model(
  otu_table = bcse_core$otu_table,
  core_set = bcse_core$increase_core,
  abundance_occupancy = bcse_core$abundance_occupancy
)

plot_neutral_model(bcse_neutral)
```

## Documentation

For detailed examples and use cases, see the package vignette:

``` r
vignette("BRCore-vignette", package = "BRCore")
```

## Contributing

Contributions to BRCore are welcome! Please see the
[CONTRIBUTING.md](https://github.com/germs-lab/BRCore/blob/main/.github/CONTRIBUTING.md)
file for guidelines on how to contribute.

## Code of Conduct

Please note that this project is released with a [Contributor Code of
Conduct](https://github.com/germs-lab/BRCore/blob/main/.github/CODE_OF_CONDUCT.md).
By participating in this project you agree to abide by its terms.

# Test Sets Directory

This directory contains `test_vignette_data.rda` produced on 2026-02-05 by BAR. 

## Purpose

This file serves as the reference dataset for validating the complete BRCore workflow implementation. It is used when executing `tests/testthat/test-vignette-workflow.R`, which acts as the litmus test for the package's core analysis pipeline.

## Contents

`test_vignette_data.rda` is a list that contains saved outputs from a complete run of the BRCore vignette workflow, including:

- Rarefied phyloseq objects
- Core microbiome identification results (elbow and % increase methods)
- Abundance-occupancy data
- Neutral model fits
- All intermediate data structures and plots

## Usage

The test suite in `test-vignette-workflow.R` compares fresh workflow executions against these reference data to ensure:

- Reproducibility of results (with fixed seeds)
- Consistency of core identification algorithms
- Stability of plotting functions
- Integrity of the complete analysis pipeline

## Regenerating Reference Data

After making changes to `multi_rarefy()` or other functions that affect the vignette workflow outputs, the reference data must be regenerated to match the new implementation.

### Using the regeneration script

```bash
cd BRCore
Rscript tests/test_sets/regenerate_test_data.R
```

### Manual regeneration

Alternatively, run the complete vignette workflow:

```r
# Run the complete vignette workflow
knitr::purl("vignettes/BRCore-vignette.Rmd.orig", output = "vignettes/BRCore-vignette.R")
source("vignettes/BRCore-vignette.R")

# Save all intermediate objects
test_vignette_data <- list(
  test_bcse_metrics = bcse_metrics,
  test_rarefaction_plot = rarefaction_plot,
  test_bcse_rarefied_otutable = bcse_rarefied_list,
  test_bcse_rare = bcse_rare,
  test_bcse_rare_core = bcse_rare_core,
  test_bcse_identified_core = bcse_identified_core,
  test_plot_abund_occ_increase = plot_abund_occ_increase,
  test_plot_abund_occ_elbow = plot_abund_occ_elbow,
  test_plot_core_dist_bar = plot_core_dist_bar,
  test_plot_core_dist_line = plot_core_dist_line,
  test_plot_core_dist_heatmap = plot_core_dist_heatmap,
  test_bcse_rare_core_neutral_fit = bcse_rare_core_neutral_fit,
  test_plot_bcse_neutral_fit = plot_bcse_neutral_fit
)

save(test_vignette_data,
  file = "tests/test_sets/test_vignette_data.rda")

#' Fit Sloan Neutral Community Model (SNCM)
#'
#' The function implements the Sloan Neutral Community Model which
#'  predicts the occurrence frequency of taxa based on their relative abundance
#' in the source pool and a migration parameter (\code{m}).  It compares the
#' neutral model against binomial and Poisson null models using various
#' statistical measures.
#'
#' @param spp A matrix or data frame where rows represent communities (samples)
#' and columns represent species/taxa. Values should be abundances or counts.
#' @param pool Optional. A matrix or data frame representing the source pool for
#'   calculating relative abundances. If NULL, the source pool is calculated
#'   from the spp matrix. Default is \code{NULL}.
#' @param stats Logical. If \code{TRUE}, returns fit statistics including AIC, BIC,
#'  R-squared, and RMSE for model comparison. If \code{FALSE}, returns predicted vs.
#'  observed frequencies with confidence intervals. Default is \code{TRUE}.
#' @param taxon Optional. A data frame containing taxonomic information to
#'  merge with results when \code{stats = FALSE}. Should have row names matching species
#'  names. Default is \code{NULL}.
#'
#' @return If \code{stats = TRUE}, returns a data frame with model fit statistics including:
#'   \itemize{
#'     \item \code{m}: Migration rate parameter from NLS fit
#'     \item \code{m.ci}: Confidence interval for m parameter
#'     \item \code{m.mle}: Migration rate from maximum likelihood estimation
#'     \item \code{maxLL}, \code{binoLL}, \code{poisLL}: Log-likelihood values for SNCM, binomial, and Poisson models
#'     \item \code{Rsqr}, \code{Rsqr.bino}, \code{Rsqr.pois}: R-squared values for each model
#'     \item \code{RMSE}, \code{RMSE.bino}, \code{RMSE.pois}: Root mean squared error for each model
#'     \item \code{AIC}, \code{BIC}: Information criteria for model selection
#'     \item \code{N}: Average number of individuals per community
#'     \item \code{Samples}: Number of samples/communities
#'     \item \code{Richness}: Number of taxa analyzed
#'     \item \code{Detect}: Detection limit (1/N)
#'   }
#'   If \code{stats = FALSE}, returns a data frame with observed and predicted
#'   frequencies along with confidence intervals for visualization.
#'
#' @details The model assumes neutral processes govern community assembly. Three models are compared: SNCM (beta distribution), binomial null model, and Poisson null model.
#'
#' @references
#' Sloan WT, Lunn M, Woodcock S, Head IM, Nee S, Curtis TP. (2006) Quantifying
#' the roles of immigration and chance in shaping prokaryote community
#' structure. Environ Microbiol. 8(4):732-40.
#' <doi:10.1111/j.1462-2920.2005.00956.x>
#'
#' @examples
#' # Generate community data
#' set.seed(42)
#' n_samples <- 100
#' n_species <- 80
#'
#' # Log-normal abundances
#' mean_abund <- rlnorm(n_species, meanlog = 2, sdlog = 1.5)
#' # Simulate community matrix with multinomial sampling
#' spp_data <- t(sapply(seq_len(n_samples), function(i) {
#'   rmultinom(
#'     1,
#'     size = sample(500:2000, 1),
#'     prob = mean_abund / sum(mean_abund)
#'   )[, 1]
#'}))
#' colnames(spp_data) <- paste0("Species_", seq_len(n_species))
#'
#' fit_stats <- sncm.fit(spp_data, stats = TRUE)
#' predictions <- sncm.fit(spp_data, stats = FALSE)
#'
#'
#' @seealso [plot_neutral_model()], [fit_neutral_model()]
#'
#' @importFrom minpack.lm nlsLM
#' @importFrom Hmisc binconf
#' @importFrom stats4 mle
#'
#' @export
sncm.fit <- function(spp, pool = NULL, stats = TRUE, taxon = NULL) {
  # Calculate the number of individuals per community
  N <- base::mean(base::apply(spp, 1, sum))
  N.int <- base::round(N)

  # Calculate the average relative abundance of each taxa across communities
  if (base::is.null(pool)) {
    p.m <- base::apply(spp, 2, mean)
    p.m <- p.m[p.m != 0]
    p <- p.m / N
  } else {
    p.m <- base::apply(pool, 2, mean)
    p.m <- p.m[p.m != 0]
    p <- p.m / N
  }

  # Calculate the occurrence frequency of each taxa across communities
  spp.bi <- 1 * (spp > 0)
  freq <- base::apply(spp.bi, 2, mean)
  freq <- freq[freq != 0]

  # Combine
  C <- base::merge(p, freq, by = 0)
  C <- C[base::order(C[, 2]), ]
  C <- base::as.data.frame(C)
  C.0 <- C[!(base::apply(C, 1, function(y) base::any(y == 0))), ] # Removes rows with any zero (absent in either source pool or local communities)
  p <- C.0[, 2]
  freq <- C.0[, 3]
  base::names(p) <- C.0[, 1]
  base::names(freq) <- C.0[, 1]

  # Calculate the limit of detection
  d <- 1 / N

  ## Fit model parameter m (or Nm) using Non-linear least squares (NLS)
  warned_nls_pbeta <- FALSE
  m.fit <- minpack.lm::nlsLM(
    freq ~ .internal_warn_sncm(
      \() stats::pbeta(d, N * m * p, N * m * (1 - p), lower.tail = FALSE),
      message = "NLS `stats::pbeta()` produced a warning during fitting",
      warned = warned_nls_pbeta,
      warn_setter = function() warned_nls_pbeta <<- TRUE
    ),
    start = list(m = 0.1)
  )
  m.ci <- stats::confint(m.fit, "m", level = 0.95)

  ## Fit neutral model parameter m (or Nm) using Maximum likelihood estimation (MLE)
  sncm.LL <- local({
    warned_pbeta <- FALSE
    warned_dnorm <- FALSE
    function(m, sigma) {
      R <- freq -
        .internal_warn_sncm(
          \() stats::pbeta(d, N * m * p, N * m * (1 - p), lower.tail = FALSE),
          message = "SNCM `stats::pbeta()` produced a warning during MLE",
          warned = warned_pbeta,
          warn_setter = function() warned_pbeta <<- TRUE
        )
      R <- .internal_warn_sncm(
        \() stats::dnorm(R, 0, sigma),
        message = "SNCM `stats::dnorm()` produced a warning during MLE",
        warned = warned_dnorm,
        warn_setter = function() warned_dnorm <<- TRUE
      )
      -base::sum(base::log(R))
    }
  })
  m.mle <- stats4::mle(
    sncm.LL,
    start = list(m = 0.1, sigma = 0.1),
    nobs = base::length(p)
  )

  ## Calculate Akaike's Information Criterion (AIC)
  aic.fit <- stats::AIC(m.mle, k = 2)
  bic.fit <- stats::BIC(m.mle)

  ## Calculate goodness-of-fit (R-squared and Root Mean Squared Error)
  warned_goodns_pbeta <- FALSE
  freq.pred <- .internal_warn_sncm(
    \() {
      stats::pbeta(
        d,
        N * stats::coef(m.fit) * p,
        N * stats::coef(m.fit) * (1 - p),
        lower.tail = FALSE
      )
    },
    message = "SNCM `stats::pbeta()` produced a warning during goodness-of-fit calculation",
    warned = warned_goodns_pbeta,
    warn_setter = function() warned_goodns_pbeta <<- TRUE
  )

  Rsqr <- 1 -
    (base::sum((freq - freq.pred)^2)) /
      (base::sum((freq - base::mean(freq))^2))
  RMSE <- base::sqrt(
    base::sum((freq - freq.pred)^2) / (base::length(freq) - 1)
  )

  pred.ci <- Hmisc::binconf(
    freq.pred * base::nrow(spp),
    base::nrow(spp),
    alpha = 0.05,
    method = "wilson",
    return.df = TRUE
  )

  ## Calculate AIC for binomial model
  bino.LL <- local({
    warned_dnorm <- FALSE
    function(mu, sigma) {
      R <- freq - stats::pbinom(d, N.int, p, lower.tail = FALSE)
      R <- .internal_warn_sncm(
        \() stats::dnorm(R, mu, sigma),
        message = "Binomial model `stats::dnorm()` produced a warning during MLE",
        warned = warned_dnorm,
        warn_setter = function() warned_dnorm <<- TRUE
      )
      -base::sum(base::log(R))
    }
  })
  bino.mle <- stats4::mle(
    bino.LL,
    start = list(mu = 0, sigma = 0.1),
    nobs = base::length(p)
  )

  aic.bino <- stats::AIC(bino.mle, k = 2)
  bic.bino <- stats::BIC(bino.mle)

  ## Goodness of fit for binomial model
  bino.pred <- stats::pbinom(d, N.int, p, lower.tail = FALSE)
  Rsqr.bino <- 1 -
    (base::sum((freq - bino.pred)^2)) /
      (base::sum((freq - base::mean(freq))^2))
  RMSE.bino <- base::sqrt(
    base::sum((freq - bino.pred)^2) / (base::length(freq) - 1)
  )

  bino.pred.ci <- Hmisc::binconf(
    bino.pred * base::nrow(spp),
    base::nrow(spp),
    alpha = 0.05,
    method = "wilson",
    return.df = TRUE
  )

  ## Calculate AIC for Poisson model
  pois.LL <- local({
    warned_dnorm <- FALSE
    function(mu, sigma) {
      R <- freq - stats::ppois(d, N * p, lower.tail = FALSE)
      R <- .internal_warn_sncm(
        \() stats::dnorm(R, mu, sigma),
        message = "Poisson model `stats::dnorm()` produced a warning during MLE",
        warned = warned_dnorm,
        warn_setter = function() warned_dnorm <<- TRUE
      )
      -base::sum(base::log(R))
    }
  })
  pois.mle <- stats4::mle(
    pois.LL,
    start = list(mu = 0, sigma = 0.1),
    nobs = base::length(p)
  )

  aic.pois <- stats::AIC(pois.mle, k = 2)
  bic.pois <- stats::BIC(pois.mle)

  ## Goodness of fit for Poisson model
  pois.pred <- stats::ppois(d, N * p, lower.tail = FALSE)
  Rsqr.pois <- 1 -
    (base::sum((freq - pois.pred)^2)) /
      (base::sum((freq - base::mean(freq))^2))
  RMSE.pois <- base::sqrt(
    base::sum((freq - pois.pred)^2) / (base::length(freq) - 1)
  )

  pois.pred.ci <- Hmisc::binconf(
    pois.pred * base::nrow(spp),
    base::nrow(spp),
    alpha = 0.05,
    method = "wilson",
    return.df = TRUE
  )

  cli::cli_bullets(c(
    "i" = "Neutral model fitting:",
    "*" = "Average individuals per community (N): {.val {N}}",
    "*" = "Binomial model using rounded N: {.val {N.int}}",
    "*" = "Poisson model using N: {.val {N}}",
    "*" = "Maximum likelihood estimation using N: {.val {N}}, and starting parameters: mu = 0, sigma = 0.1"
  ))

  ## Results
  if (stats == TRUE) {
    fitstats <- base::data.frame(
      m = numeric(),
      m.ci = numeric(),
      m.mle = numeric(),
      maxLL = numeric(),
      binoLL = numeric(),
      poisLL = numeric(),
      Rsqr = numeric(),
      Rsqr.bino = numeric(),
      Rsqr.pois = numeric(),
      RMSE = numeric(),
      RMSE.bino = numeric(),
      RMSE.pois = numeric(),
      AIC = numeric(),
      BIC = numeric(),
      AIC.bino = numeric(),
      BIC.bino = numeric(),
      AIC.pois = numeric(),
      BIC.pois = numeric(),
      N = numeric(),
      Samples = numeric(),
      Richness = numeric(),
      Detect = numeric()
    )
    fitstats[1, ] <- c(
      stats::coef(m.fit),
      stats::coef(m.fit) - m.ci[1],
      m.mle@coef["m"],
      m.mle@details$value,
      bino.mle@details$value,
      pois.mle@details$value,
      Rsqr,
      Rsqr.bino,
      Rsqr.pois,
      RMSE,
      RMSE.bino,
      RMSE.pois,
      aic.fit,
      bic.fit,
      aic.bino,
      bic.bino,
      aic.pois,
      bic.pois,
      N,
      base::nrow(spp),
      base::length(p),
      d
    )
    return(fitstats)
  } else {
    A <- base::cbind(
      p,
      freq,
      freq.pred,
      pred.ci[, 2:3],
      bino.pred,
      bino.pred.ci[, 2:3]
    )
    A <- base::as.data.frame(A)
    base::colnames(A) <- c(
      "p",
      "freq",
      "freq.pred",
      "pred.lwr",
      "pred.upr",
      "bino.pred",
      "bino.lwr",
      "bino.upr"
    )
    if (base::is.null(taxon)) {
      B <- A[base::order(A[, 1]), ]
    } else {
      B <- base::merge(A, taxon, by = 0, all = TRUE)
      base::row.names(B) <- B[, 1]
      B <- B[, -1]
      B <- B[base::order(B[, 1]), ]
    }
    return(B)
  }
}

#' Plot pre-rarefaction diagnostics
#'
#' This function creates a 6-panel diagnostic plot showing sequencing depth,
#' Good's coverage, and outlier behavior based on a data frame or a
#' `phyloseq` object with rare stats already added via `add_rare_stats()`.
#'
#' @param data Either a `phyloseq` object or a `data.frame` that includes columns
#' `read_num`, `goods_cov`, and `outlier`.
#'
#' @return A `ggarrange` object with six plots.
#'
#' @seealso [add_rarefaction_metrics()]
#'
#' @examples
#' library(phyloseq)
#' library(BRCore)
#'
#' # Add rarefaction metrics to the phyloseq object
#' bcse_metrics <- add_rarefaction_metrics(bcse)
#'
#' # Plot the rarefaction diagnostics
#' plot_rarefaction_metrics(bcse_metrics)
#'
#' # You can also pass a data frame directly if you have
#' # pre-computed read_num, goods_cov, and outlier columns
#' sample_data_df <- data.frame(sample_data(bcse_metrics))
#' plot_rarefaction_metrics(sample_data_df)
#'
#' @importFrom ggplot2 ggplot aes geom_histogram geom_point geom_jitter
#' @importFrom ggplot2 geom_boxplot geom_bar theme_bw theme labs
#' @importFrom ggplot2 scale_y_log10 scale_x_continuous scale_y_continuous
#' @importFrom ggplot2 theme_classic element_line coord_cartesian
#' @importFrom ggrepel geom_text_repel
#' @importFrom ggpubr ggarrange
#' @importFrom tibble rownames_to_column as_tibble
#' @importFrom dplyr filter arrange
#' @importFrom scales comma
#' @importFrom cli cli_alert_info cli_progress_step cli_alert_success cli_abort
#' @importFrom phyloseq sample_data
#'
#' @export
#'
plot_rarefaction_metrics <- function(data) {
  # Extract sample data depending on class
  if (inherits(data, "phyloseq")) {
    sample_df <- data.frame(sample_data(data))
    sample_df <- rownames_to_column(sample_df, "sample_id")
  } else if (inherits(data, "data.frame")) {
    sample_df <- data
  } else {
    stop("Input must be a phyloseq object or a data.frame")
  }

  # Force to tibble/data.frame
  sample_df <- as_tibble(sample_df)

  cli::cli_alert_info("Processing {nrow(sample_df)} sample{?s}")

  # Check required columns
  required_cols <- c("read_num", "goods_cov", "outlier")
  if (!all(required_cols %in% colnames(sample_df))) {
    cli::cli_abort(
      "Input data must contain columns: {.field read_num}, {.field goods_cov}, and {.field outlier}"
    )
  }

  # Calculate first quartile of read_num
  readno_q1 <- quantile(sample_df$read_num, probs = 0.25, na.rm = TRUE)

  cli::cli_progress_step("Generating rarefaction diagnostic plots")

  # Generate plot grid
  plot_output <- ggarrange(
    # Plot a - Histogram
    ggplot(sample_df, aes(x = read_num)) +
      geom_histogram(binwidth = 5000, fill = "firebrick", color = "white") +
      scale_x_continuous(labels = scales::comma) +
      scale_y_continuous(labels = scales::comma) +
      .brcore_theme(dashed_grid = TRUE) +
      labs(title = "Histogram", x = "Sequence reads", y = "Sample counts"),

    # Plot b - Lower 25% histogram
    ggplot(sample_df, aes(x = read_num)) +
      geom_histogram(binwidth = 1000, fill = "firebrick", color = "white") +
      coord_cartesian(xlim = c(0, readno_q1)) +
      scale_x_continuous(labels = scales::comma) +
      scale_y_continuous(labels = scales::comma) +
      .brcore_theme(dashed_grid = TRUE) +
      labs(
        title = "Histogram (Lower 25%)",
        x = "Sequence reads",
        y = "Sample counts"
      ),

    # Plot c - Good's Coverage
    ggplot(sample_df, aes(x = read_num, y = goods_cov)) +
      geom_point(shape = 19, color = "firebrick", size = 1) +
      scale_x_continuous(labels = scales::comma) +
      .brcore_theme(dashed_grid = TRUE) +
      labs(
        title = "Good's Coverage",
        x = "Sequence reads",
        y = "Good's coverage %"
      ),

    # Plot d - Log10 jitter
    ggplot(sample_df, aes(x = 1, y = read_num)) +
      geom_jitter(shape = 19, color = "firebrick", width = 0.2, size = 1) +
      scale_y_log10(labels = scales::comma) +
      .brcore_theme(dashed_grid = TRUE) +
      labs(title = "Log10 jitter", x = "Data set", y = "Sequence reads"),

    # Plot e - Log10 boxplot
    ggplot(sample_df, aes(x = 1, y = read_num)) +
      geom_boxplot(color = "firebrick", fill = "firebrick", alpha = 0.3) +
      geom_text_repel(
        data = filter(sample_df, !is.na(outlier)),
        mapping = aes(x = 1, y = read_num, label = outlier),
        max.overlaps = 15,
        size = 3
      ) +
      scale_y_log10(labels = scales::comma) +
      .brcore_theme(dashed_grid = TRUE) +
      labs(title = "Log10 boxplot", x = "Data set", y = "Sequence reads"),

    # Plot f - Ranked samples
    {
      temp_df <- arrange(sample_df, read_num)
      ggplot(temp_df, aes(x = seq_len(nrow(temp_df)), y = read_num)) +
        geom_bar(stat = "identity", fill = "firebrick", color = NA) +
        scale_y_continuous(labels = scales::comma) +
        .brcore_theme(dashed_grid = TRUE) +
        labs(title = "Ranked samples", x = "Samples", y = "Sequence reads")
    },
    ncol = 3,
    nrow = 2,
    align = "hv",
    labels = c("a", "b", "c", "d", "e", "f")
  )

  # Display success message
  cli::cli_alert_success(
    "Rarefaction diagnostic plots generated successfully"
  )

  return(plot_output)
}

# tests/testthat/test-plot_rarefaction_metrics.R
test_that("plot_rarefaction_metrics returns a ggplot and builds", {
  skip_if_not_installed("ggplot2")

  rare_metrics <- data.frame(
    otu = paste0("sample", 1:10),
    read_num = c(25, 17, 6002, 6428, 8854, 8281, 6707, 7133, 7559, 8985),
    singlton_num = c(117, 137, 26, 37, 38, 40, 23, 32, 34, 55),
    goods_cov = c(
      0.719,
      0.762,
      0.988,
      0.987,
      0.826,
      0.972,
      0.909,
      0.875,
      0.885,
      0.934
    ),
    outlier = c(25, 17, NA, NA, NA, NA, NA, NA, NA, NA)
  )

  p <- plot_rarefaction_metrics(data = rare_metrics)

  expect_s3_class(p, "ggplot")
  expect_silent(ggplot2::ggplot_build(p))
})

test_that("1: Vignette test data structures are valid", {
  skip_on_cran()

  # Load reference data
  rda_path <- getOption(
    "brcore_test_data_path",
    default = test_path("test_sets/test_vignette_data.rda")
  )
  skip_if_not(
    file.exists(rda_path),
    "Test data file not available for testing in this environment: CRAN or R CMD check"
  )
  load(rda_path)

  # Test: Rarefied OTU table is correctly structured and has expected properties
  expect_true(
    is.list(test_vignette_data$test_bcse_rarefied_otutable) &&
      is.data.frame(test_vignette_data$test_bcse_rarefied_otutable[[1]])
  )

  expect_equal(999.999999999996, 1000, tolerance = 1e-6) # Sanity check for floating-point precision

  row_sums <- rowSums(test_vignette_data$test_bcse_rarefied_otutable[[1]])
  expect_true(all(near(row_sums, 1000, tol = 1e-6)))

  # Test: Core result structure is complete
  expect_named(
    test_vignette_data$test_bcse_rare_core,
    c(
      "bray_curtis_ranked",
      "otu_ranked",
      "abundance_occupancy",
      "priority_var",
      "abundance_weight",
      "elbow",
      "bc_increase",
      "increase_value",
      "elbow_core",
      "increase_core",
      "otu_table",
      "metadata",
      "taxonomy"
    )
  )

  # Test: Core sets are non-empty
  expect_true(length(test_vignette_data$test_bcse_rare_core$elbow_core) > 0)
  expect_true(length(test_vignette_data$test_bcse_rare_core$increase_core) > 0)

  # Test: Increase core is larger than elbow core
  expect_true(
    length(test_vignette_data$test_bcse_rare_core$increase_core) >=
      length(test_vignette_data$test_bcse_rare_core$elbow_core)
  )

  # Test: All plots are ggplot objects
  plot_objects <- c(
    "test_rarefaction_plot",
    "test_plot_abund_occ_increase",
    "test_plot_abund_occ_elbow",
    "test_plot_core_dist_bar",
    "test_plot_core_dist_line",
    "test_plot_core_dist_heatmap",
    "test_plot_bcse_neutral_fit"
  )

  for (plot_name in plot_objects) {
    expect_s3_class(test_vignette_data[[plot_name]], "ggplot")
  }

  expect_s3_class(
    test_vignette_data$test_bcse_identified_core[[2]],
    "ggplot"
  )
})


test_that("2: Vignette workflow produces consistent results", {
  skip_on_cran()
  skip_if_not_installed("phyloseq")
  skip_if_not_installed("tidyverse")
  skip_if_not_installed("viridis")

  # Load reference data from saved vignette run
  rda_path <- getOption(
    "brcore_test_data_path",
    default = test_path("test_sets/test_vignette_data.rda")
  )
  skip_if_not(
    file.exists(rda_path),
    "Test data file not available for testing in this environment: CRAN or R CMD check"
  )
  load(rda_path)

  # Run the vignette workflow step by step
  library(phyloseq)
  library(tidyverse)
  library(viridis)

  # Step 1: Load bcse data
  data("bcse", package = "BRCore", envir = environment())

  # Step 2: Calculate rarefaction metrics
  bcse_metrics <- add_rarefaction_metrics(data = bcse)

  # Test: Rarefaction metrics match
  expect_s4_class(bcse_metrics, "phyloseq")
  expect_identical(
    sample_data(bcse_metrics),
    sample_data(test_vignette_data$test_bcse_metrics)
  )

  # Step 3: Plot rarefaction metrics
  rarefaction_plot <- plot_rarefaction_metrics(bcse_metrics)

  # Test: Plot structure matches
  expect_s3_class(rarefaction_plot, "ggplot")
  expect_identical(
    rarefaction_plot$labels,
    test_vignette_data$test_rarefaction_plot$labels
  )

  # Step 4: Rarefaction

  bcse_rarefied_list <- multi_rarefy(
    physeq_obj = bcse,
    depth_level = 1000,
    num_iter = 2,
    .as = "list",
    set_seed = 7642
  )

  # Test: Rarefied OTU table properties match
  expect_equal(
    dim(bcse_rarefied_list),
    dim(test_vignette_data$test_bcse_rarefied_otutable)
  )
  # Test: All samples have row sums of 1000 (accounting for floating-point precision)
  expect_equal(
    rowSums(bcse_rarefied_list[[1]]),
    setNames(
      rep(1000, nrow(bcse_rarefied_list[[1]])),
      rownames(bcse_rarefied_list[[1]])
    ),
    tolerance = 1e-6
  )
  expect_equal(
    bcse_rarefied_list,
    test_vignette_data$test_bcse_rarefied_otutable,
    tolerance = 1e-6
  )

  # Step 5: Update OTU table (optional)
  bcse_rare <- update_otu_table(
    physeq_obj = bcse,
    rarefied_otus = bcse_rarefied_list,
    iteration = 1
  )

  # Test: Updated phyloseq object matches
  expect_s4_class(bcse_rare, "phyloseq")
  expect_equal(ntaxa(bcse_rare), ntaxa(test_vignette_data$test_bcse_rare))
  expect_equal(nsamples(bcse_rare), nsamples(test_vignette_data$test_bcse_rare))
  expect_equal(
    sample_sums(bcse_rare),
    sample_sums(test_vignette_data$test_bcse_rare),
    tolerance = 1e-6
  )

  # Step 6: Identify core microbiome
  bcse_rare_core <- identify_core(
    physeq_obj = bcse,
    rarefied_list = bcse_rarefied_list,
    priority_var = "Crop",
    increase_value = 0.02,
    abundance_weight = 0,
    depth_level = 1000,
    seed = 2134
  )

  # Test: Core identification results match
  expect_s3_class(bcse_rare_core, "identify_core_result")
  expect_equal(
    bcse_rare_core$priority_var,
    test_vignette_data$test_bcse_rare_core$priority_var
  )
  expect_equal(
    bcse_rare_core$abundance_weight,
    test_vignette_data$test_bcse_rare_core$abundance_weight
  )
  expect_equal(
    bcse_rare_core$elbow,
    test_vignette_data$test_bcse_rare_core$elbow
  )
  expect_equal(
    bcse_rare_core$bc_increase,
    test_vignette_data$test_bcse_rare_core$bc_increase
  )
  expect_equal(
    bcse_rare_core$increase_value,
    test_vignette_data$test_bcse_rare_core$increase_value
  )

  # Test: Core sets match
  expect_equal(
    bcse_rare_core$elbow_core,
    test_vignette_data$test_bcse_rare_core$elbow_core
  )
  expect_equal(
    bcse_rare_core$increase_core,
    test_vignette_data$test_bcse_rare_core$increase_core
  )

  # Test: Bray-Curtis ranked data matches
  expect_equal(
    bcse_rare_core$bray_curtis_ranked,
    test_vignette_data$test_bcse_rare_core$bray_curtis_ranked,
    tolerance = 1e-3
  )

  # Step 7: Plot identified core
  bcse_identified_core <- plot_identified_core(
    bray_curtis_ranked = bcse_rare_core$bray_curtis_ranked,
    elbow = bcse_rare_core$elbow,
    lastCall = bcse_rare_core$bc_increase,
    increase_value = bcse_rare_core$increase_value
  )

  # Test: Core identification plot structure
  expect_s3_class(bcse_identified_core[[2]], "ggplot")
  expect_identical(
    bcse_identified_core$labels,
    test_vignette_data$test_bcse_identified_core$labels
  )

  # Step 8: Plot abundance-occupancy (increase)
  plot_abund_occ_increase <- plot_abundance_occupancy(
    core_result = bcse_rare_core,
    core_set = "increase"
  )

  # Test: Abundance-occupancy plot structure (increase)
  expect_s3_class(plot_abund_occ_increase, "ggplot")
  expect_identical(
    plot_abund_occ_increase$labels,
    test_vignette_data$test_plot_abund_occ_increase$labels
  )

  # Step 9: Plot abundance-occupancy (elbow)
  plot_abund_occ_elbow <- plot_abundance_occupancy(
    core_result = bcse_rare_core,
    core_set = "elbow"
  )

  # Test: Abundance-occupancy plot structure (elbow)
  expect_s3_class(plot_abund_occ_elbow, "ggplot")
  expect_identical(
    plot_abund_occ_elbow$labels,
    test_vignette_data$test_plot_abund_occ_elbow$labels
  )
})


test_that("3: Vignette core distribution plots are consistent", {
  skip_on_cran()
  skip_if_not_installed("phyloseq")
  skip_if_not_installed("tidyverse")
  skip_if_not_installed("viridis")

  # Load reference data
  rda_path <- getOption(
    "brcore_test_data_path",
    default = test_path("test_sets/test_vignette_data.rda")
  )
  skip_if_not(
    file.exists(rda_path),
    "Test data file not available for testing in this environment: CRAN or R CMD check"
  )
  load(rda_path)

  # Recreate necessary objects for plotting
  data("bcse", package = "BRCore", envir = environment())
  library(phyloseq)
  library(tidyverse)
  library(viridis)

  bcse_rarefied_list <- multi_rarefy(
    physeq_obj = bcse,
    depth_level = 1000,
    num_iter = 2,
    .as = "list",
    set_seed = 7642
  )

  # Test variance propagation plot
  # Explore variance propagation and determine the best depth_level for core identification
  p <- plot_variance_propagation(
    physeq_obj = bcse,
    rarefied = bcse_rarefied_list,
    q = 0,
    group_var = "Crop",
    group_color = "Plot"
  )

  expect_s3_class(p, "ggplot")
  expect_silent(ggplot2::ggplot_build(p))

  # Update OTU table (optional)
  bcse_updated_rare <- update_otu_table(
    physeq_obj = bcse,
    rarefied_otus = bcse_rarefied_list,
    iteration = 1
  )

  # Continue with core identification with original phyloseq or updated phyloseq object.
  # We continue with the original phyloseq object for consistency with the vignette workflow, but this can be switched to the updated phyloseq object if desired.

  bcse_rare_core <- identify_core(
    physeq_obj = bcse,
    rarefied_list = bcse_rarefied_list,
    priority_var = "Crop",
    increase_value = 0.02,
    abundance_weight = 0,
    depth_level = 1000,
    seed = 2134
  )

  # Test bar plot
  plot_core_dist_bar <- plot_core_distribution(
    core_result = bcse_rare_core,
    core_set = "increase",
    group_var = "Crop",
    plot_type = "bar"
  )

  expect_s3_class(plot_core_dist_bar, "ggplot")
  expect_identical(
    plot_core_dist_bar$labels,
    test_vignette_data$test_plot_core_dist_bar$labels
  )

  # Test line plot
  plot_core_dist_line <- plot_core_distribution(
    core_result = bcse_rare_core,
    core_set = "increase",
    group_var = "Crop",
    plot_type = "line"
  )

  expect_s3_class(plot_core_dist_line, "ggplot")
  expect_identical(
    plot_core_dist_line$labels,
    test_vignette_data$test_plot_core_dist_line$labels
  )

  # Test heatmap (with recoded Crop variable)
  bcse_rare_core$metadata <- bcse_rare_core$metadata |>
    mutate(
      Crop = recode(
        Crop,
        "Corn" = "Corn",
        "Sorghum" = "Sorghum",
        "Continuous Sorghum" = "Sorghum + cover crop",
        "Miscanthus" = "Miscanthus",
        "New Switchgrass" = "Establishing switchgrass",
        "Switchgrass" = "Mature switchgrass",
        "Early Succession" = "Successional vegetation",
        "Native Grasses" = "Native grass mix",
        "Prairie" = "Reconstructed prairie",
        "Poplar" = "Poplar"
      ),
      Crop = factor(
        Crop,
        levels = c(
          "Corn",
          "Sorghum",
          "Sorghum + cover crop",
          "Miscanthus",
          "Establishing switchgrass",
          "Mature switchgrass",
          "Successional vegetation",
          "Native grass mix",
          "Reconstructed prairie",
          "Poplar"
        )
      )
    )

  plot_core_dist_heatmap <- plot_core_distribution(
    core_result = bcse_rare_core,
    core_set = "increase",
    group_var = "Crop",
    plot_type = "heatmap"
  ) +
    viridis::scale_fill_viridis(option = "plasma", name = "Occupancy")

  expect_s3_class(plot_core_dist_heatmap, "ggplot")
  expect_identical(
    plot_core_dist_heatmap$labels$fill,
    test_vignette_data$test_plot_core_dist_heatmap$labels$fill
  )
})


test_that("4: Vignette neutral model fitting is consistent", {
  skip_on_cran()
  skip_if_not_installed("phyloseq")
  skip_if_not_installed("tidyverse")

  # Load reference data
  rda_path <- getOption(
    "brcore_test_data_path",
    default = test_path("test_sets/test_vignette_data.rda")
  )
  skip_if_not(
    file.exists(rda_path),
    "Test data file not available for testing in this environment: CRAN or R CMD check"
  )
  load(rda_path)

  # Recreate necessary objects
  data("bcse", package = "BRCore", envir = environment())
  library(phyloseq)
  library(tidyverse)

  bcse_rarefied_list <- multi_rarefy(
    physeq_obj = bcse,
    depth_level = 1000,
    num_iter = 2,
    .as = "list",
    set_seed = 7642
  )

  bcse_rare_core <- identify_core(
    physeq_obj = bcse,
    rarefied_list = bcse_rarefied_list,
    priority_var = "Crop",
    increase_value = 0.02,
    abundance_weight = 0,
    depth_level = 1000,
    seed = 2134
  )

  # Fit neutral model
  bcse_rare_core_neutral_fit <- fit_neutral_model(
    otu_table = bcse_rare_core$otu_table,
    core_set = bcse_rare_core$increase_core,
    abundance_occupancy = bcse_rare_core$abundance_occupancy
  )

  # Test: Neutral model fit structure
  expect_s3_class(bcse_rare_core_neutral_fit, "fit_neutral_model")
  expect_named(
    bcse_rare_core_neutral_fit,
    c("goodness_of_fit", "model_prediction")
  )

  # Test: Goodness of fit parameters match
  expect_equal(
    bcse_rare_core_neutral_fit$goodness_of_fit$m,
    test_vignette_data$test_bcse_rare_core_neutral_fit$goodness_of_fit$m,
    tolerance = 1e-2
  )

  expect_equal(
    bcse_rare_core_neutral_fit$goodness_of_fit$Rsqr,
    test_vignette_data$test_bcse_rare_core_neutral_fit$goodness_of_fit$Rsqr,
    tolerance = 1e-2
  )

  # Test: Model prediction dimensions match
  expect_equal(
    nrow(bcse_rare_core_neutral_fit$model_prediction),
    nrow(test_vignette_data$test_bcse_rare_core_neutral_fit$model_prediction)
  )

  # Test: Core membership classification matches
  core_membership_test <- bcse_rare_core_neutral_fit$model_prediction |>
    filter(membership == "Core")

  core_membership_ref <- test_vignette_data$test_bcse_rare_core_neutral_fit$model_prediction |>
    filter(membership == "Core")

  expect_equal(nrow(core_membership_test), nrow(core_membership_ref))
  expect_setequal(core_membership_test$otu, core_membership_ref$otu)

  # Plot neutral model
  plot_bcse_neutral_fit <- plot_neutral_model(bcse_rare_core_neutral_fit)

  # Test: Neutral model plot structure
  expect_s3_class(plot_bcse_neutral_fit, "ggplot")
  expect_identical(
    plot_bcse_neutral_fit$labels,
    test_vignette_data$test_plot_bcse_neutral_fit$labels
  )
})

# R/utils-otu_class_checks.R

#' Check if input is a phyloseq object
#' @param physeq_obj An object to check
#' @return Invisible NULL (errors if not a
#' phyloseq object)
#' @keywords internal
#' @noRd

.phyloseq_class_check <- function(physeq_obj) {
  if (inherits(physeq_obj, "phyloseq")) {
    cli::cli_alert_success("Input phyloseq object is valid!")
  }
  if (!inherits(physeq_obj, "phyloseq")) {
    cli::cli_abort(
      "Input must be a 'phyloseq' object, not a {.cls {class(physeq_obj)}}"
    )
  }
}


#' Extract OTU matrix from phyloseq object
#'
#' @param physeq_obj A phyloseq object
#' @param samples_as_rows Logical, return matrix with samples as rows and taxa
#' as columns (default TRUE)
#' @return A matrix with samples as rows and taxa as columns
#' @keywords internal
#' @noRd
.extract_otu_matrix <- function(physeq_obj, samples_as_rows = TRUE) {
  suppressMessages(.phyloseq_class_check(physeq_obj))

  otu_mat <- as(phyloseq::otu_table(physeq_obj), "matrix")

  if (phyloseq::taxa_are_rows(physeq_obj) == samples_as_rows) {
    otu_mat <- t(otu_mat)
  }

  otu_mat
}

#' Extract OTU matrix from phyloseq object or rarefied list/array
#'
#' @param rarefied A phyloseq object, list of data frames, or 3D array
#' @param iteration Which iteration to extract (for list/array input)
#' @return A matrix with samples as rows and taxa as columns
#' @keywords internal
#' @noRd
.extract_rarefied_matrix <- function(rarefied, iteration = NULL) {
  # Handle phyloseq object
  if (inherits(rarefied, "phyloseq")) {
    return(.extract_otu_matrix(rarefied))
  }

  # Handle 3D array
  if (is.array(rarefied) && length(dim(rarefied)) == 3) {
    if (is.null(iteration)) {
      cli::cli_abort("{.arg iteration} must be specified for array input.")
    }
    if (iteration < 1 || iteration > dim(rarefied)[3]) {
      cli::cli_abort(
        "{.arg iteration} must be between 1 and {dim(rarefied)[3]}."
      )
    }
    return(rarefied[,, iteration])
  }

  # Handle list of data frames/matrices
  if (is.list(rarefied)) {
    if (is.null(iteration)) {
      cli::cli_abort("{.arg iteration} must be specified for list input.")
    }
    if (iteration < 1 || iteration > length(rarefied)) {
      cli::cli_abort(
        "{.arg iteration} must be between 1 and {length(rarefied)}."
      )
    }
    return(as.matrix(rarefied[[iteration]]))
  }

  cli::cli_abort(
    "{.arg rarefied} must be a phyloseq object, list, or 3D array."
  )
}

#' Get number of iterations from rarefied object
#'
#' @param rarefied A list of data frames or 3D array from multi_rarefy()
#' @return Integer number of iterations
#' @keywords internal
#' @noRd
.get_n_iterations <- function(rarefied) {
  if (is.array(rarefied) && length(dim(rarefied)) == 3) {
    return(dim(rarefied)[3])
  } else if (is.list(rarefied)) {
    return(length(rarefied))
  } else {
    cli::cli_abort(
      "{.arg rarefied} must be a list or 3D array from multi_rarefy()."
    )
  }
}

#' Convert rarefied object to a list of matrices
#'
#' @param x A 3D array or list of matrices from `multi_rarefy()`
#' @return A list of matrices, one per iteration
#' @keywords internal
#' @noRd
.get_iter_list <- function(x) {
  if (is.array(x) && length(dim(x)) == 3) {
    lapply(seq_len(dim(x)[3]), function(i) x[,, i])
  } else if (is.list(x)) {
    x
  } else {
    list(as.matrix(x))
  }
}


#' Extract sample IDs from rarefied object
#'
#' @param rarefied A list of data frames or 3D array from multi_rarefy()
#' @param remove_suffix Logical, remove "_iter_N" suffix from list rownames
#' @return Character vector of sample IDs
#' @keywords internal
#' @noRd
.get_sample_ids <- function(rarefied, remove_suffix = TRUE) {
  if (is.array(rarefied) && length(dim(rarefied)) == 3) {
    return(dimnames(rarefied)[[1]])
  } else if (is.list(rarefied)) {
    sample_ids <- rownames(rarefied[[1]])

    return(sample_ids)
  } else {
    cli::cli_abort(
      "{.arg rarefied} must be a list or 3D array from multi_rarefy()."
    )
  }
}

#' Variance propagation diagnostic for rarefaction
#'
#' This function evaluate the variance between rarefaction iterations from \code{multi_rarefy()} by visually comparing raw vs. rarefied alpha diversity metrics calculated at each iterations. It is possible to plot observed richness (q=0), Shannon diversity (q=1), or Simpson diversity (q=2) by setting the `q` parameter to "richness" or `q` = 0, "shannon" or `q` = 1, or "shannon" or `q` = 2. The plot is faceted by method (raw vs rarefied) and colored by a specified grouping variable from the sample data.
#'
#' @param physeq_obj Raw phyloseq object
#' @param rarefied Output from multi_rarefy(). Either a list of dataframes or and array.
#' @param q Hill number order (q = 0 for richness, q = 1 for Shannon, q = 2 for Simpson)
#' @param group_var A grouping variable to use gor grouping as in the sample_data()
#' @param group_color A color variable to use present in the sample_data()
#' @param convert_to_factor Logical. If \code{TRUE}, both \code{group_var} and
#'   \code{group_color} are coerced to \code{factor} before plotting, which is
#'   useful when those columns are numeric/continuous (e.g. dates, counts) but
#'   should be treated as discrete groups. When \code{TRUE} a discrete color
#'   scale (\code{scale_color_viridis_d}) is used; otherwise the continuous
#'   scale (\code{scale_color_viridis_c}) is used. Default \code{FALSE}.
#'

#'
#' @return ggplot object comparing raw vs rarefied diversity distributions across iterations.
#'
#'
#' @examples
#' library(phyloseq)
#' library(BRCore)
#' # Example comparing hill q=1 between Poplar and Switchgrass plots
#' bcse_filt <- bcse |>
#' subset_samples(Crop %in% c("Poplar", "Switchgrass"))
#'
#' bcse_rarefied_otutable_filt <-
#'  multi_rarefy(
#'        physeq_obj = bcse_filt,
#'        depth_level = 1000,
#'        num_iter = 10,
#'        .as = "list",
#'        set_seed = 7643
#'    )
#'
#'plot_variance_propagation(
#'    physeq_obj   = bcse_filt,
#'    rarefied = bcse_rarefied_otutable_filt,
#'    q        = 1,
#'    group_var = "Crop",
#'    group_color = "Plot"
#')
#'
#' @importFrom tibble rownames_to_column column_to_rownames
#' @importFrom phyloseq otu_table sample_data taxa_are_rows
#' @importFrom cli cli_h1 cli_h2 cli_alert_info cli_alert_warning
#' @importFrom cli cli_alert_success cli_alert_danger
#' @importFrom utils head
#' @importFrom vegan rrarefy
#' @importFrom dplyr left_join all_of
#'
#' @export
plot_variance_propagation <- function(
  physeq_obj,
  rarefied,
  q = 0,
  group_var,
  group_color,
  convert_to_factor = FALSE
) {
  # Validate inputs
  .phyloseq_class_check(physeq_obj)
  .validate_group_vars(physeq_obj, group_var, group_color)

  cli::cli_h1("Rarefaction Variance Propagation Visualization")

  # Extract raw OTU

  otu_raw <- .extract_otu_matrix(physeq_obj, samples_as_rows = TRUE)

  metadata <- data.frame(phyloseq::sample_data(physeq_obj))
  metadata$sample_id <- rownames(metadata)

  rare_samples <- .get_sample_ids(rarefied)

  common_samples <- intersect(rownames(otu_raw), rare_samples)

  otu_raw <- otu_raw[common_samples, , drop = FALSE]
  metadata <- metadata[common_samples, , drop = FALSE]

  .validate_group_vars(physeq_obj, group_var, group_color)

  cli::cli_alert_info("Hill number order selected, q= {.val {q}}")

  raw_vals <- .hill_number(otu_raw, q = q)

  raw_df <- data.frame(
    sample_id = names(raw_vals),
    value = raw_vals,
    method = "Raw"
  ) |>
    left_join(
      metadata[, c("sample_id", group_var, group_color), drop = FALSE],
      by = "sample_id"
    )

  n_iter <- .get_n_iterations(rarefied)

  cli::cli_alert_info(
    "Number of rarefaction iterations, n_iter= {.val {n_iter}}"
  )

  # replicate raw values (metadata already joined)
  raw_df <- raw_df[rep(seq_len(nrow(raw_df)), n_iter), ]

  iter_list <- .get_iter_list(rarefied)

  # Check rarefied depth
  rarefied_depths <- vapply(
    iter_list,
    function(mat) {
      rowSums(as.matrix(mat))[1]
    },
    double(1)
  )

  rare_df <- lapply(seq_along(iter_list), function(i) {
    mat <- as.matrix(iter_list[[i]])
    vals <- .hill_number(mat, q = q)
    data.frame(
      sample_id = names(vals),
      value = vals,
      method = "Rarefied"
    )
  }) |>
    bind_rows() |>
    left_join(
      metadata[, c("sample_id", group_var, group_color), drop = FALSE],
      by = "sample_id"
    )

  # Combine
  all_df <- bind_rows(
    mutate(raw_df, iter = NA_integer_),
    mutate(rare_df, iter = rep(seq_len(n_iter), each = length(common_samples)))
  ) |>
    mutate(method = factor(.data$method, levels = c("Raw", "Rarefied")))

  # Optionally coerce grouping columns to factor
  if (convert_to_factor) {
    all_df <- all_df |>
      mutate(across(dplyr::all_of(c(group_var, group_color)), as.factor))
    cli::cli_alert_info(
      "Converted {.val {group_var}} and {.val {group_color}} to factor."
    )
  }

  # Choose color scale based on whether group_color is discrete
  color_scale <- if (
    convert_to_factor ||
      is.factor(all_df[[group_color]]) ||
      is.character(all_df[[group_color]])
  ) {
    ggplot2::scale_color_viridis_d(option = "turbo")
  } else {
    ggplot2::scale_color_viridis_c(option = "turbo")
  }

  p <- ggplot(
    all_df,
    aes(x = .data[[group_var]], y = .data$value, color = .data[[group_color]])
  ) +
    geom_jitter(width = 0.15, height = 0, alpha = 0.7, size = 0.8) +
    facet_wrap(~ .data$method) +
    .brcore_theme(axis_text_angle = 90) +
    color_scale +
    guides(
      color = guide_legend(override.aes = list(size = 5, shape = 15))
    ) +
    labs(
      x = group_var,
      title = "Raw vs Rarefied Alpha Diversity",
      subtitle = paste0(
        "Rarefied to depth of ",
        rarefied_depths[1],
        ", over ",
        n_iter,
        " iterations"
      ),
      y = if (is.numeric(q)) {
        switch(
          as.character(q),
          "0" = "Hill number (q = 0)",
          "1" = "Hill number (q = 1)",
          "2" = "Hill number (q = 2)",
          paste0("General Hill number (q = ", q, ")")
        )
      } else {
        switch(
          q,
          "richness" = "Observed Richness",
          "shannon" = "Shannon Diversity Index",
          "simpson" = "Simpson Diversity Index",
          paste0("Hill number ", q)
        )
      }
    )

  cli::cli_alert_info("Comparison plot generated!")
  return(p)
}


#' Compute Hill numbers for samples
#'
#' Based on vegan::diversity() function, but modified to
#' handle both vector and matrix input, and to support named
#' indices for common q values.
#' @param x Numeric vector or matrix of species abundances.
#'   If matrix, samples are rows and taxa are columns.
#' @param q Hill number order. Can be:
#'   - "richness" (q=0): Species richness
#'   - "shannon" (q=1): Exponential Shannon entropy
#'   - "simpson" (q=2): Inverse Simpson concentration
#'   - Numeric value: General Hill number of order q
#' @return Numeric vector of Hill numbers (one per sample)
#' @keywords internal
#' @noRd
.hill_number <- function(x, q = "richness") {
  # Named indices for common cases
  INDICES <- c("richness", "shannon", "simpson")

  # Handle numeric q or match named index
  if (is.numeric(q)) {
    q_val <- q
  } else {
    q_val <- match.arg(q, INDICES)
  }

  # Handle matrix (rows = samples) or vector input
  if (length(dim(x)) > 1) {
    # Matrix: rows are samples, columns are taxa
    total <- rowSums(x)
    x <- x / total # Proportions (vectorized)
    x[x == 0] <- NA # Set zeros to NA (ignored in rowSums)
  } else {
    # Vector: single sample
    total <- sum(x)
    x <- x / total
    x[x == 0] <- NA
  }

  # Compute diversity based on q
  if (q_val == 0 || q_val == "richness") {
    # Richness: count non-zero species (same regardless of hill_form)
    if (length(dim(x)) > 1) {
      H <- rowSums(!is.na(x)) # Count non-NA values
    } else {
      H <- sum(!is.na(x))
    }
  } else if (q_val == 1 || q_val == "shannon") {
    # Shannon
    x_log <- x * log(x)
    if (length(dim(x)) > 1) {
      H <- -rowSums(x_log, na.rm = TRUE)
    } else {
      H <- -sum(x_log, na.rm = TRUE)
    }

    # Convert to Hill number if requested
    if (q_val == 1) {
      H <- exp(H)
    }
  } else if (q_val == 2 || q_val == "simpson") {
    # Simpson
    x_sq <- x^2
    if (length(dim(x)) > 1) {
      H <- rowSums(x_sq, na.rm = TRUE)
    } else {
      H <- sum(x_sq, na.rm = TRUE)
    }

    if (q_val == 2) {
      #Inverse Simpson concentration: 1 / sum(p^2)
      H <- 1 / H
    } else {
      # Simpson index: 1 - sum(p^2)
      H <- 1 - H
    }
  } else {
    # General Hill number not 0,1,2: (sum(p^q))^(1/(1-q))
    x_pow <- x^q_val
    if (length(dim(x)) > 1) {
      H <- rowSums(x_pow, na.rm = TRUE)^(1 / (1 - q_val))
    } else {
      H <- sum(x_pow, na.rm = TRUE)^(1 / (1 - q_val))
    }
  }

  # Handle NA totals (samples with zero abundance)
  if (any(NAS <- is.na(total))) {
    H[NAS] <- NA
  }

  H
}

#' Validate that group variables exist in sample data
#'
#' @param physeq_obj A phyloseq object
#' @param group_var Character, grouping variable name
#' @param group_color Character, color variable name (optional)
#' @return Invisible NULL (errors if validation fails)
#' @keywords internal
#' @noRd
.validate_group_vars <- function(physeq_obj, group_var, group_color = NULL) {
  sample_df <- as(phyloseq::sample_data(physeq_obj), "data.frame")

  if (!group_var %in% colnames(sample_df)) {
    cli::cli_abort(
      "{.arg group_var} '{group_var}' not found in sample_data()."
    )
  }

  if (!is.null(group_color) && !group_color %in% colnames(sample_df)) {
    cli::cli_abort(
      "{.arg group_color} '{group_color}' not found in sample_data()."
    )
  }

  invisible(NULL)
}

# tests/testthat/test-plot_variance_propagation.R

test_that("plot_variance_propagation rejects non-phyloseq input", {
  skip_if_not_installed("phyloseq")

  expect_error(
    plot_variance_propagation(
      physeq_obj = data.frame(a = 1),
      rarefied = list(),
      q = 0,
      group_var = "x",
      group_color = "y"
    ),
    "phyloseq"
  )
})

# List input ----

test_that("plot_variance_propagation returns ggplot with list input", {
  skip_if_not_installed("phyloseq")
  skip_if_not_installed("ggplot2")

  data("bcse", package = "BRCore", envir = environment())

  rarefied <- multi_rarefy(
    physeq_obj = bcse,
    depth_level = 3000,
    num_iter = 3,
    .as = "list",
    set_seed = 7643
  )

  p <- plot_variance_propagation(
    physeq_obj = bcse,
    rarefied = rarefied,
    q = 0,
    group_var = "Crop",
    group_color = "Plot"
  )

  expect_s3_class(p, "ggplot")
  expect_silent(ggplot2::ggplot_build(p))
})

test_that("plot works with q = 1 (Shannon)", {
  skip_if_not_installed("phyloseq")
  skip_if_not_installed("ggplot2")

  data("bcse", package = "BRCore", envir = environment())

  rarefied <- multi_rarefy(
    physeq_obj = bcse,
    depth_level = 3000,
    num_iter = 3,
    .as = "list",
    set_seed = 42
  )

  p <- plot_variance_propagation(
    physeq_obj = bcse,
    rarefied = rarefied,
    q = 1,
    group_var = "Crop",
    group_color = "Plot"
  )

  expect_s3_class(p, "ggplot")
  expect_silent(ggplot2::ggplot_build(p))
})

test_that("plot works with q = 2 (Simpson)", {
  skip_if_not_installed("phyloseq")
  skip_if_not_installed("ggplot2")

  data("bcse", package = "BRCore", envir = environment())

  rarefied <- multi_rarefy(
    physeq_obj = bcse,
    depth_level = 3000,
    num_iter = 3,
    .as = "list",
    set_seed = 42
  )

  p <- plot_variance_propagation(
    physeq_obj = bcse,
    rarefied = rarefied,
    q = 2,
    group_var = "Crop",
    group_color = "Plot"
  )

  expect_s3_class(p, "ggplot")
  expect_silent(ggplot2::ggplot_build(p))
})

# Array input ----

test_that("plot_variance_propagation returns ggplot with array input", {
  skip_if_not_installed("phyloseq")
  skip_if_not_installed("ggplot2")

  data("bcse", package = "BRCore", envir = environment())

  rarefied <- multi_rarefy(
    physeq_obj = bcse,
    depth_level = 3000,
    num_iter = 3,
    .as = "array",
    set_seed = 42
  )

  expect_true(is.array(rarefied))
  expect_equal(length(dim(rarefied)), 3)

  p <- plot_variance_propagation(
    physeq_obj = bcse,
    rarefied = rarefied,
    q = 1,
    group_var = "Crop",
    group_color = "Plot"
  )

  expect_s3_class(p, "ggplot")
  expect_silent(ggplot2::ggplot_build(p))
})

# Plot structure ----

test_that("plot has correct facets (Raw vs Rarefied)", {
  skip_if_not_installed("phyloseq")
  skip_if_not_installed("ggplot2")

  data("bcse", package = "BRCore", envir = environment())

  rarefied <- multi_rarefy(
    physeq_obj = bcse,
    depth_level = 3000,
    num_iter = 3,
    .as = "list",
    set_seed = 42
  )

  p <- plot_variance_propagation(
    physeq_obj = bcse,
    rarefied = rarefied,
    q = 0,
    group_var = "Crop",
    group_color = "Plot"
  )

  build <- ggplot2::ggplot_build(p)
  plot_data <- build$data[[1]]

  expect_true("PANEL" %in% names(plot_data))
  expect_equal(length(unique(plot_data$PANEL)), 2)
})

test_that("plot y-axis label reflects q value", {
  skip_if_not_installed("phyloseq")
  skip_if_not_installed("ggplot2")

  data("bcse", package = "BRCore", envir = environment())

  rarefied <- multi_rarefy(
    physeq_obj = bcse,
    depth_level = 3000,
    num_iter = 3,
    .as = "list",
    set_seed = 42
  )

  for (q_val in c(0, 1, 2)) {
    p <- plot_variance_propagation(
      physeq_obj = bcse,
      rarefied = rarefied,
      q = q_val,
      group_var = "Crop",
      group_color = "Plot"
    )

    expect_equal(p$labels$y, paste0("Hill number (q = ", q_val, ")"))
  }
})

# Sample matching ----

test_that("plot handles rarefied subset of samples", {
  skip_if_not_installed("phyloseq")
  skip_if_not_installed("ggplot2")

  data("bcse", package = "BRCore", envir = environment())

  bcse_sub <- phyloseq::subset_samples(bcse, Crop == "Switchgrass")

  rarefied <- multi_rarefy(
    physeq_obj = bcse_sub,
    depth_level = 3000,
    num_iter = 3,
    .as = "list",
    set_seed = 42
  )

  p <- plot_variance_propagation(
    physeq_obj = bcse_sub,
    rarefied = rarefied,
    q = 1,
    group_var = "Crop",
    group_color = "Plot"
  )

  expect_s3_class(p, "ggplot")
  expect_silent(ggplot2::ggplot_build(p))
})

# Vector q (currently errors) ----

test_that("plot_variance_propagation errors with vector q", {
  skip_if_not_installed("phyloseq")

  data("bcse", package = "BRCore", envir = environment())

  rarefied <- multi_rarefy(
    physeq_obj = bcse,
    depth_level = 3000,
    num_iter = 3,
    .as = "list",
    set_seed = 42
  )

  # .hill_number does not support length(q) > 1
  expect_error(
    plot_variance_propagation(
      physeq_obj = bcse,
      rarefied = rarefied,
      q = c(0, 1, 2),
      group_var = "Crop",
      group_color = "Plot"
    )
  )
})

#' Add a rarefied otu_table to a phyloseq object
#'
#' This function updates a `phyloseq` object by replacing its OTU/ASV table
#' with a rarefied version produced by `multi_rarefy()`. The rarefied table can
#' be a data frame, a list of data frames (`.as = "list"`), or a 3D array
#' (`.as = "array"`). When providing a list or array, specify which iteration
#' to use via the `iteration` parameter.
#'
#' @param physeq_obj A `phyloseq` object in which you want to add the rarefied
#' OTU/ASV table.
#' @param rarefied_otus A data frame, list of data frames, or 3D array output from
#' `multi_rarefy()` containing the rarefied OTU/ASV tables.
#' @param iteration Integer specifying which iteration to extract from a list
#' or array. Required when `rarefied_otus` is a list or array. Ignored when
#' `rarefied_otus` is a data frame.
#'
#' @return A `phyloseq` object.
#'
#' @seealso [multi_rarefy()]
#'
#' @examples
#' library(phyloseq)
#' library(BRCore)
#' data(GlobalPatterns, package = "phyloseq")
#'
#' # List output
#' otu_list <-
#'   multi_rarefy(
#'     physeq_obj = GlobalPatterns,
#'     depth_level = 200,
#'     num_iter = 3,
#'     .as = "list",
#'     set_seed = 123
#'   )
#'
#' # Extract iteration 2
#' rarefied_gp <- update_otu_table(GlobalPatterns, otu_list, iteration = 2)
#'
#' # Array output
#' otu_array <-
#'   multi_rarefy(
#'     physeq_obj = GlobalPatterns,
#'     depth_level = 200,
#'     num_iter = 3,
#'     .as = "array",
#'     set_seed = 123
#'   )
#'
#' # Extract iteration 1
#' rarefied_gp2 <- update_otu_table(GlobalPatterns, otu_array, iteration = 1)
#'
#' @importFrom phyloseq otu_table sample_data tax_table phy_tree refseq
#' @importFrom phyloseq phyloseq sample_names prune_taxa prune_samples
#' @importFrom phyloseq sample_sums taxa_sums
#' @importFrom cli cli_inform cli_alert_success cli_alert_warning cli_alert_info
#' @importFrom cli cli_abort
#' @importFrom magrittr %>%
#' @export
update_otu_table <- function(physeq_obj, rarefied_otus, iteration = NULL) {
  suppressMessages(.phyloseq_class_check(physeq_obj))

  physeq_samples <- sample_names(physeq_obj)

  # Handle list input
  if (is.list(rarefied_otus) && !is.data.frame(rarefied_otus)) {
    if (is.null(iteration)) {
      cli::cli_abort(
        "{.arg iteration} must be specified when {.arg rarefied_otus} is a list."
      )
    }

    n_iter <- length(rarefied_otus)

    if (iteration < 1 || iteration > n_iter) {
      cli::cli_abort(
        "{.arg iteration} must be between 1 and {n_iter}."
      )
    }

    cli::cli_alert_info(
      "Extracting iteration {.val {iteration}} from list of {.val {n_iter}} iterations."
    )

    rarefied_otus <- as.data.frame(rarefied_otus[[iteration]])
  }

  # Handle 3D array input
  if (is.array(rarefied_otus) && length(dim(rarefied_otus)) == 3) {
    if (is.null(iteration)) {
      cli::cli_abort(
        "{.arg iteration} must be specified when {.arg rarefied_otus} is a 3D array."
      )
    }

    n_iter <- dim(rarefied_otus)[3]

    if (iteration < 1 || iteration > n_iter) {
      cli::cli_abort(
        "{.arg iteration} must be between 1 and {n_iter}."
      )
    }

    cli::cli_alert_info(
      "Extracting iteration {.val {iteration}} from array with {.val {n_iter}} iterations."
    )

    rarefied_otus <- as.data.frame(rarefied_otus[,, iteration])
  }

  # Now rarefied_otus is always a data frame
  otu_rare_samples <- rownames(rarefied_otus)
  shared_samples <- intersect(physeq_samples, otu_rare_samples)
  removed_samples <- setdiff(physeq_samples, otu_rare_samples)

  .report_sample_status(
    shared_samples,
    removed_samples,
    rarefied_otus,
    physeq_samples
  )

  otu_rare_ord <- rarefied_otus[shared_samples, , drop = FALSE]

  cli::cli_alert_info(
    "Building phyloseq object with {.val {nrow(otu_rare_ord)}} samples and {.val {ncol(otu_rare_ord)}} taxa"
  )

  # Build components
  phyloseq_components <- list(
    otu_table(
      t(otu_rare_ord) |> as.matrix() |> as.data.frame(),
      taxa_are_rows = TRUE
    ),
    sample_data(physeq_obj)[shared_samples, ]
  )

  # Add optional components if they exist
  .add_optional_component <- function(components, accessor) {
    obj <- accessor(physeq_obj, errorIfNULL = FALSE)
    if (!is.null(obj)) c(components, list(obj)) else components
  }

  phyloseq_components <- .add_optional_component(phyloseq_components, tax_table)
  phyloseq_components <- .add_optional_component(phyloseq_components, phy_tree)
  phyloseq_components <- .add_optional_component(phyloseq_components, refseq)

  # Build the new phyloseq object
  new_phyloseq <- do.call(phyloseq, phyloseq_components) %>%
    prune_taxa(taxa_sums(x = .) > 0, x = .) %>%
    prune_samples(sample_sums(x = .) > 0, x = .)

  cli::cli_alert_success("Update complete!")

  return(new_phyloseq)
}


#' Report sample status after rarefaction
#' Internal function to report on the status of samples after rarefaction, including any removed samples and whether sample names match between the original phyloseq object and the rarefied OTU table.
#' @noRd
#' @keywords internal
.report_sample_status <- function(
  shared_samples,
  removed_samples,
  rarefied_otus,
  physeq_samples
) {
  if (length(shared_samples) == 0) {
    cli::cli_alert_danger(
      "No matching sample names between phyloseq object and rarefied OTU table."
    )
    stop("No matching sample names. Check rownames of rarefied_otus.")
  }

  otu_rare_samples <- rownames(rarefied_otus)

  if (identical(sort(physeq_samples), sort(otu_rare_samples))) {
    cli::cli_alert_success(
      "Phyloseq object and rarefied otu_table sample names are identical."
    )
  } else {
    cli::cli_alert_warning(
      "Phyloseq object and rarefied otu_table sample names are NOT identical. Check samples removed by rarefaction below."
    )
  }

  if (length(removed_samples) > 0) {
    cli::cli_alert_warning(
      "{.val {length(removed_samples)}} sample{?s} removed due to rarefaction: {.val {paste(removed_samples, collapse = ', ')}}"
    )
  } else {
    rarefaction_depth <- sum(rarefied_otus[1, ])
    cli::cli_alert_success(
      "All samples kept after rarefaction at depth of: {.val {rarefaction_depth}}"
    )
  }
}

test_that("phyloseq object correctly created", {
  skip_if_not_installed("phyloseq")

  data("bcse", package = "BRCore", envir = environment())

  otu_table_rare <-
    multi_rarefy(
      physeq_obj = bcse,
      depth_level = 200,
      num_iter = 3,
      .as = "list",
      set_seed = 123
    )

  test_bcse_rare <- update_otu_table(bcse, otu_table_rare, iteration = 1)
  read_counts <- sample_sums(test_bcse_rare)

  expect_true(class(test_bcse_rare) == "phyloseq")

  expect_true(is.numeric(read_counts))
  expect_true(length(read_counts) > 0)
  expect_true(all(read_counts > 0))

  # Test if all the samples have the same number of reads
  expect_equal(max(read_counts) - min(read_counts), 0) # All values are identical
  expect_equal(median(read_counts), mean(read_counts))
})

# Script to regenerate test_vignette_data.rda
# Run this script after making changes to multi_rarefy() or other functions
# that affect the vignette workflow outputs.
#
# Usage:
#   cd BRCore
#   Rscript tests/test_sets/regenerate_test_data.R
#
# This will regenerate the reference data used by test-vignette-workflow.R

devtools::load_all()
library(phyloseq)
library(tidyverse)
library(viridis)

cat("=== Regenerating test_vignette_data.rda ===\n\n")

# Step 1: Load bcse data
cat("Loading bcse data...\n")
data("bcse", package = "BRCore")

# Step 2: Calculate rarefaction metrics
cat("Calculating rarefaction metrics...\n")
test_bcse_metrics <- add_rarefaction_metrics(data = bcse)

# Step 3: Plot rarefaction metrics
cat("Plotting rarefaction metrics...\n")
test_rarefaction_plot <- plot_rarefaction_metrics(test_bcse_metrics)

# Step 4: Rarefaction
# Use single thread for consistency with test expectations
cat("Running multi_rarefy (this may take a while)...\n")
test_bcse_rarefied_otutable <- multi_rarefy(
  physeq_obj = bcse,
  depth_level = 1000,
  num_iter = 2,
  .as = "list",
  set_seed = 7642
)

# Step 5: Update OTU table (optional)
cat("Updating OTU table...\n")
test_bcse_rare <- update_otu_table(
  physeq_obj = bcse,
  rarefied_otus = test_bcse_rarefied_otutable,
  iteration = 1
)

# Step 6: Identify core microbiome
cat("Identifying core microbiome...\n")
test_bcse_rare_core <- identify_core(
  physeq_obj = bcse,
  rarefied_list = test_bcse_rarefied_otutable,
  priority_var = "Crop",
  increase_value = 0.02,
  abundance_weight = 0,
  depth_level = 1000,
  seed = 2134
)

# Step 7: Plot identified core
cat("Plotting identified core...\n")
test_bcse_identified_core <- plot_identified_core(
  bray_curtis_ranked = test_bcse_rare_core$bray_curtis_ranked,
  elbow = test_bcse_rare_core$elbow,
  lastCall = test_bcse_rare_core$bc_increase,
  increase_value = test_bcse_rare_core$increase_value
)

# Step 8: Plot abundance-occupancy (increase)
cat("Plotting abundance-occupancy (increase)...\n")
test_plot_abund_occ_increase <- plot_abundance_occupancy(
  core_result = test_bcse_rare_core,
  core_set = "increase"
)

# Step 9: Plot abundance-occupancy (elbow)
cat("Plotting abundance-occupancy (elbow)...\n")
test_plot_abund_occ_elbow <- plot_abundance_occupancy(
  core_result = test_bcse_rare_core,
  core_set = "elbow"
)

# Step 10: Core distribution plots
cat("Creating core distribution plots...\n")

test_plot_core_dist_bar <- plot_core_distribution(
  core_result = test_bcse_rare_core,
  core_set = "increase",
  group_var = "Crop",
  plot_type = "bar"
)

test_plot_core_dist_line <- plot_core_distribution(
  core_result = test_bcse_rare_core,
  core_set = "increase",
  group_var = "Crop",
  plot_type = "line"
)

# Recode Crop variable for heatmap
test_bcse_rare_core_heatmap <- test_bcse_rare_core
test_bcse_rare_core_heatmap$metadata <- test_bcse_rare_core_heatmap$metadata |>
  mutate(
    Crop = recode(
      Crop,
      "Corn" = "Corn",
      "Sorghum" = "Sorghum",
      "Continuous Sorghum" = "Sorghum + cover crop",
      "Miscanthus" = "Miscanthus",
      "New Switchgrass" = "Establishing switchgrass",
      "Switchgrass" = "Mature switchgrass",
      "Early Succession" = "Successional vegetation",
      "Native Grasses" = "Native grass mix",
      "Prairie" = "Reconstructed prairie",
      "Poplar" = "Poplar"
    ),
    Crop = factor(
      Crop,
      levels = c(
        "Corn",
        "Sorghum",
        "Sorghum + cover crop",
        "Miscanthus",
        "Establishing switchgrass",
        "Mature switchgrass",
        "Successional vegetation",
        "Native grass mix",
        "Reconstructed prairie",
        "Poplar"
      )
    )
  )

test_plot_core_dist_heatmap <- plot_core_distribution(
  core_result = test_bcse_rare_core_heatmap,
  core_set = "increase",
  group_var = "Crop",
  plot_type = "heatmap"
) +
  viridis::scale_fill_viridis(option = "plasma", name = "Occupancy")

# Step 11: Fit neutral model
cat("Fitting neutral model...\n")
test_bcse_rare_core_neutral_fit <- fit_neutral_model(
  otu_table = test_bcse_rare_core$otu_table,
  core_set = test_bcse_rare_core$increase_core,
  abundance_occupancy = test_bcse_rare_core$abundance_occupancy
)

# Step 12: Plot neutral model
cat("Plotting neutral model...\n")
test_plot_bcse_neutral_fit <- plot_neutral_model(
  test_bcse_rare_core_neutral_fit
)

# Assemble the test data list
cat("\nAssembling test data...\n")
test_vignette_data <- list(
  test_bcse_metrics = test_bcse_metrics,
  test_rarefaction_plot = test_rarefaction_plot,
  test_bcse_rarefied_otutable = test_bcse_rarefied_otutable,
  test_bcse_rare = test_bcse_rare,
  test_bcse_rare_core = test_bcse_rare_core,
  test_bcse_identified_core = test_bcse_identified_core,
  test_plot_abund_occ_increase = test_plot_abund_occ_increase,
  test_plot_abund_occ_elbow = test_plot_abund_occ_elbow,
  test_plot_core_dist_bar = test_plot_core_dist_bar,
  test_plot_core_dist_line = test_plot_core_dist_line,
  test_plot_core_dist_heatmap = test_plot_core_dist_heatmap,
  test_bcse_rare_core_neutral_fit = test_bcse_rare_core_neutral_fit,
  test_plot_bcse_neutral_fit = test_plot_bcse_neutral_fit
)

# Save the test data
if (nzchar(Sys.getenv("CI"))) {
  out_path <- getOption(
    "brcore_test_data_path",
    default = "tests/testthat/test_sets/test_vignette_data.rda"
  )
} else {
  out_path <- "tests/testthat/test_sets/test_vignette_data.rda"
}
dir.create(dirname(out_path), recursive = TRUE, showWarnings = FALSE)

cat(paste0("Saving to ", out_path, "...\n"))
save(test_vignette_data, file = out_path)

cat("\n=== Done! ===\n")
cat("Reference data has been regenerated.\n")
cat("Please commit the updated test_vignette_data.rda file.\n")

# Print session info for reproducibility documentation
cat("\n=== Session Info ===\n")
print(sessionInfo())

#' Calculate and append pre-rarefaction statistics to microbiome data
#'
#' This function adds read count, singleton count, Good's coverage,
#' and marks outlier samples to a `phyloseq` object or `data.frame`
#' based on the OTU/ASV abundance table.
#'
#' @param data A `phyloseq` object or a `data.frame` with samples
#'   as rows and taxa as columns.
#'
#' @return The same object (`phyloseq` or `data.frame`) with new
#'   columns:
#' \itemize{
#'   \item `read_num`
#'   \item `singleton_num`
#'   \item `goods_cov`
#'   \item `outlier`
#' }
#'
#' @details About Good's coverage. Initially developed by Alan
#'   Turing and I.J. Good during their cryptographic analyses in
#'   World War II, it was later adopted by ecologists, particularly
#'   in microbial diversity studies, to assess the completeness of
#'   a sample's representation of the overall community.
#' It's calculated as `1 - (F1/N)`, where `F1` is the number of
#'   OTUs (Operational Taxonomic Units) represented by only one
#'   individual (singletons) and `N` is the total number of
#'   individuals in the sample. For example, a Good's coverage of
#'   0.95, means that 5% of the reads in that sample are from OTUs
#'   that appear only once.
#'
#' @seealso [plot_rarefaction_metrics()] for visualizing these metrics, and
#' [multi_rarefy()] for performing rarefaction on a `phyloseq` object.
#'
#' @examples
#' library(phyloseq)
#' library(BRCore)
#'
#' # Adding metrics to a "phyloseq" object
#' bcse_metrics <- add_rarefaction_metrics(data = bcse)
#' sample_data(bcse_metrics)|>
#' head(10)
#'
#'
#' # Adding metrics to a "data.frame" count table object
#' bcse_otutable <- as.data.frame(
#'   as.matrix(otu_table(bcse))
#' )
#'
#' bcse_otutable_metrics <- add_rarefaction_metrics(
#'   data = bcse_otutable
#' )
#' bcse_otutable_metrics[
#'   head(seq_len(nrow(bcse_otutable_metrics)), 10),
#'   tail(seq_len(ncol(bcse_otutable_metrics)), 20)
#' ]
#'
#'
#' @importFrom phyloseq otu_table sample_data sample_data<- taxa_are_rows
#' @importFrom tibble rownames_to_column column_to_rownames
#' @importFrom dplyr group_by summarize mutate ungroup left_join
#' @importFrom tidyr pivot_longer
#' @importFrom stats IQR quantile
#' @importFrom cli cli_abort
#'
#' @export
add_rarefaction_metrics <- function(data) {
  # Internal function to identify outliers
  find_outlier <- function(x) {
    x < quantile(x, 0.25) - 1.5 * IQR(x) |
      x > quantile(x, 0.75) + 1.5 * IQR(x)
  }

  # Extract OTU table and transpose if needed
  if (inherits(data, "phyloseq")) {
    otu_mat <- .extract_otu_matrix(data, samples_as_rows = FALSE)
    otu_df <- as.data.frame(otu_mat)
  } else if (inherits(data, "data.frame")) {
    otu_df <- as.data.frame(t(data))
  } else {
    cli::cli_abort("Input must be a phyloseq object or a data.frame")
  }

  # Compute metrics
  df_stats <- otu_df |>
    rownames_to_column("otu_id") |>
    pivot_longer(
      -otu_id,
      names_to = "sample_id",
      values_to = "seq_num"
    ) |>
    group_by(.data$sample_id) |>
    summarize(
      read_num = sum(.data$seq_num),
      singleton_num = sum(.data$seq_num == 1),
      goods_cov = 100 * (1 - .data$singleton_num / .data$read_num)
    ) |>
    mutate(
      outlier = ifelse(
        find_outlier(log10(.data$read_num)),
        .data$read_num,
        NA
      )
    ) |>
    ungroup()

  # Append to sample metadata or return updated data.frame
  if (inherits(data, "phyloseq")) {
    sample_df <- sample_data(data) |>
      as.matrix() |>
      as.data.frame() |>
      rownames_to_column("sample_id")

    sample_df_updated <- left_join(
      sample_df,
      df_stats,
      by = "sample_id"
    ) |>
      as.data.frame() |>
      column_to_rownames("sample_id")

    sample_data(data) <- sample_df_updated
    return(data)
  } else {
    data_out <- data |>
      rownames_to_column("sample_id") |>
      left_join(df_stats, by = "sample_id") |>
      column_to_rownames("sample_id")

    return(data_out)
  }
}

# tests/testthat/test-add_rarefaction_metrics.R
test_that("add_rarefaction_metrics works with phyloseq object", {
  skip_if_not_installed("phyloseq")

  # Create simple phyloseq object with 10 samples
  otu_mat <- matrix(sample(1:100, 50, replace = TRUE), nrow = 5, ncol = 10)
  rownames(otu_mat) <- paste0("OTU", 1:5)
  colnames(otu_mat) <- paste0("Sample", 1:10)

  sample_df <- data.frame(
    treatment = rep(c("A", "B"), 5),
    row.names = paste0("Sample", 1:10)
  )

  physeq_obj <- phyloseq::phyloseq(
    phyloseq::otu_table(otu_mat, taxa_are_rows = TRUE),
    phyloseq::sample_data(sample_df)
  )

  # Run function
  result <- add_rarefaction_metrics(physeq_obj)

  # Check it's still phyloseq
  expect_s4_class(result, "phyloseq")

  # Check new columns added
  sample_data <- as.data.frame(phyloseq::sample_data(result))
  expect_true("read_num" %in% colnames(sample_data))
  expect_true("singleton_num" %in% colnames(sample_data))
  expect_true("goods_cov" %in% colnames(sample_data))
  expect_true("outlier" %in% colnames(sample_data))

  # Check we still have 10 samples
  expect_equal(nrow(sample_data), 10)
})

test_that("add_rarefaction_metrics works with a data.frame input", {
  otu_df <- as.data.frame(
    as(phyloseq::otu_table(bcse), "matrix")
  )

  result <- add_rarefaction_metrics(data = otu_df)

  expect_s3_class(result, "data.frame")
  expect_true(all(
    c("read_num", "singleton_num", "goods_cov", "outlier") %in% names(result)
  ))
  expect_equal(nrow(result), nrow(otu_df))
})

test_that("add_rarefaction_metrics data.frame output has correct metric values", {
  otu_df <- as.data.frame(
    as(phyloseq::otu_table(bcse), "matrix")
  )

  result <- add_rarefaction_metrics(data = otu_df)

  expect_true(all(result$read_num > 0))
  expect_true(all(result$singleton_num >= 0))
  expect_true(all(result$goods_cov >= 0 & result$goods_cov <= 100))
})

test_that("add_rarefaction_metrics rejects invalid input", {
  expect_error(
    add_rarefaction_metrics(data = matrix(1:10, nrow = 2)),
    "phyloseq object or a data.frame"
  )
})

#' Plot Bray-Curtis increase over ranked OTU/ASVs
#'
#' @description
#' Visualize the cumulative normalized mean Bray-Curtis increase returned by
#' \link[=identify_core]{identify_core()}, over ranked OTU/ASVs and shows cutoff
#' points for elbow percent increase methods.
#'
#' @param bray_curtis_ranked A tibble as returned by `identify_core()$bray_curtis_ranked`.
#' @param elbow The number of OTU/ASVs identified by the elbow method (Integer).
#' @param lastCall The number of OTU/ASVs identified by the last percent
#' Bray-Curtis increase method (Integer).
#' @param increase_value The percent increase value in decimal (e.g. 0.02) used
#' for the Bray-Curtis increase method.
#' @param dataset_name Optional character string. When provided, it is
#'   prepended to the plot title (e.g. \code{"Switchgrass"}).
#'   Default \code{NULL} (no prefix).
#'
#' @return A list containing: 1) `df_for_plot`, a data frame used for plotting, and 2) `plot_identified_core`, a ggplot object visualizing the Bray-Curtis increase with annotated cutoff points.
#'
#' @details
#' The function converts `rank` to integers and zooms the x-axis to the first
#' `1.2 * lastCall` ranks. Label positions are computed dynamically from the
#' observed `proportionBC` range to avoid overlap.
#'
#' @seealso \link[=identify_core]{identify_core()}
#'
#' @examples
#' library(BRCore)
#' data("switchgrass_core", package = "BRCore")
#'
#' p <- plot_identified_core(
#'   bray_curtis_ranked = switchgrass_core$bray_curtis_ranked,
#'   elbow = switchgrass_core$elbow,
#'   lastCall = switchgrass_core$bc_increase,
#'   increase_value = switchgrass_core$increase_value
#' )
#' print(p)
#'
#' @seealso \code{\link{identify_core}}
#'
#' @importFrom grid unit
#' @importFrom dplyr mutate filter
#' @importFrom ggplot2 ggplot aes geom_point geom_vline annotate labs
#' @importFrom ggplot2 theme_classic theme element_text scale_x_continuous
#' @importFrom ggplot2 scale_y_continuous expansion
#'
#' @export
plot_identified_core <- function(
  bray_curtis_ranked,
  elbow,
  lastCall,
  increase_value = 0.02,
  dataset_name = NULL
) {
  new_data <- bray_curtis_ranked |>
    mutate(
      rank_num = suppressWarnings(as.integer(as.character(rank)))
    ) |>
    filter(rank_num <= 1.2 * lastCall) |>
    mutate(rank_fac = factor(rank_num, levels = rank_num))

  # Calculate percentage label text
  percent_label <- paste0(round(increase_value * 100), "%")

  # Calculate dynamic label positions
  max_rank <- max(new_data$rank_num, na.rm = TRUE)
  max_prop <- max(new_data$proportionBC, na.rm = TRUE)
  min_prop <- min(new_data$proportionBC, na.rm = TRUE)
  prop_range <- max_prop - min_prop

  # Dynamic y positions for labels
  elbow_y <- min_prop + 0.1 * prop_range
  lastCall_y <- min_prop + 0.35 * prop_range

  # Adjust x position for labels if too close to edges
  elbow_x_offset <- ifelse(
    elbow < max_rank * 1.2,
    elbow + max_rank * 0.02,
    elbow - max_rank * 0.02
  )
  lastCall_x_offset <- ifelse(
    lastCall > max_rank * 0.9,
    lastCall - max_rank * 0.02,
    lastCall + max_rank * 0.02
  )

  # Base plot
  p <- ggplot(new_data, aes(x = rank_num, y = proportionBC)) +
    geom_point(size = 2, alpha = 0.7) +
    geom_vline(
      xintercept = elbow,
      linetype = "dashed",
      color = "#d73027",
      linewidth = 1.2,
      alpha = 0.8
    ) +
    geom_vline(
      xintercept = lastCall,
      linetype = "dashed",
      color = "#1a9850",
      linewidth = 1.2,
      alpha = 0.8
    ) +
    annotate(
      "rect",
      xmin = 0,
      xmax = elbow,
      ymin = -Inf,
      ymax = Inf,
      fill = "white",
      alpha = 0.05
    ) +
    annotate(
      "rect",
      xmin = elbow,
      xmax = lastCall,
      ymin = -Inf,
      ymax = Inf,
      fill = "white",
      alpha = 0.05
    ) +
    annotate(
      "label",
      x = elbow_x_offset,
      y = elbow_y,
      label = paste0("Elbow (", elbow, ")"),
      color = "#d73027",
      fill = "white",
      size = 3.5,
      fontface = "bold",
      label.padding = unit(0.3, "lines")
    ) +
    annotate(
      "label",
      x = lastCall_x_offset,
      y = lastCall_y,
      label = paste0("Last ", percent_label, " (", lastCall, ")"),
      color = "#1a9850",
      fill = "white",
      size = 3.5,
      fontface = "bold",
      label.padding = unit(0.3, "lines")
    ) +
    labs(
      title = if (!is.null(dataset_name)) {
        bquote(bold(
          "Core Microbiome Identification for" ~ italic(
            .(dataset_name)
          ) ~
            "dataset"
        ))
      } else {
        expression(bold("Core Microbiome Identification"))
      },
      subtitle = paste0(
        "Identification Methods: Elbow (",
        elbow,
        " OTUs), Last ",
        percent_label,
        " (",
        lastCall,
        " OTUs)"
      ),
      x = "Ranked OTUs",
      y = "Bray-Curtis similarity"
    ) +
    .brcore_theme() +
    scale_y_continuous(labels = scales::label_percent(accuracy = 1)) +
    scale_x_continuous(breaks = seq(0, max(new_data$rank_num), by = 5))

  return(list(
    df_for_plot = new_data,
    plot_identified_core = p
  ))
}

# tests/testthat/test-plot_identified_core.R
test_that("plot_identified_core returns a ggplot and builds", {
  skip_if_not_installed("ggplot2")
  skip_if_not(
    exists("plot_identified_core"),
    "plot_identified_core() not found"
  )

  # minimal deterministic mock input
  n <- 80
  bray_curtis_ranked <- data.frame(
    rank = as.character(seq_len(n)), # function coerces this
    proportionBC = seq(0.01, 0.9, length.out = n)
  )
  elbow <- 10
  lastCall <- 30

  p <- plot_identified_core(
    bray_curtis_ranked,
    elbow = elbow,
    lastCall = lastCall
  )

  expect_type(p, "list")
  expect_s3_class(p[[1]], "data.frame")
  expect_s3_class(p[[2]], "ggplot")
  expect_silent(ggplot2::ggplot_build(p[[2]])) # renders without error
})

#' Internal warning handler for sncm.fit likelihood functions
#'
#' Wraps a function call with a `withCallingHandlers()` that captures warnings
#' and re-emits them via `cli::cli_warn()`. Deduplicates repeated warnings
#' across optimizer iterations using an external flag and setter.
#'
#' @param fun A zero-argument function (lambda) to evaluate.
#' @param message A string prefix for the warning message.
#' @param warned Logical flag indicating whether a warning has already fired.
#' @param warn_setter A zero-argument function that sets the external `warned`
#'   flag to `TRUE` via `<<-`.
#'
#' @return The result of `fun()`.
#' @keywords internal
.internal_warn_sncm <- function(
  fun,
  message = NULL,
  warned = FALSE,
  warn_setter = NULL
) {
  withCallingHandlers(
    fun(),
    warning = function(w) {
      if (!warned) {
        cli::cli_warn("{message}: {conditionMessage(w)}")
        if (!is.null(warn_setter)) warn_setter()
      }
      invokeRestart("muffleWarning")
    }
  )
}

#' BRCore ggplot2 Theme
#'
#' A consistent \pkg{ggplot2} theme used across all BRCore plotting functions.
#' Built on top of \code{\link[ggplot2]{theme_classic}} with standardised font
#' sizes and element dimensions.
#'
#' @param base_size Numeric. Base font size in points (default \code{11}).
#'   All other sizes are derived from this value.
#' @param axis_text_angle Numeric. Rotation angle for x-axis tick labels
#'   (default \code{0}). Set to e.g. \code{45} or \code{90} when labels overlap.
#' @param legend_position Character or numeric vector passed to
#'   \code{legend.position} (default \code{"right"}).
#' @param dashed_grid Logical. When \code{TRUE}, adds a dashed horizontal major
#'   grid line (used by rarefaction metric plots). Default \code{FALSE}.
#' @param extra_themes A named list of additional \pkg{ggplot2}
#'   \code{\link[ggplot2]{theme}} objects to compose onto the base theme.
#'   Useful for one-off per-plot overrides without breaking the shared theme.
#'   Example: \code{list(legend = theme(legend.title = element_blank()))}.
#'
#' @details
#' Size conventions derived from \code{base_size}:
#' \itemize{
#'   \item \strong{Plot title} – \code{base_size + 1}, bold, centred.
#'   \item \strong{Plot subtitle} – \code{base_size - 2}, centred.
#'   \item \strong{Axis titles} – \code{base_size}, bold.
#'   \item \strong{Axis text} – \code{base_size - 1}.
#'   \item \strong{Legend title} – \code{base_size - 1}, bold.
#'   \item \strong{Legend text} – \code{base_size - 2}.
#'   \item \strong{Strip text (facets)} – \code{base_size}, bold.
#'   \item \strong{Legend keys} – 0.4 cm × 0.4 cm.
#' }
#'
#' @return A \pkg{ggplot2} \code{\link[ggplot2]{theme}} object that can be added
#'   to any \code{ggplot} with \code{+}.
#'
#' @examples
#' library(ggplot2)
#' ggplot(mtcars, aes(wt, mpg)) +
#'   geom_point() +
#'   .brcore_theme()
#'
#' # With dashed grid and an extra per-plot override
#' ggplot(mtcars, aes(wt, mpg)) +
#'   geom_point() +
#'   .brcore_theme(
#'     dashed_grid  = TRUE,
#'     extra_themes = list(no_legend = theme(legend.position = "none"))
#'   )
#'
#' @importFrom ggplot2 theme_classic theme element_text element_line
#'   element_blank element_rect
#' @importFrom grid unit
#'
#' @noRd
#' @keywords internal
.brcore_theme <- function(
  base_size = 11,
  axis_text_angle = 0,
  legend_position = "right",
  dashed_grid = FALSE,
  extra_themes = list()
) {
  base <- theme_classic(base_size = base_size) +
    theme(
      # Titles ----
      plot.title = element_text(
        hjust = 0.5,
        size = base_size + 1,
        face = "bold",
        colour = "black"
      ),
      plot.subtitle = element_text(
        hjust = 0.5,
        size = base_size - 2,
        colour = "black"
      ),

      # Axes ----
      axis.title = element_text(
        size = base_size,
        face = "plain",
        colour = "black"
      ),
      axis.text = element_text(
        size = base_size - 1,
        colour = "black"
      ),
      axis.text.x = element_text(
        angle = axis_text_angle,
        hjust = if (axis_text_angle == 0) 0.5 else 1,
        vjust = if (axis_text_angle == 0) 0.5 else 1,
        size = base_size - 1
      ),
      axis.line = element_line(colour = "black"),
      axis.ticks = element_line(colour = "black"),

      # Legend ----
      legend.position = legend_position,
      legend.background = element_rect(fill = "white", colour = NA),
      legend.key = element_rect(fill = "white", colour = NA),
      legend.key.height = grid::unit(0.4, "cm"),
      legend.key.width = grid::unit(0.4, "cm"),
      legend.title = element_text(
        size = base_size - 1,
        face = "bold",
        colour = "black"
      ),
      legend.text = element_text(
        size = base_size - 2,
        colour = "black"
      ),

      # Facet strips ----
      strip.background = element_blank(),
      strip.text = element_text(
        size = base_size,
        face = "bold",
        colour = "black"
      ),

      # Panel ----
      panel.background = element_rect(fill = "white", colour = NA),
      panel.border = element_rect(fill = NA, colour = "black", linewidth = 1)
    )

  # Optional dashed horizontal grid (e.g. rarefaction metric plots)
  if (dashed_grid) {
    base <- base +
      theme(
        panel.grid.major.y = element_line(
          color = "grey",
          linetype = "dashed",
          linewidth = 0.5
        )
      )
  }

  # Compose any extra per-plot theme overrides
  for (t in extra_themes) {
    base <- base + t
  }

  base
}

# Vignettes that are computationally intensive have been precompiled:
# Must manually move image files from BRCore/figure to BRCore/vignettes/ after knit

library(knitr)
library(here)
library(devtools)

devtools::load_all()
knit("vignettes/BRCore-vignette.Rmd.orig", "vignettes/BRCore-vignette.Rmd")

# Moving files
figure_files <- list.files(here::here("images"), full.names = TRUE)
figure_dir <- here::here("images")

# Move files to vignettes and doc directories
file.copy(figure_files, here::here("vignettes/images"), recursive = TRUE)
file.copy(figure_files, here::here("doc/images"), recursive = TRUE)

# Cleanup
file.remove(figure_files, figure_dir)

# Build static vignette
build_vignettes()

# Extract R code from vignette
knitr::purl(
  "vignettes/BRCore-vignette.Rmd.orig",
  output = "vignettes/BRCore-vignette.R"
)

#' Fit a Neutral Model to Microbial Community Data
#'
#' The function fits the neutral distribution model developed by Sloan et al.
#' (2006), and implemented in R by Burns et al. (2015), to an OTU/ASV table and
#' returns several goodness of fit statistics alongside a data.frame
#' with predicted occurrence frequencies for each OTU/ASV based on their
#' abundance in the metacommunity for plotting the abundance-occupancy
#' distribution. In addition, this function identified core and not-core
#' OTU/ASVs that are neutral (i.e. stochastically or randomly distributed)
#' above (i.e. interpreted as deterministically or predictably selected) and
#' below (i.e. interpreted as dispersally limited) the model predictions.
#'
#' @param otu_table A community count matrix (or OTU table) with samples as
#' rows and OTU/ASVs as columns..
#' @param core_set Character vector of core OTU/ASVs IDs (must match column
#' names of `otu_table`).
#' @param abundance_occupancy data.frame (or tibble) with OTU/ASVs names,
#'  occupancy (`otu_occ`),
#'  and mean relative abundance (`otu_rel`) as generated by
#' \link[=identify_core]{identify_core()}.
#'
#' @return A list with:
#'   - `goodness_of_fit`: one-row tibble of summary stats (plus `above.pred`,
#'  `below.pred`) - `model_prediction`: tibble with `abundance_occupancy`
#' joined to per-taxon predictions
#'
#' @references
#' Sloan, W. T., Lunn, M., Woodcock, S., Head, I. M., Nee, S., & Curtis, T. P.
#' (2006). Quantifying the roles of immigration and chance in shaping
#' prokaryote community structure. Environmental Microbiology, 8(4), 732–740.
#' <doi:10.1111/j.1462-2920.2005.00956.x>
#'
#' Burns, A. R., Stephens, W. Z., Stagaman, K., Wong, S., Rawls, J. F.,
#' Guillemin, K., & Bohannan, B. J. M. (2016). Contribution of neutral
#' processes to the assembly of gut microbial communities in the zebrafish over
#' host development. The ISME Journal, 10(3), 655–664.
#' <doi:10.1038/ismej.2015.142>
#'
#' Shade A, Stopnisek N (2019) Abundance-occupancy distributions to prioritize
#' plant core microbiome membership. Current Opinion in Microbiology, 49:50-58
#' <doi:10.1016/j.mib.2019.09.008>
#'
#' @seealso [plot_neutral_model()]
#'
#' @examples
#' library(BRCore)
#' data("switchgrass_core", package = "BRCore")
#'
#' switchgrass_core_fit <- fit_neutral_model(
#'   otu_table = switchgrass_core$otu_table,
#'   core_set = switchgrass_core$increase_core,
#'   abundance_occupancy = switchgrass_core$abundance_occupancy
#' )
#'
#' str(switchgrass_core_fit)
#' switchgrass_core_fit$goodness_of_fit
#' switchgrass_core_fit$model_prediction |>
#' head()
#'
#' @importFrom stats confint pbeta pbinom ppois dnorm AIC BIC coef
#' @importFrom stats4 mle
#' @importFrom minpack.lm nlsLM
#' @importFrom Hmisc binconf
#'
#' @export
fit_neutral_model <- function(
  otu_table,
  core_set,
  abundance_occupancy
) {
  # input checks ----
  if (!is.matrix(otu_table) && !is.data.frame(otu_table)) {
    cli::cli_abort(
      "`otu_table` must be a matrix or data.frame (samples x taxa)."
    )
  }

  spp <- as.matrix(t(otu_table))

  core_taxa <- as.character(if (is.null(core_set)) character() else core_set)
  if (!length(core_taxa)) {
    cli::cli_abort(
      "`core_set` must be a non-empty character vector of ASV/OTU IDs."
    )
  }

  if (!("otu" %in% colnames(abundance_occupancy))) {
    cli::cli_abort(
      "`abundance_occupancy` must contain a column named {.field otu} to join predictions."
    )
  }

  # add membership (plot-friendly labels) ----
  occ_abun <- abundance_occupancy |>
    dplyr::mutate(
      membership = ifelse(.data$otu %in% core_taxa, "Core", "Not core")
    )

  # fit model ----
  obs.np <- sncm.fit(spp, core_taxa, stats = FALSE, pool = NULL)
  sta.np <- suppressMessages(
    sncm.fit(spp, core_taxa, stats = TRUE, pool = NULL)
  ) # Avoid printing of model fitting messages again, see sncm.fit() for details

  if (is.null(rownames(obs.np))) {
    cli::cli_abort(
      "Per-taxon predictions must have rownames as ASV/OTU IDs."
    )
  }
  obs_tbl <- tibble::as_tibble(obs.np, rownames = "otu")

  # ASV/OTUs in/out prediction intervals ----
  obs_tbl$fit_class <- "As predicted"
  obs_tbl[
    which(obs_tbl$freq < obs_tbl$pred.lwr),
    "fit_class"
  ] <- "Below prediction"
  obs_tbl[
    which(obs_tbl$freq > obs_tbl$pred.upr),
    "fit_class"
  ] <- "Above prediction"
  obs_tbl[which(is.na(obs_tbl$freq)), "fit_class"] <- "NA"
  fit_table <- as.data.frame(dplyr::left_join(occ_abun, obs_tbl, by = "otu"))

  # compute above/below proportions ----
  sta.np$above.pred <- sum(obs_tbl$freq > (obs_tbl$pred.upr), na.rm = TRUE) /
    sta.np$Richness
  sta.np$below.pred <- sum(obs_tbl$freq < (obs_tbl$pred.lwr), na.rm = TRUE) /
    sta.np$Richness
  fit_res <- as.data.frame(sta.np)
  # colnames(fit_res) <- "statistics"

  # return ----
  out <- list(
    goodness_of_fit = fit_res,
    model_prediction = fit_table
  )
  class(out) <- c("fit_neutral_model", class(out))

  cli::cli_alert_success("Model fitting complete!")

  out
}

test_that("GlobalPatterns pipeline -> fit_neutral_model basic structure", {
  skip_on_cran()
  skip_if_not(exists("identify_core"), "identify_core() not found")
  skip_if_not(exists("fit_neutral_model"), "fit_neutral_model() not found")

  data("switchgrass", package = "BRCore", envir = environment())
  skip_if_not(exists("switchgrass"), "switchgrass dataset not found")

  switchgrass_core <- identify_core(
    physeq_obj = switchgrass,
    priority_var = "sampling_date",
    increase_value = 0.02,
    abundance_weight = 0,
    seed = 123
  )

  switchgrass_core_fit <- fit_neutral_model(
    otu_table = switchgrass_core$otu_table,
    core_set = switchgrass_core$increase_core,
    abundance_occupancy = switchgrass_core$abundance_occupancy
  )

  expect_type(switchgrass_core_fit, "list")
  expect_true(all(
    c("model_prediction", "goodness_of_fit") %in%
      names(switchgrass_core_fit)
  ))

  if (exists("plot_neutral_model")) {
    p <- plot_neutral_model(switchgrass_core_fit)
    expect_s3_class(p, "ggplot")
    expect_silent(ggplot2::ggplot_build(p))
  }
})

test_that("fit_neutral_model errors on invalid otu_table type", {
  expect_error(
    fit_neutral_model(
      otu_table = list(a = 1, b = 2),
      core_set = "ASV1",
      abundance_occupancy = data.frame(otu = "ASV1")
    ),
    "matrix or data.frame"
  )
})

test_that("fit_neutral_model errors on NULL core_set", {
  otu <- matrix(
    1:12,
    nrow = 3,
    dimnames = list(
      c("S1", "S2", "S3"),
      c("ASV1", "ASV2", "ASV3", "ASV4")
    )
  )
  expect_error(
    fit_neutral_model(
      otu_table = otu,
      core_set = NULL,
      abundance_occupancy = data.frame(otu = c("ASV1", "ASV2"))
    ),
    "non-empty character vector"
  )
})

test_that("fit_neutral_model errors on empty core_set", {
  otu <- matrix(
    1:12,
    nrow = 3,
    dimnames = list(
      c("S1", "S2", "S3"),
      c("ASV1", "ASV2", "ASV3", "ASV4")
    )
  )
  expect_error(
    fit_neutral_model(
      otu_table = otu,
      core_set = character(0),
      abundance_occupancy = data.frame(otu = c("ASV1", "ASV2"))
    ),
    "non-empty character vector"
  )
})

test_that("fit_neutral_model errors when abundance_occupancy lacks 'otu' column", {
  otu <- matrix(
    1:12,
    nrow = 3,
    dimnames = list(
      c("S1", "S2", "S3"),
      c("ASV1", "ASV2", "ASV3", "ASV4")
    )
  )
  expect_error(
    fit_neutral_model(
      otu_table = otu,
      core_set = c("ASV1"),
      abundance_occupancy = data.frame(taxon = c("ASV1", "ASV2"))
    ),
    "otu"
  )
})

#' Plot Abundance-Occupancy Curve and Display the Core Taxa
#'
#' Creates a scatter plot showing the relationship between mean relative abundance
#' and occupancy (occurrence frequency) of taxa, with core taxa highlighted.
#'
#' @param core_result A list object returned by \code{\link{identify_core}},
#'   containing at minimum:
#'   \itemize{
#'     \item \code{occupancy_abundance}: A data frame with columns \code{otu},
#'       \code{otu_rel} (mean relative abundance), and \code{otu_occ} (occupancy).
#'     \item \code{elbow_core}: Character vector of OTU IDs identified as core
#'       using the "elbow" method.
#'     \item \code{increase_core}: Character vector of OTU IDs identified as core
#'       using the "increase" method.
#'   }
#' @param core_set Character string specifying which core set to highlight.
#'   Must be either "elbow" or "increase" (Default elbow).
#'
#' @details The function creates a scatter plot where each point represents a
#'   taxon (i.e. ASV or OTU). Core taxa (as defined by the selected method) are
#'   shown in red, while non-core taxa are shown in grey. The plot uses a log10
#'   scale for abundance to better visualize the full range of abundances typically
#'    found in microbiome data.
#'
#' @return A ggplot object showing the abundance-occupancy plot with core taxa
#'   highlighted in red and non-core taxa in grey. The x-axis shows log10-transformed
#'   mean abundance and the y-axis shows occupancy (0-1).
#'
#' @seealso [plot_core_distribution()] and [identify_core()]
#'
#' @examples
#' library(BRCore)
#'
#' data("switchgrass_core", package = "BRCore")
#'
#' p <- plot_abundance_occupancy(
#'   core_result = switchgrass_core,
#'   core_set = "increase"
#' )
#' print(p)
#'
#' @importFrom dplyr mutate
#' @importFrom ggplot2 ggplot aes geom_point scale_fill_manual theme_classic
#'   theme element_text element_rect element_blank labs alpha after_scale guides guide_legend
#' @importFrom grid unit
#'
#' @export
plot_abundance_occupancy <- function(core_result, core_set = "elbow") {
  # Validate core_set parameter
  if (!core_set %in% c("elbow", "increase")) {
    stop("core_set must be either 'elbow' or 'increase'")
  }

  # Select appropriate core OTU set
  if (core_set == "elbow") {
    core_otus <- core_result$elbow_core
  } else if (core_set == "increase") {
    core_otus <- core_result$increase_core
  }

  # Prepare data frame with membership classification
  whole_df <- core_result$abundance_occupancy |>
    mutate(
      membership = ifelse(otu %in% core_otus, "Core", "Not core")
    ) |>
    mutate(membership = as.factor(membership))

  # Create plot
  p <- whole_df |>
    ggplot(aes(
      x = log10(otu_rel),
      y = otu_occ,
      fill = membership
    )) +
    geom_point(
      shape = 21,
      size = 1.5,
      alpha = 0.9,
      aes(color = after_scale(fill))
    ) +
    scale_fill_manual(
      values = c("Core" = "#CC2D35", "Not core" = "grey")
    ) +
    .brcore_theme(
      legend_position = c(0.98, 0.02),
      extra_themes = list(
        theme(
          legend.justification = c("right", "bottom"),
          legend.background = element_rect(
            fill = alpha("white", 0.7),
            color = NA
          ),
          legend.title = element_blank()
        )
      )
    ) +
    labs(
      title = "Abundance-Occupancy curve",
      x = "Log10(mean abundance)",
      y = "Occupancy"
    )

  # change legend shape to avoid confusion with the scatter points.
  p <- p +
    guides(
      fill = guide_legend(
        override.aes = list(
          shape = 22,
          size = 5,
          colour = NA,
          stroke = 0
        )
      )
    )

  return(p)
}

# tests/testthat/test-plot_abundance_occupancy.R
test_that("plot_abundance_occupancy returns a ggplot and builds", {
  skip_if_not_installed("ggplot2")

  abundance_occupancy <- data.frame(
    otu = paste0("OTU", 1:100),
    otu_occ = round(runif(100, min = 0.05, max = 1), 8),
    otu_rel = round(runif(100, min = 0.0002, max = 0.05), 8),
    membership = sample(
      c("Core", "Not core"),
      size = 100,
      replace = TRUE,
      prob = c(0.4, 0.6)
    )
  )

  core_result_example <- list(
    abundance_occupancy = abundance_occupancy,
    increase_core = c("OTU3", "OTU100", "OTU15")
  )

  p <- plot_abundance_occupancy(
    core_result = core_result_example,
    core_set = "increase"
  )

  expect_s3_class(p, "ggplot")
  expect_silent(ggplot2::ggplot_build(p))
})

#' 16S amplicon dataset from Switchgrass (Panicum virgatum)
#'
#' @description A phyloseq containing 706 97% sequence similarity OTUs
#' (Operational Taxonomic Units) across 43 samples.
#'
#' @usage data("switchgrass")
#' @docType data
#' @format An object class "phyloseq".
#'
#' @references Shade A, Stopnisek N (2019) Abundance-occupancy distributions to
#' prioritize plant core microbiome membership. Current Opinion in
#' Microbiology, 49:50-58 <doi:10.1016/j.mib.2019.09.008>
#'
#' @source Internal test dataset.
"switchgrass"

#' Identified core microbiome members for the switchgrass dataset
#'
#' @description A phyloseq containing the identified core microbiome members
#' for the switchgrass dataset. This object is used for BRCore function
#' examples.
#' It was generated by running the following on BRCore 2.0.2 (2026-04-30):
#' ```R
#' switchgrass_core <- identify_core(
#'   physeq_obj     = switchgrass,
#'   priority_var   = "sampling_date",
#'   increase_value = 0.02,
#'   seed           = 092825
#' )
#' ```
#'
#' @usage data("switchgrass_core")
#' @docType data
#' @format An object class "phyloseq".
#'
#' @references Shade A, Stopnisek N (2019) Abundance-occupancy distributions to
#' prioritize plant core microbiome membership. Current Opinion in
#' Microbiology, 49:50-58 <doi:10.1016/j.mib.2019.09.008>
#'
#' @source Internal test dataset.
"switchgrass_core"

#' 16S amplicon dataset from yellow monkeyflower (Mimulus guttatus)
#'
#' @description A phyloseq containing 11085 ASVs (Amplicon Sequence Variants)
#'  across 39 samples.
#'
#' @usage data("mimulus")
#' @docType data
#' @format An object class "phyloseq".
#'
#' @references Shade A, Stopnisek N (2019) Abundance-occupancy distributions to
#' prioritize plant core microbiome membership. Current Opinion in
#' Microbiology, 49:50-58 <doi:10.1016/j.mib.2019.09.008>
#'
#' @source Internal test dataset.
"mimulus"

#' 16S amplicon dataset from common bean (Phaseolus vulgaris)
#'
#' @description A phyloseq containing 24885 ASVs (Amplicon Sequence Variants)
#' across 30 samples.
#'
#' @usage data("bean")
#' @docType data
#' @format An object class "phyloseq".
#'
#' @references Shade A, Stopnisek N (2019) Abundance-occupancy distributions to
#' prioritize plant core microbiome membership. Current Opinion in
#' Microbiology, 49:50-58 <doi:10.1016/j.mib.2019.09.008>
#'
#'
#' @source Internal test dataset.
"bean"

#' 16S amplicon dataset from the GLBRC Biofuel Cropping System Experiment
#' (BCSE) at Michigan State University, Kellogg Biological Station
#'
#' @description A phyloseq containing 2861 97% sequence similarity OTUs
#' (Operational Taxonomic Units) across 50 samples. Samples are for the leaf
#' microbiome only.
#'
#' @usage data("bcse")
#' @docType data
#' @format An object class "phyloseq".
#'
#' @references Haan, N. L., Benucci, G. N. M., Fiser, C. M., Bonito, G., &
#' Landis, D. A. (2023). Contrasting effects of bioenergy crops on
#' biodiversity. Science Advances, 9(38), eadh7960.
#' <doi:10.1126/sciadv.adh7960>

#'
#' @source Internal test dataset.
"bcse"

utils::globalVariables(
  c(
    ".",
    "sample_id",
    "comparison",
    "BC",
    "otu",
    "membership",
    "fill",
    "rank_num",
    "proportionBC",
    "relabun",
    "pred.upr",
    "pred.lwr",
    "grp",
    "fit_class",
    "otu_id",
    "goods_cov",
    "outlier",
    "iter",
    "freq.pred",
    "otu_occ",
    "otu_rel",
    "spatial_weight",
    "time_freq",
    "read_num"
  )
)

utils::globalVariables(c("%>%", "%T>%"))

#' Plot a fitted Neutral Model to Microbial Community Data
#'
#' This function plots ASV/OTUs log abundances into a fitted neutral model of
#' microbial abundance-occupancy distribution. ASV/OTUs that are *Core*, *As
#' predicted* (i.e. Neutral), *Below* and *Above* model predictions are drawn
#' with distinct point colors, see details.
#'
#' @param fit_result A list-like object returned by \link[=fit_neutral_model]{fit_neutral_model()}.
#'
#' @return A \link[ggplot2:ggplot]{ggplot2} object.
#'
#' @details
#' Points are split into four groups for display:
#' \itemize{
#'   \item \strong{Not core (as predicted)} – background points;
#'   \item \strong{Core (as predicted)} – core taxa whose occupancy matches the model;
#'   \item \strong{Core (above prediction)} – core taxa above the 95\% CI;
#'   \item \strong{Core (below prediction)} – core taxa below the 95\% CI.
#' }
#'
#' @return
#' A \link[ggplot2:ggplot]{ggplot} object with:
#' \itemize{
#'   \item \emph{x}-axis: \code{log10(mean abundance)} (\code{log10(otu_rel)});
#'   \item \emph{y}-axis: \code{Occupancy} (\code{otu_occ});
#'   \item Points colored by membership/fit class as described above;
#'   \item Neutral-model curve (solid) with 95\% CI (grey, twodash);
#'   \item An inset white label reporting \emph{R}\eqn{^2} and \emph{m} taken
#'         directly from \code{fit_result$goodness_of_fit}.
#' }
#'
#' The function does not recompute statistics; it only visualizes the supplied
#' predictions and metrics.
#'
#' @examples
#' library(BRCore)
#' data("switchgrass_core", package = "BRCore")
#'
#' switchgrass_core_fit <- fit_neutral_model(
#'   otu_table = switchgrass_core$otu_table,
#'   core_set = switchgrass_core$increase_core,
#'   abundance_occupancy = switchgrass_core$abundance_occupancy
#' )
#'
#' p <- plot_neutral_model(switchgrass_core_fit)
#' print(p)
#'
#' @seealso \code{\link{fit_neutral_model}}
#'
#' @importFrom dplyr filter bind_rows mutate
#' @importFrom ggplot2 ggplot aes geom_point geom_line labs theme_classic theme
#' @importFrom ggplot2 element_text element_rect element_blank annotate scale_fill_manual
#' @importFrom ggplot2 after_scale guides guide_legend
#' @importFrom grid unit
#' @importFrom cli cli_abort

#' @export
plot_neutral_model <- function(fit_result) {
  # Export dataframes
  obs1 <- as.data.frame(fit_result$model_prediction)
  obs2 <- as.data.frame(fit_result$goodness_of_fit)

  # pre-split - all 6 combinations
  pts_notcore_as <- filter(
    obs1,
    membership == "Not core",
    fit_class == "As predicted"
  )
  pts_notcore_above <- filter(
    obs1,
    membership == "Not core",
    fit_class == "Above prediction"
  )
  pts_notcore_below <- filter(
    obs1,
    membership == "Not core",
    fit_class == "Below prediction"
  )
  pts_core_as <- filter(
    obs1,
    membership == "Core",
    fit_class == "As predicted"
  )
  pts_core_above <- filter(
    obs1,
    membership == "Core",
    fit_class == "Above prediction"
  )
  pts_core_below <- filter(
    obs1,
    membership == "Core",
    fit_class == "Below prediction"
  )

  # Legend grouping labels
  all_pts <- bind_rows(
    bind_rows(pts_notcore_as, pts_notcore_above, pts_notcore_below) |>
      mutate(grp = "Not core"),
    mutate(pts_core_as, grp = "Core (as predicted)"),
    mutate(pts_core_above, grp = "Core (above prediction)"),
    mutate(pts_core_below, grp = "Core (below prediction)")
  )

  # if still empty, bail nicely
  if (nrow(all_pts) == 0) {
    cli::cli_abort(
      "Nothing to plot: no rows in model_prediction after filtering (check p>0 and membership/fit_class)."
    )
  }

  # Extract R2 and m (works for column- or rowname-shaped tables)
  R2 <- if ("Rsqr" %in% names(obs2)) {
    obs2$Rsqr[1]
  } else {
    suppressWarnings(as.numeric(obs2["Rsqr", 1]))
  }
  mV <- if ("m" %in% names(obs2)) {
    obs2$m[1]
  } else {
    suppressWarnings(as.numeric(obs2["m", 1]))
  }

  # Compute a spot just ABOVE the legend (bottom-right)
  xr <- range(log10(all_pts$otu_rel), na.rm = TRUE)
  dx <- diff(xr)
  yr <- range(all_pts$otu_occ, na.rm = TRUE)
  dy <- diff(yr)
  box_x <- xr[1] + 0.95 * dx
  box_y <- yr[1] + 0.12 * dy # raise/lower if you want different spacing

  p <- ggplot(all_pts, aes(x = log10(otu_rel), y = otu_occ, fill = grp)) +
    geom_point(
      shape = 21,
      size = 1.5,
      alpha = 0.9,
      aes(color = after_scale(fill))
    ) +
    scale_fill_manual(
      values = c(
        "Not core" = "grey",
        "Core (as predicted)" = "#CC2D35",
        "Core (above prediction)" = "#0072B2",
        "Core (below prediction)" = "#7FB800"
      ),
      breaks = c(
        "Not core",
        "Core (as predicted)",
        "Core (above prediction)",
        "Core (below prediction)"
      )
    ) +
    # Keep lines but remove their legends
    geom_line(
      data = obs1,
      aes(x = log10(p), y = freq.pred),
      color = "blue",
      linewidth = 0.8,
      alpha = 0.25,
      inherit.aes = FALSE,
      show.legend = FALSE
    ) +
    geom_line(
      data = obs1,
      aes(x = log10(p), y = pred.upr),
      color = "black",
      linewidth = 0.8,
      linetype = "twodash",
      alpha = 0.25,
      inherit.aes = FALSE,
      show.legend = FALSE
    ) +
    geom_line(
      data = obs1,
      aes(x = log10(p), y = pred.lwr),
      color = "black",
      linewidth = 0.8,
      linetype = "twodash",
      alpha = 0.25,
      inherit.aes = FALSE,
      show.legend = FALSE
    ) +
    guides(color = "none", linetype = "none") +
    .brcore_theme(
      legend_position = c(0.98, 0.02),
      extra_themes = list(theme(
        legend.justification = c("right", "bottom"),
        legend.background = element_rect(
          fill = alpha("white", 0.7),
          color = NA
        ),
        legend.title = element_blank()
      ))
    ) +
    annotate(
      "label",
      x = box_x,
      y = box_y,
      label = paste0(
        "atop(italic(R)^2==",
        sprintf("%.2f", R2),
        ",italic(m)==",
        sprintf("%.3f", mV),
        ")"
      ),
      parse = TRUE,
      hjust = 1,
      vjust = -0.75,
      size = 4.5,
      fill = "white",
      alpha = 0.9,
      lineheight = 1.05
    ) +
    labs(
      title = "Neutral model",
      x = "Log10(mean abundance)",
      y = "Occupancy"
    ) +
    scale_y_continuous(labels = scales::label_percent(accuracy = 1))
  # change legend shape to avoid confusion with the scatter points.
  p <- p +
    guides(
      fill = guide_legend(
        override.aes = list(
          shape = 22,
          size = 4,
          colour = NA,
          stroke = 0
        )
      )
    )

  return(p)
}

# tests/testthat/test-plot_neutral_model.R
test_that("plot_neutral_model returns a ggplot and builds", {
  skip_if_not_installed("ggplot2")

  fit_result <- list(
    model_prediction = data.frame(
      otu_rel = c(1e-4, 5e-4, 1e-3, 2e-3, 1e-2),
      otu_occ = c(0.10, 0.20, 0.40, 0.60, 0.80),
      membership = c("Not core", "core", "core", "Not core", "core"),
      fit_class = c(
        "As predicted",
        "Above prediction",
        "Below prediction",
        "As predicted",
        "As predicted"
      ),
      p = c(1e-4, 5e-4, 1e-3, 2e-3, 1e-2),
      freq.pred = c(0.12, 0.25, 0.38, 0.62, 0.79),
      pred.lwr = c(0.05, 0.15, 0.30, 0.50, 0.70),
      pred.upr = c(0.20, 0.35, 0.50, 0.75, 0.90)
    ),
    goodness_of_fit = data.frame(Rsqr = 0.83, m = 0.159)
  )

  p <- plot_neutral_model(fit_result)
  expect_s3_class(p, "ggplot")
  expect_silent(ggplot2::ggplot_build(p))
})

# tests/testthat/setup.R

rda_path <- test_path("test_sets/test_vignette_data.rda")

# Only regenerate if the .rda is missing or if running on CI
is_check <- nzchar(Sys.getenv("_R_CHECK_PACKAGE_NAME_"))
is_not_cran <- identical(Sys.getenv("NOT_CRAN"), "true")
is_ci <- nzchar(Sys.getenv("CI"))

needs <- c("devtools", "phyloseq", "tidyverse", "viridis")
can_regenerate <- all(vapply(
  needs,
  requireNamespace,
  logical(1),
  quietly = TRUE
))


# Does not regenerate on CRAN or during R CMD check, but does regenerate during devtools::test() if the .rda is missing
if (is_not_cran && !is_check) {
  if (!file.exists(rda_path)) {
    source(test_path("test_sets/regenerate_test_data.R"))
  }
}

# Regenerate test data if running on CI
if (is_ci && can_regenerate) {
  rda_path <- file.path(
    tempdir(),
    test_path("test_sets/test_vignette_data.rda")
  )
  options(brcore_test_data_path = rda_path)
  source(test_path("test_sets/regenerate_test_data.R"), local = TRUE)
}

#' Identify Core Microbiome Using Bray-Curtis Similarity
#' biological samples. Core taxa are selected using either a "last % increase"
#' or "elbow" method implementing the method developed by Shade
#' and Stopnisek (2019) Curr Opin Microbiol, see below for details.
#'
#' @param physeq_obj A `phyloseq` object with at least `otu_table` and
#' `sample_data`.
#' If the `phyloseq` object is already rarefied (i.e., all samples have the
#' same sequencing depth), you can omit `rarefied_list` and the function will
#' automatically use the OTU table from `physeq_obj` as a single iteration. If
#' the data is not rarefied, you must provide a `rarefied_list` generated by
#' `multi_rarefy`. Alternatively you can use `update_otu_table` to replace the
#' OTU table with a rarefied version before running `identify_core`.
#' @param rarefied_list A list of data frames, each representing a rarefied OTU
#' table (taxa x samples) generated by `multi_rarefy`. Required if `physeq_obj`
#' is not already rarefied. If `physeq_obj` is rarefied, this can be omitted
#' and the function will use the OTU table from `physeq_obj` as a single
#' iteration.
#' @param priority_var The column name in the `sample_data` (e.g.
#' sampling_date", "site") that is used for prioritizing the core microbiome.
#' @param increase_value Increase value (numeric, scalar) used in the
#' calculation (default 0.02) for "increase". The "elbow" is always calculated
#' and returned as \code{elbow_core} (see below for details).
#' @param abundance_weight Numeric in `[0,1]`; how much to weight mean relative
#' abundance in the ranking score. `0` (default) uses occupancy/composite only.
#' `1` ranks purely by abundance. Values in between blend the two (e.g.,
#' abundance_weight = 0.3 gives 70% occupancy/composite + 30% abundance).
#' @param max_otus Optional integer to limit analysis to the top N ranked OTUs.
#' If NULL (default), all OTUs are analyzed. Useful for large datasets
#' (>5000 OTUs)
#' @param depth_level Integer. The sequencing depth used for normalization in
#' Bray-Curtis calculations. If data is rarefied, this is automatically set
#' to the rarefaction depth. For unrarefied data, samples with depth below
#' this threshold are excluded from pairwise comparisons.
#' @param seed Optional integer to set the RNG seed for reproducibility.
#'
#' @return A list with:
#' \itemize{
#'   \item \code{bray_curtis_ranked} tibble with `rank`, mean Bray-Curtis
#'  similarity across sample pairs (`MeanBC`) at each cumulative `rank`,
#'  normalized proportion (`proportionBC`), the multiplicative `IncreaseBC`,
#' and the elbow metric (`elbow_slope_diffs`).
#' (`proportionBC`), the multiplicative `IncreaseBC`, and the elbow metric
#' (`elbow_slope_diffs`).
#'   \item \code{otu_ranked} tibble with ranked OTU/ASVs .
#'   \item \code{abundance_occupancy} tibble with OTU/ASVs names, occupancy
#'      (`otu_occ`), and mean relative abundance (`otu_rel`).
#'   \item \code{priority_var} character, the variable used for prioritizing
#' the core.
#'   \item \code{elbow} core set identified by elbow method (integer).
#'   \item \code{bc_increase} core set identified by last % BC-increase
#' (integer).
#'   \item \code{increase_value} increase value (numeric, scalar) used in the
#' calculation (e.g. 0.02).
#'   \item \code{elbow_core} core OTU/ASVs using elbow method (character
#' vector).
#'   \item \code{increase_core} core OTU/ASVs using last % BC-increase method
#'  (character vector).
#'   \item \code{otu_table} otu_table counts (otu x samples) used (data.frame).
#'   \item \code{sample_metadata} samples metadata (data.frame).
#'   \item \code{taxonomy_table} taxonomy if present (data.frame); otherwise
#'  NULL.
#' }
#'
#' @details
#' The core set is defined using two separate methods:
#'
#' The function rank OTU/ASVs by occupancy (optionally with abundance
#' weighting: `rank_score = (1 - weight) * rank + weight * scaled_abundance`,
#' where scaled_abundance is mean relative abundance rescaled to `[0,1]`). For
#' each `k = 1..K`, recompute `S_k` as the mean Bray-Curtis similarity across
#' all sample pairs using only the first `k` ranked OTUs; when `k = K`, this
#' yields `S_K`, the value computed with all OTUs. Normalizing by `S_K`
#' gives `C_k = S_k / S_K`.
#'
#' The **elbow** is the point of diminishing returns: for each `k`, compare the
#' average *left* slope `(S_k - S_1) / (k - 1)` to the average *right* slope
#' `(S_K - S_k) / (K - k)`, and choose the `k` that maximizes `(left - right)`.
#'
#' The **last percent Bray-Curtis increase** method uses the same accumulation
#' curve, examine the multiplicative step when adding the `k-th` OTU:
#' `Increase_k = S_k / S_{k-1}` (equivalently, `Increase_k = C_k / C_{k-1}`).
#' Choose the largest `k` such that `Increase_k >= 1 + p`, where `p` is your
#' chosen percent threshold (increase_value; recommended `p >= 0.02` or `2%`).
#' This selects the final rank for which adding one more taxon still increases
#' the explained Bray-Curtis similarity by at least `p`.
#'
#' @references Shade A, Stopnisek N (2019) Abundance-occupancy
#' distributions to prioritize plant core microbiome membership. Current
#' Opinion in Microbiology, 49:50-58
#' <doi:10.1016/j.mib.2019.09.008>
#'
#' @section Dependencies:
#' Requires \pkg{phyloseq}, \pkg{dplyr}, \pkg{tidyr}, \pkg{tibble}, \pkg{rlang},
#' and \pkg{vegan}.
#'
#' @seealso [multi_rarefy()], [plot_identified_core()], and
#' [plot_core_distribution()]
#'
#' @examples
#' \donttest{
#' library(BRCore)
#'
#' # With rarefied data
#' res <- identify_core(
#'   physeq_obj     = switchgrass,
#'   priority_var   = "sampling_date",
#'   increase_value = 0.02,
#'   seed           = 091825
#' )
#'
#' str(res)
#'
#' # With unrarefied data (requires multi_rarefy step)
#' rarefied_list <- multi_rarefy(
#'   physeq_obj = bcse,
#'   depth_level = 1000,
#'   num_iter = 3,
#'   .as = "list",
#'   set_seed = 7642
#' )
#'
#'
#' res_rare <- identify_core(
#'   physeq_obj = bcse,
#'   rarefied_list = rarefied_list,
#'   priority_var = "Crop",
#'   increase_value = 0.02,
#'   seed = 091825
#' )
#'
#' str(res_rare)
#' }
#'
#' @importFrom phyloseq sample_sums taxa_are_rows otu_table sample_data
#' @importFrom phyloseq tax_table
#' @importFrom dplyr left_join group_by summarise transmute arrange desc mutate
#' @importFrom dplyr n last select slice_head
#' @importFrom tidyr pivot_longer
#' @importFrom tibble rownames_to_column column_to_rownames
#' @importFrom rlang ensym as_name .data
#' @importFrom vegan decostand
#' @importFrom cli cli_text cli_warn cli_abort cli_alert_success cli_alert_info
#' @importFrom utils combn tail
#' @export
identify_core <- function(
  physeq_obj,
  rarefied_list = NULL,
  priority_var,
  increase_value = 0.02,
  abundance_weight = 0,
  max_otus = NULL,
  depth_level = 1000,
  seed = NULL
) {
  # Input checks ----
  cli::cli_text("\nSeed used: {seed}\n")

  if (is.null(seed)) {
    cli::cli_warn("No seed was set. Results may not be reproducible.")
  } else {
    set.seed(seed)
  }

  .phyloseq_class_check(physeq_obj)

  # Define arguments ----
  # Check if samples are rarefied (all have same depth, accounting for
  # floating-point precision)
  ## Validate / generate rarefied_list ----
  min_sum <- min(sample_sums(physeq_obj))
  max_sum <- max(sample_sums(physeq_obj))
  is_rarefied <- abs(min_sum - max_sum) < 1e-6

  if (!is.null(rarefied_list)) {
    if (!is.list(rarefied_list) || length(rarefied_list) <= 1) {
      cli::cli_abort(
        c(
          "A list of 2 or more rarefied iterations is required when {.arg physeq_obj} is not already rarefied.",
          "i" = "Provide the output of {.fn multi_rarefy} via {.arg rarefied_list} with at least 2 iterations.",
          "i" = "Alternatively, use {.fn update_otu_table} to replace the OTU table with a rarefied version."
        )
      )
    }
    cli::cli_alert_info(
      "Using provided {.arg rarefied_list} ({length(rarefied_list)} iterations)."
    )
  } else if (is_rarefied) {
    # physeq_obj is already rarefied — wrap its OTU table as a single-iteration list
    cli::cli_alert_info(
      "No {.arg rarefied_list} provided. {.arg physeq_obj} is already rarefied; wrapping as a single iteration."
    )
    otu_for_list <- .extract_otu_matrix(physeq_obj, samples_as_rows = TRUE)
    rarefied_list <- list(as.data.frame(otu_for_list))
  } else {
    cli::cli_abort(
      c(
        "{.arg physeq_obj} is not rarefied and no {.arg rarefied_list} was provided.",
        "i" = "Provide the output of {.fn multi_rarefy} via {.arg rarefied_list} with at least 2 iterations.",
        "i" = "Alternatively, use {.fn update_otu_table} to replace the OTU table with a rarefied version."
      )
    )
  }

  ## Extract OTU table, sample metadata, and taxonomy (if present) ----
  otu <- .extract_otu_matrix(physeq_obj, samples_as_rows = FALSE)
  map <- sample_data(physeq_obj) |> as("data.frame")
  map$sample_id <- rownames(map)

  ### check for a tax_table if present, if not just print a warning and continue.
  taxa <- NULL
  tx <- tryCatch(tax_table(physeq_obj), error = function(e) NULL)

  if (!is.null(tx)) {
    txm <- tryCatch(as(tx, "matrix"), error = function(e) NULL)
    if (!is.null(txm) && is.matrix(txm) && ncol(txm) > 0L) {
      taxa <- as.data.frame(txm, stringsAsFactors = FALSE)
    }
  }

  if (is.null(taxa)) {
    cli::cli_alert_info(
      "No taxonomy found (or empty). Continuing without taxonomy."
    )
  }

  ## Core prioritizing variable ----
  data_var <- priority_var
  cli::cli_alert_success("Core prioritizing variable: {data_var}")

  ## Validate abundance_weight ----
  if (
    !is.numeric(abundance_weight) ||
      length(abundance_weight) != 1L ||
      is.na(abundance_weight)
  ) {
    stop("`abundance_weight` must be a single numeric in [0,1].")
  }
  if (abundance_weight < 0 || abundance_weight > 1) {
    cli::cli_warn(
      "`abundance_weight`={abundance_weight} is outside [0,1]; clamping."
    )
    abundance_weight <- max(0, min(1, abundance_weight))
  }

  # Abundance occupancy ----
  otu_PA <- 1 * (otu > 0)
  otu_occ <- rowSums(otu_PA) / ncol(otu_PA)
  otu_rel <- apply(
    decostand(otu, method = "total", MARGIN = 2),
    1,
    mean
  )

  occ_abun <- data.frame(otu_occ = otu_occ, otu_rel = otu_rel) |>
    tibble::rownames_to_column("otu")

  # PresenceSum ----
  PresenceSum <- data.frame(
    otu = as.factor(rownames(otu)),
    otu,
    check.names = FALSE
  ) |>
    pivot_longer(-otu, names_to = "sample_id", values_to = "abun") |>
    left_join(map, by = "sample_id") |>
    group_by(otu, .data[[data_var]]) |>
    summarise(
      time_freq = sum(.data$abun > 0) / length(.data$abun), # frequency within date
      coreTime = ifelse(.data$time_freq == 1, 1, 0), # 1 if occupancy=1 at that date
      .groups = "drop"
    ) |>
    group_by(otu) |>
    summarise(
      sumF = sum(.data$time_freq),
      sumG = sum(.data$coreTime),
      nS = length(unique(.data[[data_var]])), # unique time points, not 2 * all rows
      rank = (.data$sumF + .data$sumG) / .data$nS, # can be up to 2 (original behavior)
      .groups = "drop"
    )

  # Rank and weight in abundance ----
  otu_ranked <- occ_abun |>
    left_join(PresenceSum, by = "otu")

  rank_method <- if (abundance_weight > 0) {
    occupancy_weight <- 1 - abundance_weight
    paste0(
      "Occupancy: ",
      occupancy_weight,
      " and Abundance: ",
      abundance_weight
    )
  } else {
    "Rank only"
  }

  if (abundance_weight == 0) {
    # No abundance weighting - use rank directly
    otu_ranked <- otu_ranked |>
      arrange(desc(rank))

    cli::cli_alert_info(
      "Ranked by {rank_method}"
    )
  } else {
    otu_ranked <- otu_ranked |>
      mutate(
        occ_norm = .data$otu_occ / max(.data$otu_occ, na.rm = TRUE),
        abun_norm = .data$otu_rel / max(.data$otu_rel, na.rm = TRUE),
        spatial_weight = (abundance_weight * .data$abun_norm) +
          ((1 - abundance_weight) * .data$occ_norm),
        rank = spatial_weight * .data$rank
      ) |>
      arrange(desc(.data$rank), .data$otu) |>
      select(
        "otu",
        "rank",
        "spatial_weight",
        "occ_norm",
        "abun_norm",
        "otu_occ",
        "otu_rel"
      )
    cli::cli_alert_info("Ranked by {rank_method}")
  }

  # Optional: Limit analysis to top N OTUs
  if (!is.null(max_otus)) {
    if (!is.numeric(max_otus) || max_otus < 1) {
      cli::cli_abort("`max_otus` must be a positive integer.")
    }

    max_otus <- as.integer(max_otus)

    if (max_otus < nrow(otu_ranked)) {
      total_otus <- nrow(otu_ranked)
      otu_ranked <- otu_ranked |> slice_head(n = max_otus)
      cli::cli_alert_info(
        "Limiting analysis to top {max_otus} of {total_otus} OTUs (ranked by {rank_method})."
      )
    } else {
      cli::cli_alert_info(
        "max_otus ({max_otus}) >= total OTUs ({nrow(otu_ranked)}). Using all OTUs (ranked by {rank_method})."
      )
    }
  }

  otu_ranked_ordered <- otu_ranked$otu

  # BC accumulation ----
  cli::cli_alert_info(
    "Ranking OTUs based on BC dissimilarity, starting at {Sys.time()}"
  )

  # Pre-compute sample pairs ONCE from the first rarefied iteration
  # (all iterations share the same samples after multi_rarefy filtering)
  sample_names <- colnames(as.matrix(t(rarefied_list[[1]])))

  sample_pairs <- utils::combn(length(sample_names), 2)
  pair_names <- apply(sample_pairs, 2, function(x) {
    paste(sample_names[x], collapse = " - ")
  })

  n_iter <- length(rarefied_list)
  n_pairs <- ncol(sample_pairs)

  # Rank 1
  bc_start <- .mean_bc_over_iters(
    otu_ranked_ordered[1],
    depth_level,
    rarefied_list,
    sample_pairs,
    n_iter,
    n_pairs
  )

  BCaddition <- NULL
  BCaddition <- data.frame(
    pair_names = pair_names,
    `1` = bc_start,
    check.names = FALSE
  )

  if (nrow(otu_ranked) > 1) {
    progressbar_calc_bc <- cli::cli_progress_bar(
      name = "Calculating BC rankings",
      total = nrow(otu_ranked) - 1,
      format = "{cli::pb_bar} {cli::pb_percent} | ETA: {cli::pb_eta}",
      .auto_close = TRUE
    )

    cumulative_otus <- otu_ranked_ordered[1]

    for (i in 2:nrow(otu_ranked)) {
      cumulative_otus <- c(cumulative_otus, otu_ranked_ordered[i])

      bc_i <- .mean_bc_over_iters(
        cumulative_otus,
        depth_level,
        rarefied_list,
        sample_pairs,
        n_iter,
        n_pairs
      )

      BCaddition <- left_join(
        BCaddition,
        data.frame(
          pair_names = pair_names,
          value = bc_i,
          check.names = FALSE
        ),
        by = "pair_names"
      )
      names(BCaddition)[ncol(BCaddition)] <- as.character(i)

      cli::cli_progress_update()
    }
  }
  temp_BC_matrix <- NULL
  temp_BC_matrix <- BCaddition |>
    tibble::column_to_rownames("pair_names") |>
    as.matrix()

  BC_ranked <- NULL
  BC_ranked <- data.frame(
    rank = as.factor(rownames(t(temp_BC_matrix))),
    t(temp_BC_matrix),
    check.names = FALSE
  ) |>
    pivot_longer(-rank, names_to = "comparison", values_to = "BC") |>
    group_by(.data$rank) |>
    summarise(MeanBC = mean(.data$BC), .groups = "drop") |>
    mutate(rank_num = as.numeric(as.character(.data$rank))) |>
    arrange(.data$rank_num) |>
    mutate(proportionBC = .data$MeanBC / max(.data$MeanBC))

  # Increase method ----
  if (nrow(BC_ranked) >= 2) {
    Increase <- ifelse(
      BC_ranked$MeanBC[-nrow(BC_ranked)] == 0,
      NA_real_,
      BC_ranked$MeanBC[-1] / BC_ranked$MeanBC[-nrow(BC_ranked)]
    )
    increaseDF <- data.frame(
      IncreaseBC = c(NA_real_, Increase),
      rank = factor(seq_len(length(Increase) + 1))
    )
  } else {
    increaseDF <- data.frame(IncreaseBC = NA_real_, rank = factor(1))
  }
  BC_ranked <- left_join(BC_ranked, increaseDF, by = "rank")

  # add OTU name, delta_pct_max_BC ----
  BC_ranked <- BC_ranked |>
    mutate(
      rank_num = as.numeric(as.character(.data$rank)),
      otu_added = otu_ranked_ordered[.data$rank_num],
      delta_pct_max_BC = (.data$IncreaseBC - 1) * 100
    )

  # Elbow method ----
  elbow_slope_differences <- function(pos) {
    left <- (BC_ranked$MeanBC[pos] - BC_ranked$MeanBC[1]) / pos
    right <- (BC_ranked$MeanBC[nrow(BC_ranked)] - BC_ranked$MeanBC[pos]) /
      max(1, (nrow(BC_ranked) - pos))
    left - right
  }
  BC_ranked$elbow_slope_diffs <- vapply(
    seq_len(nrow(BC_ranked)),
    elbow_slope_differences,
    numeric(1)
  )

  elbow <- which.max(BC_ranked$elbow_slope_diffs)

  # FALLBACK: if elbow is empty, NA, or 0, default to 1
  if (length(elbow) == 0 || is.na(elbow) || elbow == 0) {
    cli::cli_warn(
      "Elbow calculation failed (possibly due to low variation). Defaulting to 1."
    )
    elbow <- 1
  }

  # Threshold cut based on multiplicative increase
  thr <- 1 + increase_value
  valid_increases <- which(BC_ranked$IncreaseBC >= thr)

  lastCall <- if (length(valid_increases)) {
    utils::tail(valid_increases, 1)
  } else {
    1
  }

  # Add core set flags
  BC_ranked <- BC_ranked |>
    mutate(
      is_BC_core = .data$rank_num <= lastCall,
      last_pctBC_cutoff = .data$rank_num == lastCall,
      is_elbow_core = .data$rank_num <= elbow,
      last_elbow_cutoff = .data$rank_num == elbow
    ) |>
    arrange(.data$rank_num) |>

    select(
      "rank",
      "rank_num",
      "otu_added",
      "MeanBC",
      "proportionBC",
      "IncreaseBC",
      "elbow_slope_diffs",
      "delta_pct_max_BC",
      "is_BC_core",
      "last_pctBC_cutoff",
      "is_elbow_core",
      "last_elbow_cutoff"
    )

  # identified core sets ----

  elbow_core <- otu_ranked_ordered[seq_len(elbow)]
  cli::cli_alert_success(
    "Elbow method identified {.val {length(elbow_core)}} core OTUs"
  )
  increase_core <- otu_ranked_ordered[seq_len(lastCall)]
  cli::cli_alert_success(
    "% increase method identified {.val {length(increase_core)}} core OTUs"
  )

  # return ----
  out <- list(
    bray_curtis_ranked = BC_ranked,
    otu_ranked = otu_ranked,
    abundance_occupancy = occ_abun,
    priority_var = data_var,
    abundance_weight = abundance_weight,
    elbow = as.integer(elbow),
    bc_increase = as.integer(lastCall),
    increase_value = increase_value,
    elbow_core = elbow_core,
    increase_core = increase_core,
    otu_table = otu,
    metadata = map,
    taxonomy = taxa
  )

  class(out) <- c("identify_core_result", class(out))

  cli::cli_alert_success("Analysis complete!")

  out
}

#' Helper: compute mean BC dissimilarity across all rarefied iterations
#' for a given subset of OTUs (rows of each iteration matrix)
#'
#' #' This is the core computational step that is repeated for each rank as
#' OTUs are added.
#' This function is called internally by `identify_core` and is not exported.
#'
#' @param otu_subset Character vector of OTU names to include in the
#' calculation.
#' @param depth_level Numeric value representing the sequencing depth level.
#' @param rarefied_list List of rarefied OTU tables.
#' @param sample_pairs Matrix of sample pairs for BC calculation.
#' @param n_iter Number of iterations.
#' @param n_pairs Number of sample pairs.
#' @return Numeric vector of mean BC values for each sample pair.
#'
#' @noRd
#' @keywords internal
#'
.mean_bc_over_iters <- function(
  otu_subset,
  depth_level,
  rarefied_list,
  sample_pairs,
  n_iter,
  n_pairs
) {
  # otu_subset: character vector of OTU names
  pair_sums <- matrix(0, nrow = n_iter, ncol = n_pairs)

  for (iter_i in seq_len(n_iter)) {
    iter_mat <- as.matrix(t(rarefied_list[[iter_i]])) # taxa x samples
    # subset to OTUs present in this iteration (some may have dropped to 0)
    keep_otus <- intersect(otu_subset, rownames(iter_mat))
    sub_mat <- iter_mat[keep_otus, , drop = FALSE]

    bc_iter <- as.vector(
      vegan::vegdist(t(sub_mat), method = "bray")
    )

    # Re-normalize: vegdist uses sum(x+y) as denominator;
    # we want sum(abs(x-y)) / (2 * depth_level) to match find_core
    pair_sums_iter <- apply(sample_pairs, 2, function(x) {
      sum(sub_mat[, x[1]]) + sum(sub_mat[, x[2]])
    })
    pair_sums[iter_i, ] <- bc_iter * pair_sums_iter / (2 * depth_level)
  }

  colMeans(pair_sums)
}

test_that("identify_core returns correctly formatted outputs on switchgrass", {
  skip_if_not_installed("phyloseq")
  skip_if_not_installed("vegan")
  skip_if_not(is.function(identify_core), "identify_core() not found")

  # load test data shipped with the package
  data(
    "switchgrass",
    package = "BRCore",
    envir = environment()
  )
  skip_if_not(exists("switchgrass"), "switchgrass dataset not found")

  # sanity: required grouping column
  sm <- as(phyloseq::sample_data(switchgrass), "data.frame")
  skip_if_not(
    "sampling_date" %in% names(sm),
    "'sampling_date' column not found in sample_data(switchgrass)"
  )

  # run once with a fixed seed for reproducibility
  res <- identify_core(
    physeq_obj = switchgrass,
    priority_var = "sampling_date",
    increase_value = 0.02,
    abundance_weight = 0,
    seed = 4
  )

  # ---- presence / type checks ----
  expect_true(is.list(res))
  expect_true(all(
    c(
      "bray_curtis_ranked",
      "otu_ranked",
      "abundance_occupancy",
      "elbow",
      "bc_increase"
    ) %in%
      names(res)
  ))

  # bray_curtis_ranked
  expect_true(is.data.frame(res$bray_curtis_ranked))
  expect_true(all(
    c("rank", "MeanBC", "proportionBC", "IncreaseBC") %in%
      names(res$bray_curtis_ranked)
  ))
  expect_true(is.numeric(res$bray_curtis_ranked$MeanBC))
  expect_true(is.numeric(res$bray_curtis_ranked$proportionBC))
  expect_true(is.numeric(res$bray_curtis_ranked$IncreaseBC))
  expect_equal(max(res$bray_curtis_ranked$proportionBC), 1)
  expect_true(is.na(res$bray_curtis_ranked$IncreaseBC[1]))

  # otu_ranked
  expect_true(is.data.frame(res$otu_ranked))
  expect_true(all(c("otu", "rank") %in% names(res$otu_ranked)))
  expect_true(is.numeric(res$otu_ranked$rank))
  expect_gt(nrow(res$otu_ranked), 0)

  # abundance occupancy
  expect_true(is.data.frame(res$abundance_occupancy))
  expect_true(all(
    c("otu", "otu_occ", "otu_rel") %in% names(res$abundance_occupancy)
  ))
  expect_true(is.numeric(res$abundance_occupancy$otu_occ))
  expect_true(is.numeric(res$abundance_occupancy$otu_rel))
  expect_true(all(
    res$abundance_occupancy$otu_occ >= 0 &
      res$abundance_occupancy$otu_occ <= 1
  ))
  expect_true(all(
    is.finite(res$abundance_occupancy$otu_rel) &
      res$abundance_occupancy$otu_rel >= 0
  ))
  # same OTU universe (ignoring order)
  expect_true(setequal(res$abundance_occupancy$otu, res$otu_ranked$otu))

  # elbow & bc_increase are length-1 integers/numerics within bounds
  expect_length(as.integer(res$elbow), 1)
  expect_length(as.integer(res$bc_increase), 1)
  expect_gte(as.integer(res$elbow), 1)
  expect_gte(as.integer(res$bc_increase), 1)
  expect_lte(as.integer(res$elbow), nrow(res$otu_ranked))
  expect_lte(as.integer(res$bc_increase), nrow(res$otu_ranked))

  # optional: number of BC ranks should match number of taxa considered
  expect_equal(nrow(res$bray_curtis_ranked), nrow(res$otu_ranked))

  # ---- reproducibility with same seed ----
  res2 <- identify_core(
    physeq_obj = switchgrass,
    priority_var = "sampling_date",
    increase_value = 0.02,
    abundance_weight = 0,
    seed = 4
  )
  expect_equal(
    res$bray_curtis_ranked$MeanBC,
    res2$bray_curtis_ranked$MeanBC,
    tolerance = 1e-12
  )
  expect_equal(res$elbow, res2$elbow)
  expect_equal(res$bc_increase, res2$bc_increase)
})

test_that("max_otus parameter correctly limits the number of OTUs analyzed", {
  skip_if_not_installed("phyloseq")
  skip_if_not_installed("vegan")

  data(
    "switchgrass",
    package = "BRCore",
    envir = environment()
  )
  skip_if_not(exists("switchgrass"), "switchgrass dataset not found")

  # Get total number of OTUs
  total_otus <- phyloseq::ntaxa(switchgrass)

  # Test 1: max_otus smaller than total OTUs should truncate
  max_test <- 50
  res_limited <- identify_core(
    physeq_obj = switchgrass,
    priority_var = "sampling_date",
    max_otus = max_test,
    seed = 123
  )

  expect_equal(nrow(res_limited$otu_ranked), max_test)
  expect_equal(nrow(res_limited$bray_curtis_ranked), max_test)
  expect_lte(res_limited$elbow, max_test)
  expect_lte(res_limited$bc_increase, max_test)

  # Test 2: max_otus larger than total OTUs should use all
  res_all <- identify_core(
    physeq_obj = switchgrass,
    priority_var = "sampling_date",
    max_otus = total_otus + 100,
    seed = 123
  )

  expect_equal(nrow(res_all$otu_ranked), total_otus)
  expect_equal(nrow(res_all$bray_curtis_ranked), total_otus)

  # Test 3: NULL max_otus should use all OTUs
  res_null <- identify_core(
    physeq_obj = switchgrass,
    priority_var = "sampling_date",
    max_otus = NULL,
    seed = 123
  )

  expect_equal(nrow(res_null$otu_ranked), total_otus)
  expect_equal(nrow(res_null$bray_curtis_ranked), total_otus)

  # Test 4: Results with NULL and large max_otus should be identical
  expect_equal(res_all$otu_ranked$otu, res_null$otu_ranked$otu)
  expect_equal(
    res_all$bray_curtis_ranked$MeanBC,
    res_null$bray_curtis_ranked$MeanBC
  )
  expect_equal(res_all$elbow, res_null$elbow)
  expect_equal(res_all$bc_increase, res_null$bc_increase)
})

test_that("max_otus parameter validates inputs correctly", {
  skip_if_not_installed("phyloseq")
  skip_if_not_installed("vegan")

  data(
    "switchgrass",
    package = "BRCore",
    envir = environment()
  )
  skip_if_not(exists("switchgrass"), "switchgrass dataset not found")

  # Test invalid inputs
  expect_error(
    identify_core(
      physeq_obj = switchgrass,
      priority_var = "sampling_date",
      max_otus = -5,
      seed = 123
    ),
    "positive integer"
  )

  expect_error(
    identify_core(
      physeq_obj = switchgrass,
      priority_var = "sampling_date",
      max_otus = 0,
      seed = 123
    ),
    "positive integer"
  )

  expect_error(
    identify_core(
      physeq_obj = switchgrass,
      priority_var = "sampling_date",
      max_otus = "fifty",
      seed = 123
    ),
    "positive integer"
  )
})

test_that("max_otus preserves top-ranked OTUs based on ranking method", {
  skip_if_not_installed("phyloseq")
  skip_if_not_installed("vegan")

  data(
    "switchgrass",
    package = "BRCore",
    envir = environment()
  )
  skip_if_not(exists("switchgrass"), "switchgrass dataset not found")

  # Run with all OTUs
  res_full <- identify_core(
    physeq_obj = switchgrass,
    priority_var = "sampling_date",
    seed = 456
  )

  # Run with limited OTUs
  max_test <- 30
  res_limited <- identify_core(
    physeq_obj = switchgrass,
    priority_var = "sampling_date",
    max_otus = max_test,
    seed = 456
  )

  # The limited set should contain the top N OTUs from the full ranking
  top_otus_full <- head(res_full$otu_ranked$otu, max_test)

  expect_equal(res_limited$otu_ranked$otu, top_otus_full)
  expect_true(all(res_limited$otu_ranked$otu %in% res_full$otu_ranked$otu))
})

test_that("max_otus with abundance_weight preserves correct ranking", {
  skip_if_not_installed("phyloseq")
  skip_if_not_installed("vegan")

  data(
    "switchgrass",
    package = "BRCore",
    envir = environment()
  )
  skip_if_not(exists("switchgrass"), "switchgrass dataset not found")

  # Test with abundance weighting
  res_full_weighted <- identify_core(
    physeq_obj = switchgrass,
    priority_var = "sampling_date",
    abundance_weight = 0.3,
    seed = 789
  )

  max_test <- 40
  res_limited_weighted <- identify_core(
    physeq_obj = switchgrass,
    priority_var = "sampling_date",
    abundance_weight = 0.3,
    max_otus = max_test,
    seed = 789
  )

  # Check that the limited set matches the top N from full weighted ranking
  top_weighted <- head(res_full_weighted$otu_ranked$otu, max_test)
  expect_equal(res_limited_weighted$otu_ranked$otu, top_weighted)
  expect_equal(nrow(res_limited_weighted$otu_ranked), max_test)
})

# Test identify_core internal functions if accesible

#' Run rarefaction for microbiome count tables
#'
#' This function performs rarefaction on a `phyloseq` object by randomly
#' sub-sampling OTUs/ASVs within samples without replacement for a number of
#' iterations specified by the user. Samples with fewer OTUs/ASVs than the
#' specified `depth_level` are discarded.
#'
#' @param physeq_obj A `phyloseq` object containing an OTU/ASV table.
#' @param depth_level An integer specifying the sequencing depth (number of
#'   OTUs/ASVs) to which samples should be rarefied.
#' @param num_iter An integer specifying the number of iterations to perform
#'   for rarefaction.
#' @param .as A character string indicating whether to return the results as a
#' 3D array or as a list of data frames. If `"array"`, returns a 3D array with
#' dimensions (samples x taxa x iterations). If `"list"`, returns a list of
#' data frames, one for each iteration, with samples as rows and taxa as
#' columns. (default = "list")
#' @param set_seed An optional integer to set the random seed for
#' reproducibility (default = NULL).
#'
#' @return A data frame with taxa as rows and samples as columns. The values
#'   represent the average sequence counts calculated across all iterations.
#'   Samples with less than `depth_level` sequences are discarded.
#'
#' @seealso [update_otu_table()] for updating the OTU table in a `phyloseq`
#' object and [vegan::rrarefy()] for the underlying rarefaction method used in
#' this function.
#'
#' @examples
#' library(BRCore)
#'
#'
#' # Example rarefaction (single iteration, single core to keep examples fast)
#' otu_table_rare <- multi_rarefy(
#'   physeq_obj = bcse,
#'   depth_level = 1000,
#'   num_iter = 10,
#'   .as = "list",
#'   set_seed = 7642
#' )
#'
#' rowSums(otu_table_rare[[1]])
#'
#'
#' @importFrom dplyr group_by summarise across everything filter near
#' @importFrom dplyr where
#' @importFrom tibble rownames_to_column column_to_rownames
#' @importFrom phyloseq otu_table
#' @importFrom cli cli_h1 cli_h2 cli_alert_info cli_alert_warning
#' @importFrom cli cli_alert_success cli_alert_danger
#' @importFrom utils head
#' @importFrom vegan rrarefy
#'
#' @export
multi_rarefy <- function(
  physeq_obj,
  depth_level,
  num_iter = 100,
  .as = "list",
  set_seed = NULL
) {
  # Input validation ----
  cli::cli_h1("Rarefaction iterations starting...")
  cli::cli_h2("Input Validation")

  if (!requireNamespace("phyloseq", quietly = TRUE)) {
    cli::cli_alert_danger(
      "The 'phyloseq' package is required but not installed."
    )
    stop("Please install 'phyloseq' to use this function.")
  }

  .phyloseq_class_check(physeq_obj)

  if (!is.numeric(depth_level) || depth_level <= 0) {
    cli::cli_alert_danger("depth_level must be a positive number")
    stop("Invalid depth_level")
  }

  if (!is.numeric(num_iter) || num_iter <= 0) {
    cli::cli_alert_danger("num_iter must be a positive integer")
    stop("Invalid num_iter")
  }

  # Seed setup: iteration-specific seeds deterministically ---
  if (is.null(set_seed)) {
    cli::cli_alert_warning("No seed set. Results may not be reproducible.")
    iteration_seeds <- rep(NA_integer_, num_iter)
  } else {
    cli::cli_alert_info("Seed: {.val {set_seed}}")
    set.seed(set_seed)
    iteration_seeds <- sample.int(.Machine$integer.max, num_iter)
  }

  # Prepare data ----
  otu_mat <- .extract_otu_matrix(physeq_obj, samples_as_rows = TRUE)

  ## Filter samples by depth ----
  n_samples_before <- nrow(otu_mat)
  original_sample_ids <- rownames(otu_mat) |>
    unique()
  n_taxa_before <- ncol(otu_mat)

  keep <- rowSums(otu_mat) >= depth_level

  if (length(keep) != nrow(otu_mat)) {
    stop("Logical subsetting mismatch: check OTU table orientation.")
  }

  otu_mat <- otu_mat[keep, , drop = FALSE]

  n_samp <- nrow(otu_mat)
  n_taxa <- ncol(otu_mat)

  # Input parameter checks ----
  cli::cli_alert_info(
    "Input (matrix/df dim): {.val {nrow(otu_mat)}} samples x {.val {ncol(otu_mat)}} taxa"
  )
  cli::cli_alert_info("Rarefaction depth: {.val {depth_level}}")
  cli::cli_alert_info("Iterations: {.val {num_iter}}")

  cli::cli_alert_info("taxa_are_rows: {phyloseq::taxa_are_rows(physeq_obj)}")

  cli::cli_alert_info(
    "OTU matrix/df rownames head: {paste(head(rownames(otu_mat)), collapse = ', ')}"
  )
  cli::cli_alert_info(
    "OTU matrix/df colnames head: {paste(head(colnames(otu_mat)), collapse = ', ')}"
  )

  cli::cli_alert_info(
    "Row sums summary: Min={min(rowSums(otu_mat))}, Max={max(rowSums(otu_mat))}, Median={median(rowSums(otu_mat))}"
  )

  # Single iteration ----
  if (num_iter == 1) {
    if (!is.na(iteration_seeds[1])) {
      set.seed(iteration_seeds[1])
    }
    processed_data <- as.data.frame(vegan::rrarefy(
      otu_mat,
      sample = depth_level
    ))
    if (.as == "list") {
      stats::setNames(list(processed_data), "iter_1")
    }
    if (.as == "array") {
      processed_data <- array(
        as.matrix(processed_data),
        dim = c(nrow(processed_data), ncol(processed_data), 1),
        dimnames = list(
          rownames(processed_data),
          colnames(processed_data),
          "iter_1"
        )
      )
    }
  }

  # Multiple iterations ----
  if (.as == "array") {
    processed_data <- array(
      0,
      dim = c(n_samp, n_taxa, num_iter),
      dimnames = list(
        rownames(otu_mat),
        colnames(otu_mat),
        paste0("iter_", seq_len(num_iter))
      )
    )

    for (i in seq_len(num_iter)) {
      processed_data[,, i] <- vegan::rrarefy(otu_mat, sample = depth_level)
    }
  }
  if (.as == "list") {
    processed_data <- lapply(seq_len(num_iter), function(i) {
      if (!is.na(iteration_seeds[i])) {
        set.seed(iteration_seeds[i])
      }

      rare_result <- vegan::rrarefy(otu_mat, sample = depth_level)

      if (is.null(dim(rare_result))) {
        rare_result <- matrix(
          rare_result,
          nrow = 1,
          dimnames = list(rownames(otu_mat), names(rare_result))
        )
      }

      as.data.frame(rare_result)
    })
    names(processed_data) <- paste0("iter_", seq_len(num_iter))
  }

  avg_taxa_removed <- NULL

  if (.as == "array") {
    n_samples_removed <- n_samples_before - n_samp
    removed_samples <- setdiff(original_sample_ids, rownames(otu_mat))
    n_taxa_removed <- n_taxa - dim(processed_data)[2]
  }
  if (.as == "list") {
    n_taxa_after <- min(vapply(processed_data, ncol, integer(1)))

    n_samples_after <- nrow(processed_data[[1]])
    n_samples_removed <- n_samples_before - n_samples_after
    removed_samples <- setdiff(
      rownames(otu_mat),
      rownames(processed_data[[1]])
    )

    removed_samples <- setdiff(original_sample_ids, rownames(otu_mat))

    n_taxa_removed <- n_taxa_before - n_taxa_after

    avg_taxa_removed <- mean(vapply(processed_data, ncol, double(1)))
  }

  # Report results ----
  .report_rarefaction_results(
    n_samples_removed = n_samples_removed,
    n_taxa_before = n_taxa_before,
    n_taxa_removed = n_taxa_removed,
    avg_taxa_removed = avg_taxa_removed,
    removed_samples = removed_samples,
    dataframe = otu_mat,
    processed_data = processed_data,
    depth_level = depth_level,
    unique_samples = original_sample_ids,
    num_iter = num_iter
  )

  # Return the processed data frame
  return(processed_data)
}


#' Count zeros and sparsity in a data frame
#' @param dataframe A data frame to analyze.
#' @return A list containing the number of zeros
#'
#' @noRd
#' @keywords internal
.sparsity_count <- function(dataframe) {
  if (inherits(dataframe, "list")) {
    min_zeros <- min(vapply(dataframe, function(df) sum(df == 0), integer(1)))
    min_total <- min(vapply(
      dataframe,
      function(df) nrow(df) * ncol(df),
      integer(1)
    ))

    max_zeros <- max(vapply(dataframe, function(df) sum(df == 0), integer(1)))
    max_total <- max(vapply(
      dataframe,
      function(df) nrow(df) * ncol(df),
      integer(1)
    ))

    avg_zeros <- mean(vapply(dataframe, function(df) sum(df == 0), double(1)))
    avg_total <- mean(vapply(
      dataframe,
      function(df) nrow(df) * ncol(df),
      double(1)
    ))
  } else {
    original_zeros <- sum(dataframe == 0)
    original_total <- nrow(dataframe) * ncol(dataframe)
  }

  if (inherits(dataframe, "list")) {
    sparsity <- list(
      min_zeros = min_zeros,
      max_zeros = max_zeros,
      avg_zeros = avg_zeros,
      min_total = min_total,
      max_total = max_total,
      avg_total = avg_total,
      min_sparsity = round(min_zeros / min_total * 100, 2),
      max_sparsity = round(max_zeros / max_total * 100, 2),
      avg_sparsity = round(avg_zeros / avg_total * 100, 2)
    )
  } else {
    sparsity <- round(original_zeros / original_total * 100, 2)

    list(
      zeros = original_zeros,
      total = original_total,
      sparsity = sparsity
    )
  }
}

#' Helper function to report rarefaction results
#' @param n_samples_removed Number of samples removed during rarefaction.
#' @param n_taxa_removed Number of taxa removed during rarefaction.
#' @param removed_samples A vector of sample names that were removed.
#' @param dataframe The original data frame before rarefaction.
#' @param processed_data The data frame after rarefaction.
#' @param depth_level The depth level used for rarefaction.
#' @return None. This function is used for side-effect reporting via cli messages.
#' @noRd
#' @keywords internal
#'
.report_rarefaction_results <- function(
  n_samples_removed,
  n_taxa_before,
  n_taxa_removed,
  avg_taxa_removed,
  removed_samples,
  dataframe,
  processed_data,
  depth_level,
  unique_samples = NULL,
  num_iter = NULL
) {
  cli::cli_h2("Rarefaction Results")

  # Detect if processed_data is a 3D array or list of data frames

  is_array <- is.array(processed_data) && length(dim(processed_data)) == 3

  # Sample Removal
  cli::cli_h3("Sample Removal")
  if (n_samples_removed > 0) {
    cli::cli_alert_warning(
      "{.val {n_samples_removed}} sample{?s} removed (depth < {.val {depth_level}})"
    )
    cli::cli_alert_warning(
      "Samples removed: {.val {paste(removed_samples, collapse = ', ')}}"
    )
  } else {
    cli::cli_alert_success("No samples removed.")
  }

  # Taxa Removal
  cli::cli_h3("Taxa Removal")
  if (n_taxa_removed == 0) {
    cli::cli_alert_success("No taxa removed.")
    cli::cli_alert_warning(
      "Taxa are not removed across iterations to maintain consistent dimensions. \nDownstream analyses should handle zero-abundance taxa appropriately."
    )
  }

  # Data Sparsity
  cli::cli_h3("Data Sparsity")

  if (is_array) {
    iter_zeros <- apply(processed_data, 3, function(mat) sum(mat == 0))
    iter_total <- dim(processed_data)[1] * dim(processed_data)[2]

    cli::cli_ul()
    cli::cli_li("Rarefied matrix (across {.val {num_iter}} iterations):")
    cli::cli_ul(c(
      "Min: {.val {min(iter_zeros)}} zeros ({.val {round(min(iter_zeros) / iter_total * 100, 2)}}% sparsity) out of {.val {iter_total}} entries",
      "Max: {.val {max(iter_zeros)}} zeros ({.val {round(max(iter_zeros) / iter_total * 100, 2)}}% sparsity) out of {.val {iter_total}} entries",
      "Avg: {.val {round(mean(iter_zeros), 1)}} zeros ({.val {round(mean(iter_zeros) / iter_total * 100, 2)}}% sparsity) out of {.val {iter_total}} entries"
    ))
    cli::cli_end()
    cli::cli_end()
  } else {
    cli::cli_alert_info(
      "Returning list of data frames for each iteration."
    )

    rarefied_sparsity <- .sparsity_count(processed_data)
    # cli::cli_alert_info(
    #   "Rarefied matrix (across {.val {num_iter}} iterations):"
    # )
    cli::cli_ul()
    cli::cli_li("Rarefied matrix (across {.val {num_iter}} iterations):")
    cli::cli_ul(c(
      "Min: {.val {rarefied_sparsity$min_zeros}} zeros ({.val {rarefied_sparsity$min_sparsity}}% sparsity) out of {.val {rarefied_sparsity$min_total}} entries",
      "Max: {.val {rarefied_sparsity$max_zeros}} zeros ({.val {rarefied_sparsity$max_sparsity}}% sparsity) out of {.val {rarefied_sparsity$max_total}} entries",
      "Avg: {.val {round(rarefied_sparsity$avg_zeros, 1)}} zeros ({.val {rarefied_sparsity$avg_sparsity}}% sparsity) out of {.val {round(rarefied_sparsity$avg_total, 1)}} entries"
    ))
    cli::cli_end()
    cli::cli_end()
  }

  # Final Data Dimensions
  cli::cli_h3("Final Data Dimensions")

  if (is_array) {
    iter_nonzero_taxa <- apply(processed_data, 3, function(mat) {
      sum(colSums(mat) > 0)
    })
    n_samp <- dim(processed_data)[1]
    n_taxa <- dim(processed_data)[2]

    cli::cli_alert_success(
      "Output: {.val {num_iter}} iteration{?s} with {.val {n_samp}} sample{?s}"
    )
    cli::cli_ul()
    cli::cli_li("Array dimensions:")
    cli::cli_ul(c(
      "Samples: {.val {n_samp}}",
      "Taxa: {.val {n_taxa}}",
      "Iterations: {.val {num_iter}}"
    ))
    cli::cli_end()

    cli::cli_li("Non-zero taxa per iteration:")
    cli::cli_ul(c(
      "Min: {.val {min(iter_nonzero_taxa)}}",
      "Max: {.val {max(iter_nonzero_taxa)}}",
      "Avg: {.val {round(mean(iter_nonzero_taxa), 1)}}"
    ))
    cli::cli_end()
    cli::cli_end()
  } else {
    processed_data <- lapply(processed_data, function(df) {
      df[, colSums(df) > 0, drop = FALSE]
    })
    summary_stats <- list(
      min_taxa = min(vapply(processed_data, ncol, integer(1))),
      max_taxa = max(vapply(processed_data, ncol, integer(1))),
      avg_taxa = mean(vapply(processed_data, ncol, double(1))),
      min_samples = min(vapply(processed_data, nrow, integer(1))),
      max_samples = max(vapply(processed_data, nrow, integer(1)))
    )

    cli::cli_alert_success(
      "Output: {.val {num_iter}} iteration{?s} with {.val {length(unique_samples)}} unique sample{?s}"
    )
    cli::cli_ul()
    cli::cli_li("Samples per iteration:")
    cli::cli_ul(c(
      "Min: {.val {summary_stats$min_samples}}",
      "Max: {.val {summary_stats$max_samples}}"
    ))
    cli::cli_end()

    cli::cli_li("Non-zero taxa per iteration:")
    cli::cli_ul(c(
      "Min: {.val {summary_stats$min_taxa}}",
      "Max: {.val {summary_stats$max_taxa}}",
      "Avg: {.val {round(summary_stats$avg_taxa, 1)}}"
    ))
    cli::cli_end()
    cli::cli_end()
  }
}

test_that("Test if default behaviour is correct", {
  skip_if_not_installed("phyloseq")

  data("bcse", package = "BRCore", envir = environment())

  otu_table_rare <- multi_rarefy(
    physeq_obj = bcse,
    depth_level = 3000,
    num_iter = 3,
    .as = "array",
    set_seed = 123
  )

  # Should return a single averaged data frame (default behavior)
  expect_true(is.array(otu_table_rare) && length(dim(otu_table_rare)) == 3)

  # Test if all the samples have the same number of reads
  read_counts <- rowSums(otu_table_rare[,, 1])
  expect_equal(max(read_counts) - min(read_counts), 0) # All values are identical
  expect_true(all(abs(read_counts - 3000) < 1e-6))
})

test_that("multi_rarefy retains samples with row sums near depth_level (floating-point tolerance)", {
  skip_if_not_installed("phyloseq")

  # Simulate the floating-point scenario: a summarised data frame where some
  # samples have row sums that are VERY slightly below depth_level due to
  # floating-point arithmetic, which should still be kept.
  depth <- 1000L
  # Use a sub-epsilon offset to mimic the macOS floating-point mean artifact
  eps <- .Machine$double.eps

  # Build a tiny mock: one sample exactly at depth, one sub-epsilon below
  # (simulating macOS floating-point mean), one clearly below.
  mock_df <- data.frame(
    sample_id = c("exact", "fp_below", "too_low"),
    taxon1 = c(500, 500 - eps, 200),
    taxon2 = c(500, 500 + eps * 0.1, 100)
  )

  # Apply the same near()-based filter used inside multi_rarefy()
  result <- dplyr::filter(
    mock_df,
    dplyr::near(
      rowSums(across(where(is.numeric))),
      depth,
      tol = 1e-6
    )
  )

  # "exact" (rowSum == 1000) and "fp_below" (rowSum ≈ 1000) should be kept;
  # "too_low" (rowSum == 300) should be dropped.
  expect_equal(nrow(result), 2L)
  expect_true("exact" %in% result$sample_id)
  expect_true("fp_below" %in% result$sample_id)
  expect_false("too_low" %in% result$sample_id)
})

test_that("multi_rarefy outputs all rarefaction iterations when .as = 'list'", {
  skip_if_not_installed("phyloseq")

  data("bcse", package = "BRCore", envir = environment())

  num_iterations <- 3
  rarefied_data <- multi_rarefy(
    physeq_obj = bcse,
    depth_level = 3000,
    num_iter = num_iterations,
    .as = "list",
    set_seed = 123
  )

  # Should return a list of data frames
  expect_type(rarefied_data, "list")
  expect_equal(length(rarefied_data), num_iterations)
  expect_true(all(vapply(rarefied_data, is.data.frame, logical(1))))

  # Each iteration should have the correct naming
  expect_equal(names(rarefied_data), paste0("iter_", 1:num_iterations))

  # Extract unique sample IDs from the phyloseq object
  unique_samples <- nrow(phyloseq::sample_data(bcse))

  # Calculate expected samples after removing low-depth samples
  removed_samples <- setdiff(
    rownames(phyloseq::sample_data(bcse)),
    rownames(rarefied_data[[1]])
  )

  expected_samples_per_iter <- unique_samples - length(removed_samples)

  # Check each iteration has the expected number of samples
  for (i in seq_along(rarefied_data)) {
    expect_equal(
      nrow(rarefied_data[[i]]),
      expected_samples_per_iter,
      info = paste(
        "Iteration",
        i,
        "should have",
        expected_samples_per_iter,
        "samples"
      )
    )
  }

  # Verify row names are consistent across iterations (no iteration suffixes in new implementation)
  for (i in seq_along(rarefied_data)) {
    expect_equal(
      rownames(rarefied_data[[1]]),
      rownames(rarefied_data[[i]]),
      info = paste(
        "Iteration",
        i,
        "should have same sample names as iteration 1"
      )
    )
  }

  # Verify all samples are rarefied to the correct depth
  for (i in seq_along(rarefied_data)) {
    row_sums <- rowSums(rarefied_data[[i]])
    expect_true(
      all(abs(row_sums - 3000) < 1e-6),
      info = paste("All samples in iteration", i, "should have depth ~3000")
    )
  }
})

test_that("multi_rarefy with .as = 'array' returns 3D array", {
  skip_if_not_installed("phyloseq")

  data("bcse", package = "BRCore", envir = environment())

  num_iterations <- 5
  rarefied_array <- multi_rarefy(
    physeq_obj = bcse,
    depth_level = 3000,
    num_iter = num_iterations,
    .as = "array",
    set_seed = 456
  )

  # Should return a 3D array
  expect_true(is.array(rarefied_array))
  expect_equal(length(dim(rarefied_array)), 3)

  # Check dimension names
  expect_equal(dimnames(rarefied_array)[[3]], paste0("iter_", 1:num_iterations))

  # All iterations should have same depth
  for (i in seq_len(num_iterations)) {
    row_sums <- rowSums(rarefied_array[,, i])
    expect_true(
      all(row_sums == 3000),
      info = paste("All samples in iteration", i, "should have depth 3000")
    )
  }
})

test_that("multi_rarefy list and array have consistent dimensions", {
  skip_if_not_installed("phyloseq")

  data("bcse", package = "BRCore", envir = environment())

  # Same parameters for both
  rarefied_list <- multi_rarefy(
    physeq_obj = bcse,
    depth_level = 3000,
    num_iter = 3,
    .as = "list",
    set_seed = 789
  )

  rarefied_array <- multi_rarefy(
    physeq_obj = bcse,
    depth_level = 3000,
    num_iter = 3,
    .as = "array",
    set_seed = 789
  )

  # Array dimensions should match list dimensions
  expect_equal(dim(rarefied_array)[1], nrow(rarefied_list[[1]]))
  expect_equal(
    length(intersect(
      colnames(rarefied_list[[1]]),
      colnames(rarefied_array[,, 1])
    )),
    2861L
  )

  expect_equal(dim(rarefied_array)[1], nrow(rarefied_list[[1]]))
  expect_equal(dim(rarefied_array)[3], length(rarefied_list))
})

test_that("multi_rarefy removes zero-abundance taxa independently per iteration", {
  skip_if_not_installed("phyloseq")

  data("bcse", package = "BRCore", envir = environment())

  rarefied_data <- multi_rarefy(
    physeq_obj = bcse,
    depth_level = 3000,
    num_iter = 3,
    .as = "list",
    set_seed = 321
  )

  # Each iteration may have different numbers of non-zero taxa
  taxa_counts <- vapply(rarefied_data, ncol, integer(1))

  # Should have some variation (not all identical)
  # This tests that zero-taxa removal happens per iteration
  expect_true(length(unique(taxa_counts)) >= 1)

  # All should be > 0
  expect_true(all(taxa_counts > 0))
})

test_that("multi_rarefy handles low-depth samples correctly", {
  skip_if_not_installed("phyloseq")

  data("bcse", package = "BRCore", envir = environment())

  # Use high depth to force some samples to be removed
  rarefied_data <- multi_rarefy(
    physeq_obj = bcse,
    depth_level = 50000,
    num_iter = 2,
    .as = "list",
    set_seed = 111
  )

  # Should retain 2 samples at the depth
  expect_true(length(rarefied_data) == 2)

  # Remaining samples should all have depth == 50000
  for (i in seq_along(rarefied_data)) {
    row_sums <- rowSums(rarefied_data[[i]])
    expect_true(all(row_sums == 50000))
  }
})

test_that("multi_rarefy single iteration returns data frame (not list)", {
  skip_if_not_installed("phyloseq")

  data("bcse", package = "BRCore", envir = environment())

  # Single iteration with .as = "list" should return data frame
  rarefied_single <- multi_rarefy(
    physeq_obj = bcse,
    depth_level = 3000,
    num_iter = 1,
    .as = "list",
    set_seed = 999
  )

  # Should be a DF
  expect_type(rarefied_single, "list") # Still a list, but should contain one data frame
  expect_s3_class(rarefied_single[[1]], "data.frame")

  # Length of taxa should match original (minus any removed)
  expect_equal(
    length(rarefied_single[[1]]),
    phyloseq::otu_table(bcse) |> nrow()
  )
})

#' Plot Core Taxa  Occupancy Across Metadata Groups
#'
#' Creates a plot showing core taxa (i.e. OTUs/ASVs) occupancy patterns across
#' a grouping variable.
#'
#' @param core_result A list object returned by \code{\link{identify_core}},
#'   containing at minimum:
#'   \itemize{
#'     \item \code{otu_table}: A data frame with ASV/OTUs as rows and samples
#'       as columns.
#'     \item \code{metadata}: A data frame with samples as rows and grouping
#'       variables as columns.
#'     \item \code{otu_ranked}: A data frame with ranked taxa containing:
#'       \itemize{
#'         \item \code{otu}: A column with taxa names.
#'         \item \code{rank}: A column with the rank for each taxon.
#'       }
#'     \item \code{elbow_core}: Character vector of OTU IDs identified as core
#'       using the elbow method.
#'     \item \code{increase_core}: Character vector of OTU IDs identified as core
#'       using the increase method.
#' }
#'
#' @param core_set Which core set to plot: "elbow" (default) or "increase".
#' @param group_var Metadata column for bar coloring. Default: "sampling_date".
#' @param plot_type Allows selection of 3 different plot types: `bar`, `line`, or `heatmap`.
#'
#' @return A ggplot2 object that can be further customized.
#'
#' @seealso [identify_core()], [plot_abundance_occupancy()], and
#' [plot_identified_core()]
#'
#' @examples
#' library(BRCore)
#' data("switchgrass_core", package = "BRCore")
#'
#' p <- plot_core_distribution(
#'   core_result = switchgrass_core,
#'   core_set = "increase",
#'   group_var = "sampling_date",
#'   plot_type = "bar"
#' )
#' print(p)
#'
#' @importFrom tibble rownames_to_column
#' @importFrom tidyr pivot_longer
#' @importFrom dplyr filter left_join group_by summarise n
#' @importFrom ggplot2 ggplot aes geom_bar coord_flip scale_x_discrete
#' @importFrom ggplot2 theme_classic theme element_text element_line
#' @importFrom ggplot2 element_blank labs geom_line geom_point facet_wrap
#' @importFrom ggplot2 scale_y_continuous geom_tile
#' @importFrom vegan decostand
#' @importFrom grid unit
#'
#' @export
plot_core_distribution <- function(
  core_result,
  core_set = "elbow",
  group_var = "Crop",
  plot_type = c("bar", "line", "heatmap")
) {
  # match plot_type
  plot_type <- match.arg(plot_type)

  # Validate core_set argument
  if (!core_set %in% c("elbow", "increase")) {
    cli::cli_abort("core_set must be either 'elbow' or 'increase'")
  }

  # Select appropriate core OTU set
  if (core_set == "elbow") {
    core_otus <- core_result$elbow_core
  } else {
    core_otus <- core_result$increase_core
  }

  # Calculate relative abundance
  otu_relabun <- decostand(
    as.data.frame(core_result$otu_table),
    method = "total",
    MARGIN = 2
  )

  # Prepare metadata
  map <- as.data.frame(core_result$metadata)

  if (!"sample_id" %in% colnames(map)) {
    map <- map |> rownames_to_column("sample_id")
  }

  # Validate required columns
  if (is.null(group_var)) {
    cli::cli_abort("group_var must be specified.")
  }
  if (!group_var %in% colnames(map)) {
    cli::cli_abort(paste0(
      "Missing required column '",
      group_var,
      "' in metadata. Available columns: ",
      paste(colnames(map), collapse = ", ")
    ))
  }

  otu_ranked <- core_result$otu_ranked

  # Prepare plotting data
  plotDF <- otu_relabun |>
    as.data.frame() |>
    rownames_to_column(var = "otu") |>
    pivot_longer(
      cols = -otu,
      names_to = "sample_id",
      values_to = "relabun"
    ) |>
    left_join(map, by = "sample_id") |>
    left_join(otu_ranked, by = "otu") |>
    filter(otu %in% core_otus) |>
    group_by(otu, .data[[group_var]]) |>
    summarise(
      time_freq = sum(relabun > 0) / n(),
      coreTime = ifelse(time_freq == 1, 1, 0),
      detect = ifelse(time_freq > 0, 1, 0),
      .groups = "drop"
    )

  # Set factor levels for proper ordering
  core_levels <- otu_ranked$otu[otu_ranked$otu %in% core_otus]
  plotDF$otu <- factor(plotDF$otu, levels = core_levels)

  # Create plot
  if (plot_type == "bar") {
    p <- ggplot(
      plotDF,
      aes(x = otu, y = time_freq, fill = factor(.data[[group_var]]))
    ) +
      geom_bar(stat = "identity", position = "dodge") +
      coord_flip() +
      scale_x_discrete(limits = rev(levels(plotDF$otu))) +
      .brcore_theme(
        extra_themes = list(theme(legend.title = element_blank()))
      ) +
      ggplot2::scale_fill_viridis_d(option = "turbo") +
      labs(
        title = paste("Core set occupancy across:", group_var),
        x = "Ranked ASV/OTUs",
        y = "Occupancy"
      )
  } else if (plot_type == "line") {
    group_levels <- length(levels(factor(plotDF[[group_var]])))

    # one facet per group_var level, one line per panel
    p <- ggplot(
      plotDF,
      aes(
        x = otu,
        y = time_freq,
        group = factor(.data[[group_var]]),
        color = factor(.data[[group_var]])
      )
    ) +
      geom_line() +
      geom_point(size = 0.7) +
      facet_wrap(
        stats::as.formula(paste("~", group_var)),
        nrow = group_levels
      ) +
      scale_y_continuous(limits = c(0, 1)) +
      .brcore_theme(
        axis_text_angle = 45,
        extra_themes = list(theme(legend.position = "none"))
      ) +
      ggplot2::scale_color_viridis_d(option = "turbo") +
      labs(
        title = paste("Core set occupancy across:", group_var),
        x = "Ranked ASV/OTUs",
        y = "Occupancy"
      )
  } else if (plot_type == "heatmap") {
    p <- ggplot(
      plotDF,
      aes(x = factor(.data[[group_var]]), y = otu, fill = time_freq)
    ) +
      geom_tile(color = "white", linewidth = 0.5) +
      .brcore_theme(
        axis_text_angle = 33,
        extra_themes = list(
          theme(
            legend.title = element_text(size = 9, face = "bold")
          )
        )
      ) +
      ggplot2::scale_fill_viridis_c(option = "plasma") +
      labs(
        title = paste("Core set occupancy across:", group_var),
        x = paste(group_var),
        y = "Ranked ASV/OTUs",
        fill = "Occupancy"
      )
  }

  return(p)
}

# tests/testthat/test-plot_core_distribution.R
test_that("plot_core_distribution returns a ggplot and builds", {
  skip_if_not_installed("ggplot2")

  # Create a matrix of random counts
  otu_table <- base::as.data.frame(
    matrix(
      rpois(6 * 15, lambda = 20) * (runif(6 * 15) > 0.4),
      nrow = 15,
      ncol = 6
    )
  )

  rownames(otu_table) <- paste0("OTU", 1:15)
  colnames(otu_table) <- paste0("Sample", 1:6)

  metadata <- base::data.frame(
    sample_id = colnames(otu_table),
    time = factor(c(rep(2016, 3), rep(2017, 3)))
  )

  increase_core <- c("OTU1", "OTU2", "OTU3", "OTU5")

  # Create the dataframe
  otu_ranked <- data.frame(
    otu = rownames(otu_table),
    abundance = rowSums(otu_table)
  ) |>
    dplyr::arrange(desc(abundance)) |>
    dplyr::mutate(rank = dplyr::row_number())

  core_result_example <- list(
    otu_table = otu_table,
    metadata = metadata,
    increase_core = increase_core,
    otu_ranked = otu_ranked
  )

  p <- plot_core_distribution(
    core_result = core_result_example,
    core_set = "increase",
    group_var = "time"
  )

  expect_s3_class(p, "ggplot")
  expect_silent(ggplot2::ggplot_build(p))
})

---
title: "Introduction to BRCore"
author: "By Bolívar Aponte Rolón and Gian Maria Niccolò Benucci"
date: "Last modified: 18 May, 2026"
output: 
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 3

vignette: >
  %\VignetteIndexEntry{Introduction to BRCore}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: references.bib
link-citations: true
---



<style type="text/css">
.note {
  background-color: #e7f3fe;
  border-left: 6px solid #2196F3;
  padding: 10px;
  margin: 15px 0;
}

h1, .h1 {
    margin-top: 84px;
    margin-bottom: 42px;
}
h2, .h2, h3, .h3 {
    margin-top: 42px;
    margin-bottom: 21px;
}

p.caption {
    font-size: 1em;
    font-style: italic;
    color: grey;
    margin-right: 10%;
    margin-left: 10%;
    text-align: justify;
}
</style>

## Load R libraries


``` r
invisible(
  lapply(
    c("BRCore", "phyloseq", "tidyverse", "viridis"),
    library,
    character.only = TRUE
  )
)
```

## Datasets included
The `BRCore` R package comes with a few datasets. Three 16S datasets: `switchgrass`, 
`mimulus`, and `bean`, from @Shade2019Abundance; a 16S dataset, called `bcse`, 
from leaves of ten different cropping systems in the Bioenergy Crop Research Experiment 
from @Haan2023Contrasting. Here we are going to use `bcse` since it is not rarefied 
in contrast to the other three.


``` r
str(bcse)
#> Formal class 'phyloseq' [package "phyloseq"] with 5 slots
#>   ..@ otu_table:Formal class 'otu_table' [package "phyloseq"] with 2 slots
#>   .. .. ..@ .Data        : int [1:2861, 1:50] 0 1 0 0 27777 0 7 2 119 0 ...
#>   .. .. .. ..- attr(*, "dimnames")=List of 2
#>   .. .. .. .. ..$ : chr [1:2861] "OTU_427" "OTU_11" "OTU_253" "OTU_148" ...
#>   .. .. .. .. ..$ : chr [1:50] "bcse50" "bcse69" "bcse73" "bcse191" ...
#>   .. .. ..@ taxa_are_rows: logi TRUE
#>   .. .. ..$ dim     : int [1:2] 2861 50
#>   .. .. ..$ dimnames:List of 2
#>   .. .. .. ..$ : chr [1:2861] "OTU_427" "OTU_11" "OTU_253" "OTU_148" ...
#>   .. .. .. ..$ : chr [1:50] "bcse50" "bcse69" "bcse73" "bcse191" ...
#>   ..@ tax_table:Formal class 'taxonomyTable' [package "phyloseq"] with 1 slot
#>   .. .. ..@ .Data: chr [1:2861, 1:10] "OTU_427" "OTU_11" "OTU_253" "OTU_148" ...
#>   .. .. .. ..- attr(*, "dimnames")=List of 2
#>   .. .. .. .. ..$ : chr [1:2861] "OTU_427" "OTU_11" "OTU_253" "OTU_148" ...
#>   .. .. .. .. ..$ : chr [1:10] "OTU_ID" "Kingdom" "Phylum" "Class" ...
#>   .. .. ..$ dim     : int [1:2] 2861 10
#>   .. .. ..$ dimnames:List of 2
#>   .. .. .. ..$ : chr [1:2861] "OTU_427" "OTU_11" "OTU_253" "OTU_148" ...
#>   .. .. .. ..$ : chr [1:10] "OTU_ID" "Kingdom" "Phylum" "Class" ...
#>   ..@ sam_data :'data.frame':	50 obs. of  3 variables:
#> Formal class 'sample_data' [package "phyloseq"] with 4 slots
#>   .. .. ..@ .Data    :List of 3
#>   .. .. .. ..$ : chr [1:50] "Leaf" "Leaf" "Leaf" "Leaf" ...
#>   .. .. .. ..$ : chr [1:50] "Corn" "Sorghum" "Switchgrass" "Miscanthus" ...
#>   .. .. .. ..$ : chr [1:50] "R2" "R4" "R5" "R5" ...
#>   .. .. ..@ names    : chr [1:3] "Niche" "Crop" "Plot"
#>   .. .. ..@ row.names: chr [1:50] "bcse50" "bcse69" "bcse73" "bcse191" ...
#>   .. .. ..@ .S3Class : chr "data.frame"
#>   ..@ phy_tree : NULL
#>   ..@ refseq   :Formal class 'DNAStringSet' [package "Biostrings"] with 5 slots
#>   .. .. ..@ pool           :Formal class 'SharedRaw_Pool' [package "XVector"] with 2 slots
#>   .. .. .. .. ..@ xp_list                    :List of 1
#>   .. .. .. .. .. ..$ :<externalptr> 
#>   .. .. .. .. ..@ .link_to_cached_object_list:List of 1
#>   .. .. .. .. .. ..$ :<environment: 0x5ad933230678> 
#>   .. .. ..@ ranges         :Formal class 'GroupedIRanges' [package "XVector"] with 7 slots
#>   .. .. .. .. ..@ group          : int [1:2861] 1 1 1 1 1 1 1 1 1 1 ...
#>   .. .. .. .. ..@ start          : int [1:2861] 106501 2501 63001 36751 501 19251 14251 37751 3751 8501 ...
#>   .. .. .. .. ..@ width          : int [1:2861] 250 250 250 250 250 250 250 250 250 250 ...
#>   .. .. .. .. ..@ NAMES          : chr [1:2861] "OTU_427" "OTU_11" "OTU_253" "OTU_148" ...
#>   .. .. .. .. ..@ elementType    : chr "ANY"
#>   .. .. .. .. ..@ elementMetadata: NULL
#>   .. .. .. .. ..@ metadata       : list()
#>   .. .. ..@ elementType    : chr "DNAString"
#>   .. .. ..@ elementMetadata: NULL
#>   .. .. ..@ metadata       : list()
```

# Data Exploration and Parameter Selection for Core Identification


## Rarefaction metrics and visualization
We are not going to talk about the importance of rarefaction as it is not the goal
of this vignette, but if you are interested you should read @Schloss2024Rarefaction, 
@Schloss2024Waste, @Hong2022Rarefy, and @Weiss2017Normalization as a start.

In `BRCore`, to identify the ideal rarefaction cutoff we can plot a simple series 
of visuals to guide us in the decision. We can, more or less, 
estimate how well a DNA sample will sequence, but there are always some samples 
that fail to sequence (or sequence badly) and some others that will sequence 
"too well" for reasons that are outside our control or by chance.

In the end we will need to decide how many samples we are willing to discard and
how much diversity we want to retain in our data. To do that we can use some help 
by plotting some diagnostics plots as shown below.



``` r
bcse_metrics <- add_rarefaction_metrics(data = bcse)
bcse_metrics
#> phyloseq-class experiment-level object
#> otu_table()   OTU Table:         [ 2861 taxa and 50 samples ]
#> sample_data() Sample Data:       [ 50 samples by 7 sample variables ]
#> tax_table()   Taxonomy Table:    [ 2861 taxa by 10 taxonomic ranks ]
#> refseq()      DNAStringSet:      [ 2861 reference sequences ]
```


``` r
rarefaction_plot <- plot_rarefaction_metrics(bcse_metrics)
#> ℹ Processing 50 samples
#> 
ℹ Generating rarefaction diagnostic plots

                                         
✔ Rarefaction diagnostic plots generated successfully
#> ℹ Generating rarefaction diagnostic plots

✔ Generating rarefaction diagnostic plots [1s]
print(rarefaction_plot)
```

<div class="figure" style="text-align: center">
<img src="images/fig1_plot_metrics-1.png" alt="Figure 1: Rarefaction metrics. a-b, histograms of samples. c, Good's coverage per total number of sequences in a sample. d-e, log10 of sequences in a sample. f,  samples ranked by sequence reads in a sample."  />
<p class="caption">Figure 1: Rarefaction metrics. a-b, histograms of samples. c, Good's coverage per total number of sequences in a sample. d-e, log10 of sequences in a sample. f,  samples ranked by sequence reads in a sample.</p>
</div>


<div class="note">
**Note:** The Good's coverage" is a statistics used in ecology that estimates the 
proportion of species in a community that are represented in a sample, based on 
the number of species encountered and the total number of individuals sampled. 
Essentially, it quantifies how well a sample represents the overall diversity of 
a population or ecosystem. Good's coverage is calculated as $(1-(n1/N)$, where $n1$ 
is the number of unique ASV/OTUs (i.e. species) found only once, and $N$ is the 
total number of sequence reads (i.e. individuals); a high percentage (e.g., >95%) 
means most reads are from common taxa, suggesting sufficient sampling, while low 
coverage indicates many rare, potentially missed, taxa. 
</div>

## Rarefy the data and explore variance propagation
Originally developed by @Sanders1968Marine rarefaction has been adopted widely in 
ecology and microbiome research. It consists of randomly resampling the same number of species (ASV/OTUs) multiple times across all samples to estimate the expected number of species that would be observed if all samples were identical. It prevents samples with more individuals (e.g., more sequencing reads) from appearing
more diverse simply because they were sampled more in depth. 

Because rarefaction is a random process, it adds noise to the data. Each resampling event is slightly different from others due to chance. This can be problematic, especially when a very low rarefaction cutoff is selected, when, for example, we want to compare 2 treatments. With lower rarefaction cutoffs, the greater the chance you have that rarefaction iterations will differ from one another, as also shown by @Cameron2021Rarefying.

We developed functions that can perform and store all the rarefaction iterations to evaluate (and visualize) the noise generated in the process, as shown below:


``` r
bcse_rarefied_list <-
  multi_rarefy(
    physeq_obj = bcse,
    depth_level = 1000,
    num_iter = 3,
    .as = "list",
    set_seed = 7642
  )
#> 
#> ── Rarefaction iterations starting... ──────────────────────────────────────────────
#> 
#> ── Input Validation ──
#> 
#> ✔ Input phyloseq object is valid!
#> ℹ Seed: 7642
#> ℹ Input (matrix/df dim): 47 samples x 2861 taxa
#> ℹ Rarefaction depth: 1000
#> ℹ Iterations: 3
#> ℹ taxa_are_rows: TRUE
#> ℹ OTU matrix/df rownames head: bcse50, bcse69, bcse73, bcse191, bcse82, bcse102
#> ℹ OTU matrix/df colnames head: OTU_427, OTU_11, OTU_253, OTU_148, OTU_3, OTU_78
#> ℹ Row sums summary: Min=1193, Max=107643, Median=25209
#> 
#> ── Rarefaction Results ──
#> 
#> ── Sample Removal
#> ! 3 samples removed (depth < 1000)
#> ! Samples removed: "bcse108, bcse105, bcse110"
#> 
#> ── Taxa Removal
#> ✔ No taxa removed.
#> ! Taxa are not removed across iterations to maintain consistent dimensions. 
#> Downstream analyses should handle zero-abundance taxa appropriately.
#> 
#> ── Data Sparsity
#> ℹ Returning list of data frames for each iteration.
#> • Rarefied matrix (across 3 iterations):
#>   • Min: 130570 zeros (97.1% sparsity) out of 134467 entries
#>   • Max: 130663 zeros (97.17% sparsity) out of 134467 entries
#>   • Avg: 130608 zeros (97.13% sparsity) out of 134467 entries
#> 
#> ── Final Data Dimensions
#> ✔ Output: 3 iterations with 50 unique samples
#> • Samples per iteration:
#>   • Min: 47
#>   • Max: 47
#> • Non-zero taxa per iteration:
#>   • Min: 1046
#>   • Max: 1098
#>   • Avg: 1073
```

Let's explore the results.


``` r
class(bcse_rarefied_list)
#> [1] "list"
str(bcse_rarefied_list[[1]], list.len = 10) # Dimensions of iteration #1
#> 'data.frame':	47 obs. of  2861 variables:
#>  $ OTU_427  : int  0 0 0 0 0 0 0 0 0 0 ...
#>  $ OTU_11   : int  0 0 0 0 0 0 0 0 0 0 ...
#>  $ OTU_253  : int  0 0 0 0 0 0 0 0 0 0 ...
#>  $ OTU_148  : int  0 1 0 11 2 0 0 0 0 78 ...
#>  $ OTU_3    : int  261 86 651 272 249 114 192 73 96 320 ...
#>  $ OTU_78   : int  0 0 0 0 0 0 0 0 0 0 ...
#>  $ OTU_58   : int  0 0 0 0 0 0 0 0 0 0 ...
#>  $ OTU_152  : int  0 0 0 0 0 0 0 0 0 0 ...
#>  $ OTU_16   : int  2 0 0 1 1 0 0 1 0 0 ...
#>  $ OTU_35   : int  0 0 0 0 0 0 0 0 0 0 ...
#>   [list output truncated]
```

We can plot the variance (the noise we talked above) across the 3 iterations of 
rarefaction. We can use the `plot_variance_propagation()` function to visualize 
this variance in Alpha diversity across samples and iterations and compare 
Raw (non-rarefied) vs. Rarefied data.


``` r
rarefaction_variance_plot <- plot_variance_propagation(
  physeq_obj = bcse,
  rarefied = bcse_rarefied_list,
  q = 0,
  group_var = "Crop",
  group_color = "Plot"
)
#> ✔ Input phyloseq object is valid!
#> 
#> ── Rarefaction Variance Propagation Visualization ──────────────────────────────────
#> ℹ Hill number order selected, q= 0
#> ℹ Number of rarefaction iterations, n_iter= 3
#> ℹ Comparison plot generated!
print(rarefaction_variance_plot)
```

<div class="figure" style="text-align: center">
<img src="images/fig2_rarefaction_variance-1.png" alt="Figure 2: Alpha diversity variance propagation (variance between rarefaction iteration) across 3 iterations of rarefaction. Points represent samples. The  x-axis represent the grouping variable (Crop). The y-axis represent the alpha diversity metric (q = 0, i.e. richness) calculated on the samples. The color of the points represent the 'Plot' variable, which is a nested variable within 'Crop'."  />
<p class="caption">Figure 2: Alpha diversity variance propagation (variance between rarefaction iteration) across 3 iterations of rarefaction. Points represent samples. The  x-axis represent the grouping variable (Crop). The y-axis represent the alpha diversity metric (q = 0, i.e. richness) calculated on the samples. The color of the points represent the 'Plot' variable, which is a nested variable within 'Crop'.</p>
</div>


**Review the rarefaction and variance plots to determine the ideal sampling depth.**
You are looking for a "sweet spot" that maintains high diversity and samples
while keeping variance across iterations low. Pay extra attention when rarefaction
cutoff is quite low, as show by @Cameron2021Rarefying. **Note this depth, as it will
serve as the cutoff for the `identify_core()` step in your pipeline.**


### Recreate a phyloseq object with a rarefied `otu_table()`

***This is an optional step.***

The `identify_core()` function accepts a phyloseq object, whether rarefied or non-rarefied. When supplying a phyloseq object with a non-rarefied `otu_table`, you must provide a list of rarefied OTU tables that result from `multi_rarefy()`. In our case, we generated `bcse_rarefied_list`.

 The results from `add_rarefaction_metrics()`,
`plot_rarefaction_metrics()`, and `multi_rarefy()` are meant to explore the data
and inform the chosen parameters for `identify_core()`. Function `identify_core()` will calculate the distance matrix for all iterations supplied and rank OTUs based on the average distance across iterations.

If you have a phyloseq object with a rarefied `otu_table()` or if you want to use a specific rarefaction iteration generated by `multi_rarefy()`, you also have these options. *The choice is up to you and depends on your preferences and the specific goals of your analysis.* The  `update_otu_table()` function allows for replacing the `otu_table()` with the rarefied table you prefer.



``` r
bcse_rare_single <- update_otu_table(
  physeq_obj = bcse,
  rarefied_otus = bcse_rarefied_list,
  iteration = 1 # Speficify which iteration to use for the updated OTU table
)
#> ℹ Extracting iteration 1 from list of 3 iterations.
#> ! Phyloseq object and rarefied otu_table sample names are NOT identical. Check samples removed by rarefaction below.
#> ! 3 samples removed due to rarefaction: "bcse108, bcse105, bcse110"
#> ℹ Building phyloseq object with 47 samples and 2861 taxa
#> ✔ Update complete!
```
Let's proceed to the core analysis.

# Core analysis


### Identify the core ASV/OTUs set

We can now identify how many core ASV/OTUs are present in the dataset. One important difference that separates `identify_core()` from other methods of identification of core taxa is that you can specify a variable (in the case of the `bcse` dataset is `Crop`) so that OTUs can be core for a level of the variable but possibly not for others. The idea stems from the assumption that the microbiome can be transient in space and time, as shown by @Shade2019Abundance.

**We continue with the `bcse` phyloseq object and the rarefied data `bcse_rarefied_list` throughout the vignette.** See below for example using a single iteration.



``` r
bcse_core_multi <- identify_core(
  physeq_obj = bcse,
  rarefied_list = bcse_rarefied_list,
  priority_var = "Crop",
  increase_value = 0.02,
  abundance_weight = 0,
  depth_level = 1000,
  seed = 2134
)
#> Seed used: 2134
#> ✔ Input phyloseq object is valid!
#> ℹ Using provided `rarefied_list` (3 iterations).
#> ✔ Core prioritizing variable: Crop
#> ℹ Ranked by Rank only
#> ℹ Ranking OTUs based on BC dissimilarity, starting at 2026-05-18 15:17:06.698128
#> ✔ Elbow method identified 7 core OTUs
#> ✔ % increase method identified 30 core OTUs
#> ✔ Analysis complete!

# With a single iteration 
# bcse_core_single <- identify_core(
#   physeq_obj = bcse_rare_single,
#   priority_var = "Crop",
#   increase_value = 0.02,
#   seed = 2134
# )
```

## *How the is the core identified?*

As shown by @Shade2019Abundance, an interesting way to explore the abundance and occupancy inclusion thresholds used to define the core microbiome is to evaluate how well the resulting core membership captures the overarching patterns of beta-diversity found in the complete dataset. This approach treats the core microbiome as a representative subset that should preserve the main ecological relationships and sample-to-sample differences observed when using all taxa. By measuring how closely the beta-diversity patterns calculated from core taxa alone match those from the full dataset, researchers can objectively assess whether their chosen thresholds yield a meaningful and informative core that retains the essential structural information of the microbial community while reducing complexity.

To identify the optimal point where increasing the core inclusion threshold provides diminishing returns in explanatory value, we offer two automated methods.  

* The first is a more stringent `elbow` approach that finds the bend in the 
abundance-occupancy curve where adding more taxa yields progressively smaller improvements. This method calculates the first-order difference (numerical differentiation) by assigning a score to each potential cutoff point, splitting the curve into two parts, and finding the cutoff that maximizes the difference in average rates of change between the two parts.  

* The second method uses a final percent `increase` threshold in beta-diversity 
(recommend 2% or more), which continues adding taxa until improvements fall 
below this percentage. 

Both methods measure improvement using Bray-Curtis similarity through the 
equation $C = 1 - (BC_{core}/BC_{all})$, where $C$ represents the contribution of 
ranked taxa to total similarity, $BC_{core}$ is the Bray-Curtis similarity using 
only core taxa, and $BC_{all}$ uses the complete dataset. 

The cumulative explanatory value of adding each next-ranked taxon can be plotted using the `plot_identified_core()` function, with the vertical lines indicating the elbow and 2% threshold cutoff methods. These approaches eliminate the need  for subjective threshold setting by automatically identifying natural breakpoints  where additional taxa provide marginal returns in explanatory power.

**The output is a list of 13 named items.**



``` r
str(bcse_core_multi)
#> List of 13
#>  $ bray_curtis_ranked : tibble [2,861 × 12] (S3: tbl_df/tbl/data.frame)
#>   ..$ rank             : Factor w/ 2861 levels "1","10","100",..: 1 1112 2085 2196 2307 2418 2529 2640 2751 2 ...
#>   ..$ rank_num         : num [1:2861] 1 2 3 4 5 6 7 8 9 10 ...
#>   ..$ otu_added        : chr [1:2861] "OTU_3" "OTU_12" "OTU_21" "OTU_68" ...
#>   ..$ MeanBC           : num [1:2861] 0.0744 0.0862 0.1019 0.1036 0.1264 ...
#>   ..$ proportionBC     : num [1:2861] 0.12 0.139 0.164 0.167 0.203 ...
#>   ..$ IncreaseBC       : num [1:2861] NA 1.16 1.18 1.02 1.22 ...
#>   ..$ elbow_slope_diffs: num [1:2861] -0.000191 0.005744 0.009008 0.007127 0.010232 ...
#>   ..$ delta_pct_max_BC : num [1:2861] NA 15.96 18.22 1.63 22 ...
#>   ..$ is_BC_core       : logi [1:2861] TRUE TRUE TRUE TRUE TRUE TRUE ...
#>   ..$ last_pctBC_cutoff: logi [1:2861] FALSE FALSE FALSE FALSE FALSE FALSE ...
#>   ..$ is_elbow_core    : logi [1:2861] TRUE TRUE TRUE TRUE TRUE TRUE ...
#>   ..$ last_elbow_cutoff: logi [1:2861] FALSE FALSE FALSE FALSE FALSE FALSE ...
#>  $ otu_ranked         :'data.frame':	2861 obs. of  7 variables:
#>   ..$ otu    : chr [1:2861] "OTU_3" "OTU_12" "OTU_21" "OTU_68" ...
#>   ..$ otu_occ: num [1:2861] 1 1 1 1 1 1 1 1 1 1 ...
#>   ..$ otu_rel: num [1:2861] 0.21728 0.01799 0.03296 0.00278 0.05897 ...
#>   ..$ sumF   : num [1:2861] 10 10 10 10 10 10 10 10 10 10 ...
#>   ..$ sumG   : num [1:2861] 10 10 10 10 10 10 10 10 10 10 ...
#>   ..$ nS     : int [1:2861] 10 10 10 10 10 10 10 10 10 10 ...
#>   ..$ rank   : num [1:2861] 2 2 2 2 2 2 2 2 2 2 ...
#>  $ abundance_occupancy:'data.frame':	2861 obs. of  3 variables:
#>   ..$ otu    : chr [1:2861] "OTU_427" "OTU_11" "OTU_253" "OTU_148" ...
#>   ..$ otu_occ: num [1:2861] 0.08 0.46 0.04 0.74 1 0.32 0.3 0.06 0.96 0.04 ...
#>   ..$ otu_rel: num [1:2861] 2.47e-05 5.55e-04 4.90e-06 4.82e-03 2.17e-01 ...
#>  $ priority_var       : chr "Crop"
#>  $ abundance_weight   : num 0
#>  $ elbow              : int 7
#>  $ bc_increase        : int 30
#>  $ increase_value     : num 0.02
#>  $ elbow_core         : chr [1:7] "OTU_3" "OTU_12" "OTU_21" "OTU_68" ...
#>  $ increase_core      : chr [1:30] "OTU_3" "OTU_12" "OTU_21" "OTU_68" ...
#>  $ otu_table          : int [1:2861, 1:50] 0 1 0 0 27777 0 7 2 119 0 ...
#>   ..- attr(*, "dimnames")=List of 2
#>   .. ..$ : chr [1:2861] "OTU_427" "OTU_11" "OTU_253" "OTU_148" ...
#>   .. ..$ : chr [1:50] "bcse50" "bcse69" "bcse73" "bcse191" ...
#>  $ metadata           :'data.frame':	50 obs. of  4 variables:
#>   ..$ Niche    : chr [1:50] "Leaf" "Leaf" "Leaf" "Leaf" ...
#>   ..$ Crop     : chr [1:50] "Corn" "Sorghum" "Switchgrass" "Miscanthus" ...
#>   ..$ Plot     : chr [1:50] "R2" "R4" "R5" "R5" ...
#>   ..$ sample_id: chr [1:50] "bcse50" "bcse69" "bcse73" "bcse191" ...
#>  $ taxonomy           :'data.frame':	2861 obs. of  10 variables:
#>   ..$ OTU_ID   : chr [1:2861] "OTU_427" "OTU_11" "OTU_253" "OTU_148" ...
#>   ..$ Kingdom  : chr [1:2861] "Bacteria" "Bacteria" "Bacteria" "Bacteria" ...
#>   ..$ Phylum   : chr [1:2861] "Myxococcota" "Actinobacteriota" "Proteobacteria" "Proteobacteria" ...
#>   ..$ Class    : chr [1:2861] "Polyangia" "Actinobacteria" "Alphaproteobacteria" "Gammaproteobacteria" ...
#>   ..$ Order    : chr [1:2861] "Polyangiales" "Streptomycetales" "Rhizobiales" "Enterobacterales" ...
#>   ..$ Family   : chr [1:2861] "Polyangiaceae" "Streptomycetaceae" "Rhizobiaceae" "Enterobacteriaceae" ...
#>   ..$ Genus    : chr [1:2861] "Aetherobacter" "Streptomyces" "Mesorhizobium" NA ...
#>   ..$ Species  : chr [1:2861] "Uncultured_bacterium_3097" NA "Mesorhizobium_sophorae" NA ...
#>   ..$ BestMatch: chr [1:2861] "Uncultured bacterium 3097" "Streptomyces" "Mesorhizobium sophorae" "Enterobacteriaceae" ...
#>   ..$ Taxonomy : chr [1:2861] "OTU_427-Uncultured bacterium 3097" "OTU_11-Streptomyces" "OTU_253-Mesorhizobium sophorae" "OTU_148-Enterobacteriaceae" ...
#>  - attr(*, "class")= chr [1:2] "identify_core_result" "list"
```

### Visualize the abundance-occupancy distribution and the core ASV/OTUs set

The output from `identify_core()` can be used for plotting as below:


``` r
bcse_identified_core <- plot_identified_core(
  bray_curtis_ranked = bcse_core_multi$bray_curtis_ranked,
  elbow = bcse_core_multi$elbow,
  lastCall = bcse_core_multi$bc_increase,
  increase_value = bcse_core_multi$increase_value
)

# plot_identified_core() outputs a list with a dataframe and a ggplot

print(bcse_identified_core$plot_identified_core)
```

<div class="figure" style="text-align: center">
<img src="images/fig3_identified_core-1.png" alt="Figure 3: Percent Bray-Curtis similarity between samples per ranked ASV/OTUs. Number of core ASV/OTUs identified by Elbow and Last 2% increase in Bray-Curtis similarity are shown."  />
<p class="caption">Figure 3: Percent Bray-Curtis similarity between samples per ranked ASV/OTUs. Number of core ASV/OTUs identified by Elbow and Last 2% increase in Bray-Curtis similarity are shown.</p>
</div>




### Visualize the abundance-occupancy curve 

Abundance-Occupancy distributions can be plotted:




``` r
plot_abund_occ_increase <- plot_abundance_occupancy(
  core_result = bcse_core_multi,
  core_set = "increase"
)
print(plot_abund_occ_increase)
```

<div class="figure" style="text-align: center">
<img src="images/fig4_plot_increase-1.png" alt="Figure 4: Abundance-occupancy distribution for the 'bcse' dataset. The core ASV/OTUs identified by the last 2% increase method are highlighted in red."  />
<p class="caption">Figure 4: Abundance-occupancy distribution for the 'bcse' dataset. The core ASV/OTUs identified by the last 2% increase method are highlighted in red.</p>
</div>

And for the `elbow` method.



``` r
plot_abund_occ_elbow <- plot_abundance_occupancy(
  core_result = bcse_core_multi,
  core_set = "elbow"
)

plot_abund_occ_elbow +
  scale_fill_manual(values = c("darkgreen", "grey"))
#> Scale for fill is already present.
#> Adding another scale for fill, which will replace the existing scale.
```

<div class="figure" style="text-align: center">
<img src="images/fig5_plot_elbow-1.png" alt="Figure 5: Abundance-occupancy distribution for the 'bcse' dataset. The core ASV/OTUs identified by the elbow method are highlighted in dark green."  />
<p class="caption">Figure 5: Abundance-occupancy distribution for the 'bcse' dataset. The core ASV/OTUs identified by the elbow method are highlighted in dark green.</p>
</div>


### Visualize the core set across the variable used for detecting the core

The `plot_core_distribution()` function can be used to visualize ASV/OTU occupancy across the variable specified during core identification. The function provides three visualization methods (bar, line, and heatmap) that can be toggled via the plot_type argument. The optimal choice often depends on the number of levels in your grouping variable. For instance, bar or line plots are highly effective for up to five levels, whereas heatmaps are better suited for more complex datasets. 
While these are general recommendations, the core data is fully accessible in `bcse_core_multi` should you wish to develop a custom visualization.


``` r
plot_core_dist_bar <- plot_core_distribution(
  core_result = bcse_core_multi,
  core_set = "increase",
  group_var = "Crop",
  plot_type = "bar"
)

print(plot_core_dist_bar)
```

<div class="figure" style="text-align: center">
<img src="images/fig6_plot_type_bar-1.png" alt="Figure 6: Occupancy of core ASV/OTUs across the 'Crop' variable. Each bar represents the average occupancy of core ASV/OTUs in samples belonging to each level of the 'Crop' variable. A bar plot won't work here, way too many variable levels."  />
<p class="caption">Figure 6: Occupancy of core ASV/OTUs across the 'Crop' variable. Each bar represents the average occupancy of core ASV/OTUs in samples belonging to each level of the 'Crop' variable. A bar plot won't work here, way too many variable levels.</p>
</div>






``` r
plot_core_dist_line <- plot_core_distribution(
  core_result = bcse_core_multi,
  core_set = "increase",
  group_var = "Crop",
  plot_type = "line"
)
print(plot_core_dist_line)
```

<div class="figure" style="text-align: center">
<img src="images/fig7_plot_type_line-1.png" alt="Figure 7: Occupancy of core ASV/OTUs across the 'Crop' variable. Each point represents the average occupancy of core ASV/OTUs in samples belonging to each level of the 'Crop' variable. A line plot is better than a bar plot but still not ideal for this many variable levels."  />
<p class="caption">Figure 7: Occupancy of core ASV/OTUs across the 'Crop' variable. Each point represents the average occupancy of core ASV/OTUs in samples belonging to each level of the 'Crop' variable. A line plot is better than a bar plot but still not ideal for this many variable levels.</p>
</div>



We can reorder the variable levels with the following logic before plotting.


``` r
bcse_core_multi$metadata <- bcse_core_multi$metadata %>%
  mutate(
    Crop = recode(
      Crop,
      "Corn" = "Corn",
      "Sorghum" = "Sorghum",
      "Continuous Sorghum" = "Sorghum + cover crop",
      "Miscanthus" = "Miscanthus",
      "New Switchgrass" = "Establishing switchgrass",
      "Switchgrass" = "Mature switchgrass",
      "Early Succession" = "Successional vegetation",
      "Native Grasses" = "Native grass mix",
      "Prairie" = "Reconstructed prairie",
      "Poplar" = "Poplar"
    ),
    Crop = factor(
      Crop,
      levels = c(
        "Corn",
        "Sorghum",
        "Sorghum + cover crop",
        "Miscanthus",
        "Establishing switchgrass",
        "Mature switchgrass",
        "Successional vegetation",
        "Native grass mix",
        "Reconstructed prairie",
        "Poplar"
      )
    )
  )
```






``` r
plot_core_dist_heatmap <- plot_core_distribution(
  core_result = bcse_core_multi,
  core_set = "increase",
  group_var = "Crop",
  plot_type = "heatmap"
) +
  viridis::scale_fill_viridis(option = "plasma", name = "Occupancy")
#> Scale for fill is already present.
#> Adding another scale for fill, which will replace the existing scale.

print(plot_core_dist_heatmap)
```

<div class="figure" style="text-align: center">
<img src="images/fig8_plot_type_heatmap-1.png" alt="Figure 8: Occupancy of core ASV/OTUs across the 'Crop' variable. Each cell represents the average occupancy of core ASV/OTUs in samples belonging to each level of the 'Crop' variable. A heatmap is more compact and shows well enough the average occupancy across samples in each variable level."  />
<p class="caption">Figure 8: Occupancy of core ASV/OTUs across the 'Crop' variable. Each cell represents the average occupancy of core ASV/OTUs in samples belonging to each level of the 'Crop' variable. A heatmap is more compact and shows well enough the average occupancy across samples in each variable level.</p>
</div>


# Fitting Neutral Model


Neutral models in ecology assume that species (ASVs/OTUs) are functionally 
equivalent and that community patterns arise from random chance rather than 
deterministic niches or selection. As a specific case of null models [@Gotelli2006Null], they act as a baseline. If real-world data significantly differ from the model's predictions, we can infer that non-random ecological forces are at play.

Building on the work of @Sloan2006Quantifying and @Burns2015Contribution, 
BRCore implements accessible functions to fit and visualize these models. Our goal is to make these powerful ecological tools easy to use without losing the conceptual rigor of the original math.

To get started, we first fit the neutral model:


``` r
bcse_core_multi_neutral_fit <- fit_neutral_model(
  otu_table = bcse_core_multi$otu_table,
  core_set = bcse_core_multi$increase_core,
  abundance_occupancy = bcse_core_multi$abundance_occupancy
)
#> Waiting for profiling to be done...
#> Warning: SNCM `stats::pbeta()` produced a warning during MLE: NaNs produced
#> Warning: Binomial model `stats::dnorm()` produced a warning during MLE: NaNs
#> produced
#> Warning: Poisson model `stats::dnorm()` produced a warning during MLE: NaNs
#> produced
#> ℹ Neutral model fitting:
#> • Average individuals per community (N): 40276.38
#> • Binomial model using rounded N: 40276
#> • Poisson model using N: 40276.38
#> • Maximum likelihood estimation using N: 40276.38, and starting parameters: mu = 0,
#>   sigma = 0.1
#> Warning: SNCM `stats::pbeta()` produced a warning during MLE: NaNs produced
#> Warning: Binomial model `stats::dnorm()` produced a warning during MLE: NaNs
#> produced
#> Warning: Poisson model `stats::dnorm()` produced a warning during MLE: NaNs
#> produced
#> ✔ Model fitting complete!
```

What we get is...



``` r
str(bcse_core_multi_neutral_fit)
#> List of 2
#>  $ goodness_of_fit :'data.frame':	1 obs. of  24 variables:
#>   ..$ m         : num 0.0205
#>   ..$ m.ci      : num 0.00186
#>   ..$ m.mle     : num 0.0205
#>   ..$ maxLL     : num -2279
#>   ..$ binoLL    : num -720
#>   ..$ poisLL    : num -720
#>   ..$ Rsqr      : num 0.528
#>   ..$ Rsqr.bino : num -0.833
#>   ..$ Rsqr.pois : num -0.833
#>   ..$ RMSE      : num 0.109
#>   ..$ RMSE.bino : num 0.215
#>   ..$ RMSE.pois : num 0.215
#>   ..$ AIC       : num -4554
#>   ..$ BIC       : num -4542
#>   ..$ AIC.bino  : num -1435
#>   ..$ BIC.bino  : num -1423
#>   ..$ AIC.pois  : num -1435
#>   ..$ BIC.pois  : num -1423
#>   ..$ N         : num 40276
#>   ..$ Samples   : num 50
#>   ..$ Richness  : num 2861
#>   ..$ Detect    : num 2.48e-05
#>   ..$ above.pred: num 0.165
#>   ..$ below.pred: num 0.035
#>  $ model_prediction:'data.frame':	2861 obs. of  14 variables:
#>   ..$ otu       : chr [1:2861] "OTU_427" "OTU_11" "OTU_253" "OTU_148" ...
#>   ..$ otu_occ   : num [1:2861] 0.08 0.46 0.04 0.74 1 0.32 0.3 0.06 0.96 0.04 ...
#>   ..$ otu_rel   : num [1:2861] 2.47e-05 5.55e-04 4.90e-06 4.82e-03 2.17e-01 ...
#>   ..$ membership: chr [1:2861] "Not core" "Not core" "Not core" "Not core" ...
#>   ..$ p         : num [1:2861] 3.48e-06 6.31e-05 2.48e-06 7.98e-03 2.54e-01 ...
#>   ..$ freq      : num [1:2861] 0.08 0.46 0.04 0.74 1 0.32 0.3 0.06 0.96 0.04 ...
#>   ..$ freq.pred : num [1:2861] 0.00951 0.16077 0.0068 1 1 ...
#>   ..$ pred.lwr  : num [1:2861] 0.000953 0.083916 0.000514 0.928652 0.928652 ...
#>   ..$ pred.upr  : num [1:2861] 0.088 0.286 0.0835 1 1 ...
#>   ..$ bino.pred : num [1:2861] 0.1306 0.9211 0.0952 1 1 ...
#>   ..$ bino.lwr  : num [1:2861] 0.0632 0.8131 0.0405 0.9287 0.9287 ...
#>   ..$ bino.upr  : num [1:2861] 0.251 0.969 0.208 1 1 ...
#>   ..$ y         : chr [1:2861] NA NA NA NA ...
#>   ..$ fit_class : chr [1:2861] "As predicted" "Above prediction" "As predicted" "Below prediction" ...
#>  - attr(*, "class")= chr [1:2] "fit_neutral_model" "list"
```

### Plot the neutral model and the core set 


``` r
plot_bcse_neutral_fit <- plot_neutral_model(bcse_core_multi_neutral_fit)

print(plot_bcse_neutral_fit)
```

<div class="figure" style="text-align: center">
<img src="images/fig9_plot_neutral_fit-1.png" alt="Figure 9: Neutral model fit illustrates the neutral model of abundance-occupancy distributions for the 'bcse' dataset. The $R^2$ value represents a standard coefficient of determination, calculated as $R^2 = 1 - \frac{SS_{err}}{SS_{total}}$, providing a measure of goodness of fit. The immigration parameter ($m$) estimates the probability that an individual in a local sample originates from the metacommunity rather than from local dispersal. We also observe that the neutral model provides a poor fit; the $R^2$ is notably low, and the majority of taxa (including both core and non-core ASV/OTUs) fall above the model's prediction line. Furthermore, the estimated immigration parameter ($m = 0.63$) suggests that the local community is dominated by strong dispersal and mixing from the metacommunity. "  />
<p class="caption">Figure 9: Neutral model fit illustrates the neutral model of abundance-occupancy distributions for the 'bcse' dataset. The $R^2$ value represents a standard coefficient of determination, calculated as $R^2 = 1 - \frac{SS_{err}}{SS_{total}}$, providing a measure of goodness of fit. The immigration parameter ($m$) estimates the probability that an individual in a local sample originates from the metacommunity rather than from local dispersal. We also observe that the neutral model provides a poor fit; the $R^2$ is notably low, and the majority of taxa (including both core and non-core ASV/OTUs) fall above the model's prediction line. Furthermore, the estimated immigration parameter ($m = 0.63$) suggests that the local community is dominated by strong dispersal and mixing from the metacommunity. </p>
</div>
 

<div class="note">
**Note:** 
Interpretations of the Migration Parameter, as a rule of thumb:
<ul>
  <li><strong>Low</strong> ($m \approx 0–0.05$): strong dispersal limitation; 
  local history dominates community assembly.</li>
  <li><strong>Moderate</strong> ($m \approx 0.1–0.3$): partial connectivity among 
  sites; both dispersal and local structure influence community composition.</li>
  <li><strong>High</strong> ($m \geq 0.4$): high dispersal rates and strong mixing 
  from the regional source pool.</li>
</ul>
</div>


# Supplemental Information


It is critical to note that the core taxa identified from a single rarefaction 
iteration may differ from those identified through multiple iterations. To illustrate
this stochasticity, if we identify the core using only the first iteration of our
`bcse` phyloseq object with rarefied `otu_table()` generated above using
`update_otu_table()`, we obtain the following results:


``` r
bcse_core_single_iter1 <- identify_core(
  physeq_obj = bcse_updated_rare,
  priority_var = "Crop",
  increase_value = 0.02,
  abundance_weight = 0,
  seed = 2135
)
#> Seed used: 2135
#> Error: object 'bcse_updated_rare' not found

bcse_identified_core_iter1 <- plot_identified_core(bray_curtis_ranked = bcse_core_single_iter1$bray_curtis_ranked,
  elbow = bcse_identified_core_iter1$elbow,
  lastCall = bcse_identified_core_iter1$bc_increase,
  increase_value = bcse_identified_core_iter1$increase_value
)
#> Error: object 'bcse_core_single_iter1' not found
print(bcse_identified_core_iter1$plot_idenfied_core)
#> Error: object 'bcse_identified_core_iter1' not found
```

## Further Customization of `identify_core()`

While the default settings of `identify_core()` are suitable for most analyses, the 
function provides additional parameters to balance computational efficiency with 
ecological nuance. In large datasets with thousands of ASV/OTUs, ranking every single 
taxon can be computationally expensive. The `max_otus` parameter allows you to cap 
the number of taxa processed. Rare taxa (those with very low abundance) rarely 
contribute more than 2% to the Bray-Curtis similarity. Limiting the ranking can 
significantly speed up your analysis without losing the core signal. However, 
if set too low, you may inadvertently exclude "satellite" taxa—those that are low
in abundance but highly persistent (high occupancy) across your samples.

By default, `identify_core()` prioritizes occupancy (how often a taxon appears). 
However, you may want to account for relative abundance (how dominant a taxon is). 
The `abundance_weight` argument (numeric, 0 to 1) allows you to blend these two metrics:

* abundance_weight = 0 (Default): Ranking is based strictly on occupancy and composite metrics.

* abundance_weight = 1: Ranking is based purely on mean relative abundance.

* max_otus: Limits the number of taxa processed. Useful for large datasets to improve computational efficiency.

* Intermediate Values: A value such as 0.3 creates a hybrid score, weighting occupancy at 70% and abundance at 30%.

Notably, incorporating abundance weighting will likely result in a different core set than 
an occupancy-only approach. 


# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_rarefaction_metrics.R
\name{add_rarefaction_metrics}
\alias{add_rarefaction_metrics}
\title{Calculate and append pre-rarefaction statistics to microbiome data}
\usage{
add_rarefaction_metrics(data)
}
\arguments{
\item{data}{A \code{phyloseq} object or a \code{data.frame} with samples
as rows and taxa as columns.}
}
\value{
The same object (\code{phyloseq} or \code{data.frame}) with new
columns:
\itemize{
\item \code{read_num}
\item \code{singleton_num}
\item \code{goods_cov}
\item \code{outlier}
}
}
\description{
This function adds read count, singleton count, Good's coverage,
and marks outlier samples to a \code{phyloseq} object or \code{data.frame}
based on the OTU/ASV abundance table.
}
\details{
About Good's coverage. Initially developed by Alan
Turing and I.J. Good during their cryptographic analyses in
World War II, it was later adopted by ecologists, particularly
in microbial diversity studies, to assess the completeness of
a sample's representation of the overall community.
It's calculated as \code{1 - (F1/N)}, where \code{F1} is the number of
OTUs (Operational Taxonomic Units) represented by only one
individual (singletons) and \code{N} is the total number of
individuals in the sample. For example, a Good's coverage of
0.95, means that 5\% of the reads in that sample are from OTUs
that appear only once.
}
\examples{
library(phyloseq)
library(BRCore)

# Adding metrics to a "phyloseq" object
bcse_metrics <- add_rarefaction_metrics(data = bcse)
sample_data(bcse_metrics)|>
head(10)


# Adding metrics to a "data.frame" count table object
bcse_otutable <- as.data.frame(
  as.matrix(otu_table(bcse))
)

bcse_otutable_metrics <- add_rarefaction_metrics(
  data = bcse_otutable
)
bcse_otutable_metrics[
  head(seq_len(nrow(bcse_otutable_metrics)), 10),
  tail(seq_len(ncol(bcse_otutable_metrics)), 20)
]


}
\seealso{
\code{\link[=plot_rarefaction_metrics]{plot_rarefaction_metrics()}} for visualizing these metrics, and
\code{\link[=multi_rarefy]{multi_rarefy()}} for performing rarefaction on a \code{phyloseq} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{bcse}
\alias{bcse}
\title{16S amplicon dataset from the GLBRC Biofuel Cropping System Experiment
(BCSE) at Michigan State University, Kellogg Biological Station}
\format{
An object class "phyloseq".
}
\source{
Internal test dataset.
}
\usage{
data("bcse")
}
\description{
A phyloseq containing 2861 97\% sequence similarity OTUs
(Operational Taxonomic Units) across 50 samples. Samples are for the leaf
microbiome only.
}
\references{
Haan, N. L., Benucci, G. N. M., Fiser, C. M., Bonito, G., &
Landis, D. A. (2023). Contrasting effects of bioenergy crops on
biodiversity. Science Advances, 9(38), eadh7960.
\url{doi:10.1126/sciadv.adh7960}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_abundance_occupancy.R
\name{plot_abundance_occupancy}
\alias{plot_abundance_occupancy}
\title{Plot Abundance-Occupancy Curve and Display the Core Taxa}
\usage{
plot_abundance_occupancy(core_result, core_set = "elbow")
}
\arguments{
\item{core_result}{A list object returned by \code{\link{identify_core}},
containing at minimum:
\itemize{
\item \code{occupancy_abundance}: A data frame with columns \code{otu},
\code{otu_rel} (mean relative abundance), and \code{otu_occ} (occupancy).
\item \code{elbow_core}: Character vector of OTU IDs identified as core
using the "elbow" method.
\item \code{increase_core}: Character vector of OTU IDs identified as core
using the "increase" method.
}}

\item{core_set}{Character string specifying which core set to highlight.
Must be either "elbow" or "increase" (Default elbow).}
}
\value{
A ggplot object showing the abundance-occupancy plot with core taxa
highlighted in red and non-core taxa in grey. The x-axis shows log10-transformed
mean abundance and the y-axis shows occupancy (0-1).
}
\description{
Creates a scatter plot showing the relationship between mean relative abundance
and occupancy (occurrence frequency) of taxa, with core taxa highlighted.
}
\details{
The function creates a scatter plot where each point represents a
taxon (i.e. ASV or OTU). Core taxa (as defined by the selected method) are
shown in red, while non-core taxa are shown in grey. The plot uses a log10
scale for abundance to better visualize the full range of abundances typically
found in microbiome data.
}
\examples{
library(BRCore)

data("switchgrass_core", package = "BRCore")

p <- plot_abundance_occupancy(
  core_result = switchgrass_core,
  core_set = "increase"
)
print(p)

}
\seealso{
\code{\link[=plot_core_distribution]{plot_core_distribution()}} and \code{\link[=identify_core]{identify_core()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-internal_warn_sncm.R
\name{.internal_warn_sncm}
\alias{.internal_warn_sncm}
\title{Internal warning handler for sncm.fit likelihood functions}
\usage{
.internal_warn_sncm(fun, message = NULL, warned = FALSE, warn_setter = NULL)
}
\arguments{
\item{fun}{A zero-argument function (lambda) to evaluate.}

\item{message}{A string prefix for the warning message.}

\item{warned}{Logical flag indicating whether a warning has already fired.}

\item{warn_setter}{A zero-argument function that sets the external \code{warned}
flag to \code{TRUE} via \verb{<<-}.}
}
\value{
The result of \code{fun()}.
}
\description{
Wraps a function call with a \code{withCallingHandlers()} that captures warnings
and re-emits them via \code{cli::cli_warn()}. Deduplicates repeated warnings
across optimizer iterations using an external flag and setter.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_core_distribution.R
\name{plot_core_distribution}
\alias{plot_core_distribution}
\title{Plot Core Taxa  Occupancy Across Metadata Groups}
\usage{
plot_core_distribution(
  core_result,
  core_set = "elbow",
  group_var = "Crop",
  plot_type = c("bar", "line", "heatmap")
)
}
\arguments{
\item{core_result}{A list object returned by \code{\link{identify_core}},
containing at minimum:
\itemize{
\item \code{otu_table}: A data frame with ASV/OTUs as rows and samples
as columns.
\item \code{metadata}: A data frame with samples as rows and grouping
variables as columns.
\item \code{otu_ranked}: A data frame with ranked taxa containing:
\itemize{
\item \code{otu}: A column with taxa names.
\item \code{rank}: A column with the rank for each taxon.
}
\item \code{elbow_core}: Character vector of OTU IDs identified as core
using the elbow method.
\item \code{increase_core}: Character vector of OTU IDs identified as core
using the increase method.
}}

\item{core_set}{Which core set to plot: "elbow" (default) or "increase".}

\item{group_var}{Metadata column for bar coloring. Default: "sampling_date".}

\item{plot_type}{Allows selection of 3 different plot types: \code{bar}, \code{line}, or \code{heatmap}.}
}
\value{
A ggplot2 object that can be further customized.
}
\description{
Creates a plot showing core taxa (i.e. OTUs/ASVs) occupancy patterns across
a grouping variable.
}
\examples{
library(BRCore)
data("switchgrass_core", package = "BRCore")

p <- plot_core_distribution(
  core_result = switchgrass_core,
  core_set = "increase",
  group_var = "sampling_date",
  plot_type = "bar"
)
print(p)

}
\seealso{
\code{\link[=identify_core]{identify_core()}}, \code{\link[=plot_abundance_occupancy]{plot_abundance_occupancy()}}, and
\code{\link[=plot_identified_core]{plot_identified_core()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/identify_core.R
\name{identify_core}
\alias{identify_core}
\title{Identify Core Microbiome Using Bray-Curtis Similarity
biological samples. Core taxa are selected using either a "last \% increase"
or "elbow" method implementing the method developed by Shade
and Stopnisek (2019) Curr Opin Microbiol, see below for details.}
\usage{
identify_core(
  physeq_obj,
  rarefied_list = NULL,
  priority_var,
  increase_value = 0.02,
  abundance_weight = 0,
  max_otus = NULL,
  depth_level = 1000,
  seed = NULL
)
}
\arguments{
\item{physeq_obj}{A \code{phyloseq} object with at least \code{otu_table} and
\code{sample_data}.
If the \code{phyloseq} object is already rarefied (i.e., all samples have the
same sequencing depth), you can omit \code{rarefied_list} and the function will
automatically use the OTU table from \code{physeq_obj} as a single iteration. If
the data is not rarefied, you must provide a \code{rarefied_list} generated by
\code{multi_rarefy}. Alternatively you can use \code{update_otu_table} to replace the
OTU table with a rarefied version before running \code{identify_core}.}

\item{rarefied_list}{A list of data frames, each representing a rarefied OTU
table (taxa x samples) generated by \code{multi_rarefy}. Required if \code{physeq_obj}
is not already rarefied. If \code{physeq_obj} is rarefied, this can be omitted
and the function will use the OTU table from \code{physeq_obj} as a single
iteration.}

\item{priority_var}{The column name in the \code{sample_data} (e.g.
sampling_date", "site") that is used for prioritizing the core microbiome.}

\item{increase_value}{Increase value (numeric, scalar) used in the
calculation (default 0.02) for "increase". The "elbow" is always calculated
and returned as \code{elbow_core} (see below for details).}

\item{abundance_weight}{Numeric in \verb{[0,1]}; how much to weight mean relative
abundance in the ranking score. \code{0} (default) uses occupancy/composite only.
\code{1} ranks purely by abundance. Values in between blend the two (e.g.,
abundance_weight = 0.3 gives 70\% occupancy/composite + 30\% abundance).}

\item{max_otus}{Optional integer to limit analysis to the top N ranked OTUs.
If NULL (default), all OTUs are analyzed. Useful for large datasets
(>5000 OTUs)}

\item{depth_level}{Integer. The sequencing depth used for normalization in
Bray-Curtis calculations. If data is rarefied, this is automatically set
to the rarefaction depth. For unrarefied data, samples with depth below
this threshold are excluded from pairwise comparisons.}

\item{seed}{Optional integer to set the RNG seed for reproducibility.}
}
\value{
A list with:
\itemize{
\item \code{bray_curtis_ranked} tibble with \code{rank}, mean Bray-Curtis
similarity across sample pairs (\code{MeanBC}) at each cumulative \code{rank},
normalized proportion (\code{proportionBC}), the multiplicative \code{IncreaseBC},
and the elbow metric (\code{elbow_slope_diffs}).
(\code{proportionBC}), the multiplicative \code{IncreaseBC}, and the elbow metric
(\code{elbow_slope_diffs}).
\item \code{otu_ranked} tibble with ranked OTU/ASVs .
\item \code{abundance_occupancy} tibble with OTU/ASVs names, occupancy
(\code{otu_occ}), and mean relative abundance (\code{otu_rel}).
\item \code{priority_var} character, the variable used for prioritizing
the core.
\item \code{elbow} core set identified by elbow method (integer).
\item \code{bc_increase} core set identified by last \% BC-increase
(integer).
\item \code{increase_value} increase value (numeric, scalar) used in the
calculation (e.g. 0.02).
\item \code{elbow_core} core OTU/ASVs using elbow method (character
vector).
\item \code{increase_core} core OTU/ASVs using last \% BC-increase method
(character vector).
\item \code{otu_table} otu_table counts (otu x samples) used (data.frame).
\item \code{sample_metadata} samples metadata (data.frame).
\item \code{taxonomy_table} taxonomy if present (data.frame); otherwise
NULL.
}
}
\description{
Identify Core Microbiome Using Bray-Curtis Similarity
biological samples. Core taxa are selected using either a "last \% increase"
or "elbow" method implementing the method developed by Shade
and Stopnisek (2019) Curr Opin Microbiol, see below for details.
}
\details{
The core set is defined using two separate methods:

The function rank OTU/ASVs by occupancy (optionally with abundance
weighting: \code{rank_score = (1 - weight) * rank + weight * scaled_abundance},
where scaled_abundance is mean relative abundance rescaled to \verb{[0,1]}). For
each \verb{k = 1..K}, recompute \code{S_k} as the mean Bray-Curtis similarity across
all sample pairs using only the first \code{k} ranked OTUs; when \code{k = K}, this
yields \code{S_K}, the value computed with all OTUs. Normalizing by \code{S_K}
gives \code{C_k = S_k / S_K}.

The \strong{elbow} is the point of diminishing returns: for each \code{k}, compare the
average \emph{left} slope \code{(S_k - S_1) / (k - 1)} to the average \emph{right} slope
\code{(S_K - S_k) / (K - k)}, and choose the \code{k} that maximizes \code{(left - right)}.

The \strong{last percent Bray-Curtis increase} method uses the same accumulation
curve, examine the multiplicative step when adding the \code{k-th} OTU:
\verb{Increase_k = S_k / S_\{k-1\}} (equivalently, \verb{Increase_k = C_k / C_\{k-1\}}).
Choose the largest \code{k} such that \code{Increase_k >= 1 + p}, where \code{p} is your
chosen percent threshold (increase_value; recommended \code{p >= 0.02} or \verb{2\%}).
This selects the final rank for which adding one more taxon still increases
the explained Bray-Curtis similarity by at least \code{p}.
}
\section{Dependencies}{

Requires \pkg{phyloseq}, \pkg{dplyr}, \pkg{tidyr}, \pkg{tibble}, \pkg{rlang},
and \pkg{vegan}.
}

\examples{
\donttest{
library(BRCore)

# With rarefied data
res <- identify_core(
  physeq_obj     = switchgrass,
  priority_var   = "sampling_date",
  increase_value = 0.02,
  seed           = 091825
)

str(res)

# With unrarefied data (requires multi_rarefy step)
rarefied_list <- multi_rarefy(
  physeq_obj = bcse,
  depth_level = 1000,
  num_iter = 3,
  .as = "list",
  set_seed = 7642
)


res_rare <- identify_core(
  physeq_obj = bcse,
  rarefied_list = rarefied_list,
  priority_var = "Crop",
  increase_value = 0.02,
  seed = 091825
)

str(res_rare)
}

}
\references{
Shade A, Stopnisek N (2019) Abundance-occupancy
distributions to prioritize plant core microbiome membership. Current
Opinion in Microbiology, 49:50-58
\url{doi:10.1016/j.mib.2019.09.008}
}
\seealso{
\code{\link[=multi_rarefy]{multi_rarefy()}}, \code{\link[=plot_identified_core]{plot_identified_core()}}, and
\code{\link[=plot_core_distribution]{plot_core_distribution()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_identified_core.R
\name{plot_identified_core}
\alias{plot_identified_core}
\title{Plot Bray-Curtis increase over ranked OTU/ASVs}
\usage{
plot_identified_core(
  bray_curtis_ranked,
  elbow,
  lastCall,
  increase_value = 0.02,
  dataset_name = NULL
)
}
\arguments{
\item{bray_curtis_ranked}{A tibble as returned by \code{identify_core()$bray_curtis_ranked}.}

\item{elbow}{The number of OTU/ASVs identified by the elbow method (Integer).}

\item{lastCall}{The number of OTU/ASVs identified by the last percent
Bray-Curtis increase method (Integer).}

\item{increase_value}{The percent increase value in decimal (e.g. 0.02) used
for the Bray-Curtis increase method.}

\item{dataset_name}{Optional character string. When provided, it is
prepended to the plot title (e.g. \code{"Switchgrass"}).
Default \code{NULL} (no prefix).}
}
\value{
A list containing: 1) \code{df_for_plot}, a data frame used for plotting, and 2) \code{plot_identified_core}, a ggplot object visualizing the Bray-Curtis increase with annotated cutoff points.
}
\description{
Visualize the cumulative normalized mean Bray-Curtis increase returned by
\link[=identify_core]{identify_core()}, over ranked OTU/ASVs and shows cutoff
points for elbow percent increase methods.
}
\details{
The function converts \code{rank} to integers and zooms the x-axis to the first
\code{1.2 * lastCall} ranks. Label positions are computed dynamically from the
observed \code{proportionBC} range to avoid overlap.
}
\examples{
library(BRCore)
data("switchgrass_core", package = "BRCore")

p <- plot_identified_core(
  bray_curtis_ranked = switchgrass_core$bray_curtis_ranked,
  elbow = switchgrass_core$elbow,
  lastCall = switchgrass_core$bc_increase,
  increase_value = switchgrass_core$increase_value
)
print(p)

}
\seealso{
\link[=identify_core]{identify_core()}

\code{\link{identify_core}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_neutral_model.R
\name{fit_neutral_model}
\alias{fit_neutral_model}
\title{Fit a Neutral Model to Microbial Community Data}
\usage{
fit_neutral_model(otu_table, core_set, abundance_occupancy)
}
\arguments{
\item{otu_table}{A community count matrix (or OTU table) with samples as
rows and OTU/ASVs as columns..}

\item{core_set}{Character vector of core OTU/ASVs IDs (must match column
names of \code{otu_table}).}

\item{abundance_occupancy}{data.frame (or tibble) with OTU/ASVs names,
occupancy (\code{otu_occ}),
and mean relative abundance (\code{otu_rel}) as generated by
\link[=identify_core]{identify_core()}.}
}
\value{
A list with:
\itemize{
\item \code{goodness_of_fit}: one-row tibble of summary stats (plus \code{above.pred},
\code{below.pred}) - \code{model_prediction}: tibble with \code{abundance_occupancy}
joined to per-taxon predictions
}
}
\description{
The function fits the neutral distribution model developed by Sloan et al.
(2006), and implemented in R by Burns et al. (2015), to an OTU/ASV table and
returns several goodness of fit statistics alongside a data.frame
with predicted occurrence frequencies for each OTU/ASV based on their
abundance in the metacommunity for plotting the abundance-occupancy
distribution. In addition, this function identified core and not-core
OTU/ASVs that are neutral (i.e. stochastically or randomly distributed)
above (i.e. interpreted as deterministically or predictably selected) and
below (i.e. interpreted as dispersally limited) the model predictions.
}
\examples{
library(BRCore)
data("switchgrass_core", package = "BRCore")

switchgrass_core_fit <- fit_neutral_model(
  otu_table = switchgrass_core$otu_table,
  core_set = switchgrass_core$increase_core,
  abundance_occupancy = switchgrass_core$abundance_occupancy
)

str(switchgrass_core_fit)
switchgrass_core_fit$goodness_of_fit
switchgrass_core_fit$model_prediction |>
head()

}
\references{
Sloan, W. T., Lunn, M., Woodcock, S., Head, I. M., Nee, S., & Curtis, T. P.
(2006). Quantifying the roles of immigration and chance in shaping
prokaryote community structure. Environmental Microbiology, 8(4), 732–740.
\url{doi:10.1111/j.1462-2920.2005.00956.x}

Burns, A. R., Stephens, W. Z., Stagaman, K., Wong, S., Rawls, J. F.,
Guillemin, K., & Bohannan, B. J. M. (2016). Contribution of neutral
processes to the assembly of gut microbial communities in the zebrafish over
host development. The ISME Journal, 10(3), 655–664.
\url{doi:10.1038/ismej.2015.142}

Shade A, Stopnisek N (2019) Abundance-occupancy distributions to prioritize
plant core microbiome membership. Current Opinion in Microbiology, 49:50-58
\url{doi:10.1016/j.mib.2019.09.008}
}
\seealso{
\code{\link[=plot_neutral_model]{plot_neutral_model()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multi_rarefy.R
\name{multi_rarefy}
\alias{multi_rarefy}
\title{Run rarefaction for microbiome count tables}
\usage{
multi_rarefy(
  physeq_obj,
  depth_level,
  num_iter = 100,
  .as = "list",
  set_seed = NULL
)
}
\arguments{
\item{physeq_obj}{A \code{phyloseq} object containing an OTU/ASV table.}

\item{depth_level}{An integer specifying the sequencing depth (number of
OTUs/ASVs) to which samples should be rarefied.}

\item{num_iter}{An integer specifying the number of iterations to perform
for rarefaction.}

\item{.as}{A character string indicating whether to return the results as a
3D array or as a list of data frames. If \code{"array"}, returns a 3D array with
dimensions (samples x taxa x iterations). If \code{"list"}, returns a list of
data frames, one for each iteration, with samples as rows and taxa as
columns. (default = "list")}

\item{set_seed}{An optional integer to set the random seed for
reproducibility (default = NULL).}
}
\value{
A data frame with taxa as rows and samples as columns. The values
represent the average sequence counts calculated across all iterations.
Samples with less than \code{depth_level} sequences are discarded.
}
\description{
This function performs rarefaction on a \code{phyloseq} object by randomly
sub-sampling OTUs/ASVs within samples without replacement for a number of
iterations specified by the user. Samples with fewer OTUs/ASVs than the
specified \code{depth_level} are discarded.
}
\examples{
library(BRCore)


# Example rarefaction (single iteration, single core to keep examples fast)
otu_table_rare <- multi_rarefy(
  physeq_obj = bcse,
  depth_level = 1000,
  num_iter = 10,
  .as = "list",
  set_seed = 7642
)

rowSums(otu_table_rare[[1]])


}
\seealso{
\code{\link[=update_otu_table]{update_otu_table()}} for updating the OTU table in a \code{phyloseq}
object and \code{\link[vegan:rarefy]{vegan::rrarefy()}} for the underlying rarefaction method used in
this function.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sncm.fit.R
\name{sncm.fit}
\alias{sncm.fit}
\title{Fit Sloan Neutral Community Model (SNCM)}
\usage{
sncm.fit(spp, pool = NULL, stats = TRUE, taxon = NULL)
}
\arguments{
\item{spp}{A matrix or data frame where rows represent communities (samples)
and columns represent species/taxa. Values should be abundances or counts.}

\item{pool}{Optional. A matrix or data frame representing the source pool for
calculating relative abundances. If NULL, the source pool is calculated
from the spp matrix. Default is \code{NULL}.}

\item{stats}{Logical. If \code{TRUE}, returns fit statistics including AIC, BIC,
R-squared, and RMSE for model comparison. If \code{FALSE}, returns predicted vs.
observed frequencies with confidence intervals. Default is \code{TRUE}.}

\item{taxon}{Optional. A data frame containing taxonomic information to
merge with results when \code{stats = FALSE}. Should have row names matching species
names. Default is \code{NULL}.}
}
\value{
If \code{stats = TRUE}, returns a data frame with model fit statistics including:
\itemize{
\item \code{m}: Migration rate parameter from NLS fit
\item \code{m.ci}: Confidence interval for m parameter
\item \code{m.mle}: Migration rate from maximum likelihood estimation
\item \code{maxLL}, \code{binoLL}, \code{poisLL}: Log-likelihood values for SNCM, binomial, and Poisson models
\item \code{Rsqr}, \code{Rsqr.bino}, \code{Rsqr.pois}: R-squared values for each model
\item \code{RMSE}, \code{RMSE.bino}, \code{RMSE.pois}: Root mean squared error for each model
\item \code{AIC}, \code{BIC}: Information criteria for model selection
\item \code{N}: Average number of individuals per community
\item \code{Samples}: Number of samples/communities
\item \code{Richness}: Number of taxa analyzed
\item \code{Detect}: Detection limit (1/N)
}
If \code{stats = FALSE}, returns a data frame with observed and predicted
frequencies along with confidence intervals for visualization.
}
\description{
The function implements the Sloan Neutral Community Model which
predicts the occurrence frequency of taxa based on their relative abundance
in the source pool and a migration parameter (\code{m}).  It compares the
neutral model against binomial and Poisson null models using various
statistical measures.
}
\details{
The model assumes neutral processes govern community assembly. Three models are compared: SNCM (beta distribution), binomial null model, and Poisson null model.
}
\examples{
# Generate community data
set.seed(42)
n_samples <- 100
n_species <- 80

# Log-normal abundances
mean_abund <- rlnorm(n_species, meanlog = 2, sdlog = 1.5)
# Simulate community matrix with multinomial sampling
spp_data <- t(sapply(seq_len(n_samples), function(i) {
  rmultinom(
    1,
    size = sample(500:2000, 1),
    prob = mean_abund / sum(mean_abund)
  )[, 1]
}))
colnames(spp_data) <- paste0("Species_", seq_len(n_species))

fit_stats <- sncm.fit(spp_data, stats = TRUE)
predictions <- sncm.fit(spp_data, stats = FALSE)


}
\references{
Sloan WT, Lunn M, Woodcock S, Head IM, Nee S, Curtis TP. (2006) Quantifying
the roles of immigration and chance in shaping prokaryote community
structure. Environ Microbiol. 8(4):732-40.
\url{doi:10.1111/j.1462-2920.2005.00956.x}
}
\seealso{
\code{\link[=plot_neutral_model]{plot_neutral_model()}}, \code{\link[=fit_neutral_model]{fit_neutral_model()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{switchgrass_core}
\alias{switchgrass_core}
\title{Identified core microbiome members for the switchgrass dataset}
\format{
An object class "phyloseq".
}
\source{
Internal test dataset.
}
\usage{
data("switchgrass_core")
}
\description{
A phyloseq containing the identified core microbiome members
for the switchgrass dataset. This object is used for BRCore function
examples.
It was generated by running the following on BRCore 2.0.2 (2026-04-30):

\if{html}{\out{<div class="sourceCode R">}}\preformatted{switchgrass_core <- identify_core(
  physeq_obj     = switchgrass,
  priority_var   = "sampling_date",
  increase_value = 0.02,
  seed           = 092825
)
}\if{html}{\out{</div>}}
}
\references{
Shade A, Stopnisek N (2019) Abundance-occupancy distributions to
prioritize plant core microbiome membership. Current Opinion in
Microbiology, 49:50-58 \url{doi:10.1016/j.mib.2019.09.008}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_variance_propagation.R
\name{plot_variance_propagation}
\alias{plot_variance_propagation}
\title{Variance propagation diagnostic for rarefaction}
\usage{
plot_variance_propagation(
  physeq_obj,
  rarefied,
  q = 0,
  group_var,
  group_color,
  convert_to_factor = FALSE
)
}
\arguments{
\item{physeq_obj}{Raw phyloseq object}

\item{rarefied}{Output from multi_rarefy(). Either a list of dataframes or and array.}

\item{q}{Hill number order (q = 0 for richness, q = 1 for Shannon, q = 2 for Simpson)}

\item{group_var}{A grouping variable to use gor grouping as in the sample_data()}

\item{group_color}{A color variable to use present in the sample_data()}

\item{convert_to_factor}{Logical. If \code{TRUE}, both \code{group_var} and
\code{group_color} are coerced to \code{factor} before plotting, which is
useful when those columns are numeric/continuous (e.g. dates, counts) but
should be treated as discrete groups. When \code{TRUE} a discrete color
scale (\code{scale_color_viridis_d}) is used; otherwise the continuous
scale (\code{scale_color_viridis_c}) is used. Default \code{FALSE}.}
}
\value{
ggplot object comparing raw vs rarefied diversity distributions across iterations.
}
\description{
This function evaluate the variance between rarefaction iterations from \code{multi_rarefy()} by visually comparing raw vs. rarefied alpha diversity metrics calculated at each iterations. It is possible to plot observed richness (q=0), Shannon diversity (q=1), or Simpson diversity (q=2) by setting the \code{q} parameter to "richness" or \code{q} = 0, "shannon" or \code{q} = 1, or "shannon" or \code{q} = 2. The plot is faceted by method (raw vs rarefied) and colored by a specified grouping variable from the sample data.
}
\examples{
library(phyloseq)
library(BRCore)
# Example comparing hill q=1 between Poplar and Switchgrass plots
bcse_filt <- bcse |>
subset_samples(Crop \%in\% c("Poplar", "Switchgrass"))

bcse_rarefied_otutable_filt <-
 multi_rarefy(
       physeq_obj = bcse_filt,
       depth_level = 1000,
       num_iter = 10,
       .as = "list",
       set_seed = 7643
   )

plot_variance_propagation(
   physeq_obj   = bcse_filt,
   rarefied = bcse_rarefied_otutable_filt,
   q        = 1,
   group_var = "Crop",
   group_color = "Plot"
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_rarefaction_metrics.R
\name{plot_rarefaction_metrics}
\alias{plot_rarefaction_metrics}
\title{Plot pre-rarefaction diagnostics}
\usage{
plot_rarefaction_metrics(data)
}
\arguments{
\item{data}{Either a \code{phyloseq} object or a \code{data.frame} that includes columns
\code{read_num}, \code{goods_cov}, and \code{outlier}.}
}
\value{
A \code{ggarrange} object with six plots.
}
\description{
This function creates a 6-panel diagnostic plot showing sequencing depth,
Good's coverage, and outlier behavior based on a data frame or a
\code{phyloseq} object with rare stats already added via \code{add_rare_stats()}.
}
\examples{
library(phyloseq)
library(BRCore)

# Add rarefaction metrics to the phyloseq object
bcse_metrics <- add_rarefaction_metrics(bcse)

# Plot the rarefaction diagnostics
plot_rarefaction_metrics(bcse_metrics)

# You can also pass a data frame directly if you have
# pre-computed read_num, goods_cov, and outlier columns
sample_data_df <- data.frame(sample_data(bcse_metrics))
plot_rarefaction_metrics(sample_data_df)

}
\seealso{
\code{\link[=add_rarefaction_metrics]{add_rarefaction_metrics()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{switchgrass}
\alias{switchgrass}
\title{16S amplicon dataset from Switchgrass (Panicum virgatum)}
\format{
An object class "phyloseq".
}
\source{
Internal test dataset.
}
\usage{
data("switchgrass")
}
\description{
A phyloseq containing 706 97\% sequence similarity OTUs
(Operational Taxonomic Units) across 43 samples.
}
\references{
Shade A, Stopnisek N (2019) Abundance-occupancy distributions to
prioritize plant core microbiome membership. Current Opinion in
Microbiology, 49:50-58 \url{doi:10.1016/j.mib.2019.09.008}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_neutral_model.R
\name{plot_neutral_model}
\alias{plot_neutral_model}
\title{Plot a fitted Neutral Model to Microbial Community Data}
\usage{
plot_neutral_model(fit_result)
}
\arguments{
\item{fit_result}{A list-like object returned by \link[=fit_neutral_model]{fit_neutral_model()}.}
}
\value{
A \link[ggplot2:ggplot]{ggplot2} object.

A \link[ggplot2:ggplot]{ggplot} object with:
\itemize{
\item \emph{x}-axis: \code{log10(mean abundance)} (\code{log10(otu_rel)});
\item \emph{y}-axis: \code{Occupancy} (\code{otu_occ});
\item Points colored by membership/fit class as described above;
\item Neutral-model curve (solid) with 95\\% CI (grey, twodash);
\item An inset white label reporting \emph{R}\eqn{^2} and \emph{m} taken
directly from \code{fit_result$goodness_of_fit}.
}

The function does not recompute statistics; it only visualizes the supplied
predictions and metrics.
}
\description{
This function plots ASV/OTUs log abundances into a fitted neutral model of
microbial abundance-occupancy distribution. ASV/OTUs that are \emph{Core}, \emph{As
predicted} (i.e. Neutral), \emph{Below} and \emph{Above} model predictions are drawn
with distinct point colors, see details.
}
\details{
Points are split into four groups for display:
\itemize{
\item \strong{Not core (as predicted)} – background points;
\item \strong{Core (as predicted)} – core taxa whose occupancy matches the model;
\item \strong{Core (above prediction)} – core taxa above the 95\\% CI;
\item \strong{Core (below prediction)} – core taxa below the 95\\% CI.
}
}
\examples{
library(BRCore)
data("switchgrass_core", package = "BRCore")

switchgrass_core_fit <- fit_neutral_model(
  otu_table = switchgrass_core$otu_table,
  core_set = switchgrass_core$increase_core,
  abundance_occupancy = switchgrass_core$abundance_occupancy
)

p <- plot_neutral_model(switchgrass_core_fit)
print(p)

}
\seealso{
\code{\link{fit_neutral_model}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update_otu_table.R
\name{update_otu_table}
\alias{update_otu_table}
\title{Add a rarefied otu_table to a phyloseq object}
\usage{
update_otu_table(physeq_obj, rarefied_otus, iteration = NULL)
}
\arguments{
\item{physeq_obj}{A \code{phyloseq} object in which you want to add the rarefied
OTU/ASV table.}

\item{rarefied_otus}{A data frame, list of data frames, or 3D array output from
\code{multi_rarefy()} containing the rarefied OTU/ASV tables.}

\item{iteration}{Integer specifying which iteration to extract from a list
or array. Required when \code{rarefied_otus} is a list or array. Ignored when
\code{rarefied_otus} is a data frame.}
}
\value{
A \code{phyloseq} object.
}
\description{
This function updates a \code{phyloseq} object by replacing its OTU/ASV table
with a rarefied version produced by \code{multi_rarefy()}. The rarefied table can
be a data frame, a list of data frames (\code{.as = "list"}), or a 3D array
(\code{.as = "array"}). When providing a list or array, specify which iteration
to use via the \code{iteration} parameter.
}
\examples{
library(phyloseq)
library(BRCore)
data(GlobalPatterns, package = "phyloseq")

# List output
otu_list <-
  multi_rarefy(
    physeq_obj = GlobalPatterns,
    depth_level = 200,
    num_iter = 3,
    .as = "list",
    set_seed = 123
  )

# Extract iteration 2
rarefied_gp <- update_otu_table(GlobalPatterns, otu_list, iteration = 2)

# Array output
otu_array <-
  multi_rarefy(
    physeq_obj = GlobalPatterns,
    depth_level = 200,
    num_iter = 3,
    .as = "array",
    set_seed = 123
  )

# Extract iteration 1
rarefied_gp2 <- update_otu_table(GlobalPatterns, otu_array, iteration = 1)

}
\seealso{
\code{\link[=multi_rarefy]{multi_rarefy()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{bean}
\alias{bean}
\title{16S amplicon dataset from common bean (Phaseolus vulgaris)}
\format{
An object class "phyloseq".
}
\source{
Internal test dataset.
}
\usage{
data("bean")
}
\description{
A phyloseq containing 24885 ASVs (Amplicon Sequence Variants)
across 30 samples.
}
\references{
Shade A, Stopnisek N (2019) Abundance-occupancy distributions to
prioritize plant core microbiome membership. Current Opinion in
Microbiology, 49:50-58 \url{doi:10.1016/j.mib.2019.09.008}
}
\keyword{datasets}
